@@ -15,22 +15,20 @@
 #include <stdio.h>
 #include <math.h>
 #include <ctype.h>
-#if defined(__STDC__)
 #include <stdlib.h>
-#endif
+
 #include <X11/Xlib.h>
 #include <Xm/Xm.h>
 
-#include "P_.h"
+#include "xephem.h"
 
-extern double atod P_((char *buf));
 
-static int next_token P_((void));
-static int chk_funcs P_((void));
-static void skip_double P_((void));
-static int compile P_((int prec));
-static int execute P_((double *result));
-static int parse_fieldname P_((char name[], int len));
+static int next_token (void);
+static int chk_funcs (void);
+static void skip_double (void);
+static int compile (int prec);
+static int execute (double *result);
+static int parse_fieldname (char name[], int len);
 
 /* parser tokens and opcodes, as necessary */
 #define	HALT	0	/* good value for HALT since program is inited to 0 */
@@ -214,7 +212,7 @@ next_token ()
 	int tok = ERR;	/* just something illegal */
 	char c;
 
-	while ((c = *cexpr) == ' ')
+	while (isspace(c = *cexpr))
 	    cexpr++;
 	lcexpr = cexpr++;
 
@@ -561,7 +559,7 @@ double *result;
 	/* result should now be on top of stack */
 	if (sp != &stack[MAX_STACK - 1]) {
 	    (void) sprintf (err_msg, "Bug! stack has %d items",
-							MAX_STACK - (sp-stack));
+							(int)(MAX_STACK - (sp-stack)));
 	    return (-1);
 	}
 	*result = *sp;
@@ -590,4 +588,4 @@ int len;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: compiler.c,v $ $Date: 1998/07/12 02:33:52 $ $Revision: 1.2 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: compiler.c,v $ $Date: 2009/01/05 20:55:54 $ $Revision: 1.5 $ $Name:  $"};
