@@ -440,33 +440,54 @@ XtPointer client;
 XtPointer call;
 {
 	Now *np = mm_get_now();
-	double ra, dec, alt, az;
+	double fov, ra, dec, alt, az;
+	int aamode;
 	char *p;
 
-	p = XmTextFieldGetString(ccra_w);
-	f_scansexa (p, &ra);
-	XtFree(p);
-	ra = hrrad(ra);
+	/* only if skyview up */
+	if (!svshell_w) {
+	    xe_msg (1, "Can not place eyepiece until Sky View window is up");
+	    return;
+	}
 
-	p = XmTextFieldGetString(ccdec_w);
-	f_scansexa (p, &dec);
-	XtFree(p);
-	dec = degrad(dec);
+	/* get sky mode */
+	sv_getcenter (&aamode, &fov, &alt, &az, &ra, &dec);
 
-	p = XmTextFieldGetString(ccalt_w);
-	f_scansexa (p, &alt);
-	XtFree(p);
-	alt = degrad(alt);
+	/* define new eyepiece */
+	if (aamode) {
 
-	p = XmTextFieldGetString(ccaz_w);
-	f_scansexa (p, &az);
-	XtFree(p);
-	az = degrad(az);
+	    p = XmTextFieldGetString(ccalt_w);
+	    f_scansexa (p, &alt);
+	    XtFree(p);
+	    alt = degrad(alt);
 
-	if (epoch == EOD)
-	    ap_as (np, mjd, &ra, &dec);
+	    p = XmTextFieldGetString(ccaz_w);
+	    f_scansexa (p, &az);
+	    XtFree(p);
+	    az = degrad(az);
 
-	se_add (ra, dec, alt, az);
+	    se_add (aamode, az, alt);
+
+	} else {
+
+	    p = XmTextFieldGetString(ccra_w);
+	    f_scansexa (p, &ra);
+	    XtFree(p);
+	    ra = hrrad(ra);
+
+	    p = XmTextFieldGetString(ccdec_w);
+	    f_scansexa (p, &dec);
+	    XtFree(p);
+	    dec = degrad(dec);
+
+	    if (epoch == EOD)
+		ap_as (np, mjd, &ra, &dec);
+
+	    se_add (aamode, ra, dec);
+
+	}
+
+	/* display on skyview */
 	sv_all (np);
 }
 
@@ -605,6 +626,8 @@ Widget w;
 	    eclng = degrad(eclng);
 
 	    ecl_eq (e, eclat, eclng, &ra, &dec);
+	    if (epoch == EOD)
+		ap_as (np, e, &ra, &dec);
 
 	} else if (w == ccra_w || w == ccdec_w) {
 	    p = XmTextFieldGetString(ccra_w);
@@ -667,8 +690,11 @@ Widget w;
 	}
 	if (w != cceclat_w && w != cceclng_w) {
 	    double eclat, eclng;
+	    double era = ra, edec = dec;	/* want current epoch setting */
 
-	    eq_ecl (e, ra, dec, &eclat, &eclng);
+	    if (epoch == EOD)
+		as_ap (np, e, &era, &edec);
+	    eq_ecl (e, era, edec, &eclat, &eclng);
 	    canonFmt (buf, raddeg(eclat), 0);
 	    XmTextFieldSetString (cceclat_w, buf);
 	    canonFmt (buf, raddeg(eclng), 0);
@@ -706,4 +732,4 @@ Widget w;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: coordsmenu.c,v $ $Date: 2008/04/14 02:12:08 $ $Revision: 1.16 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: coordsmenu.c,v $ $Date: 2011/05/16 02:32:23 $ $Revision: 1.20 $ $Name:  $"};
