@@ -101,8 +101,8 @@ static ColHdr col[] = {
     {MISC_COL,	"GLat",	   "Galactic latitude"},
 
     {MISC_COL,	"GLong",   "Galactic longitude"},
-    {MISC_COL,	"EcLat",   "Geocentric Ecliptic latitude"},
-    {MISC_COL,	"EcLong",  "Geocentric Ecliptic longitude"},
+    {MISC_COL,	"EcLat",   "Ecliptic latitude"},
+    {MISC_COL,	"EcLong",  "Ecliptic longitude"},
     {MISC_COL,	"EaDst",   "Distance from Earth, AU (moon is km)"},
     {MISC_COL,	"EaLght",  "Light travel time from Earth"},
     {MISC_COL,	"SnDst",   "Distance from Sun, AU"},
@@ -936,8 +936,6 @@ Widget w;
 	case ECLONG_ID: {
 	    double eclat, eclng, asra = op->s_ra, asdec = op->s_dec;
 	    double e = epoch == EOD ? mjd : epoch;
-	    if (epoch == EOD)
-		ap_as (np, mjd, &asra, &asdec);
 	    eq_ecl (e, asra, asdec, &eclat, &eclng);
 	    f_pangle (w, eclng);
 	    }
@@ -945,8 +943,6 @@ Widget w;
 	case ECLAT_ID: {
 	    double eclat, eclng, asra = op->s_ra, asdec = op->s_dec;
 	    double e = epoch == EOD ? mjd : epoch;
-	    if (epoch == EOD)
-		ap_as (np, mjd, &asra, &asdec);
 	    eq_ecl (e, asra, asdec, &eclat, &eclng);
 	    f_pangle (w, eclat);
 	    }
@@ -1890,4 +1886,4 @@ char buf[];
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: datamenu.c,v $ $Date: 2009/01/05 20:55:54 $ $Revision: 1.58 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: datamenu.c,v $ $Date: 2010/06/14 02:30:16 $ $Revision: 1.59 $ $Name:  $"};
