@@ -454,15 +454,15 @@ String fallbacks[] = {
     "XEphem*SkyEyep*ApparentFOV.value: 45",
     "XEphem*SkyEyep*Elliptical.set: True",
     "XEphem*SkyEyep*EyepH.value: 50",
-    "XEphem*SkyEyep*EyepPA.value: 0",
+    "XEphem*SkyEyep*EyepA.value: 0",
     "XEphem*SkyEyep*EyepW.value: 50",
     "XEphem*SkyEyep*EyepieceFL.value: 12mm",
     "XEphem*SkyEyep*FPlaneSize.value: 35mm",
-    "XEphem*SkyEyep*DefFile.value:",
+    "XEphem*SkyEyep*EyePDefFile.value:",
     "XEphem*SkyEyep*FocalLength.value: 50mm",
     "XEphem*SkyEyep*Lock.set: False",
     "XEphem*SkyEyep*MirrorFL.value: 2000mm",
-    "XEphem*SkyEyep*PosFile.value:",
+    "XEphem*SkyEyep*EyePPosFile.value:",
     "XEphem*SkyEyep*Solid.set: False",
     "XEphem*SkyEyep*Telrad.set: False",
     "XEphem*SkyEyep.x: 200",
@@ -743,10 +743,10 @@ String fallbacks[] = {
     "XEphem*WebDB*URL0.value: http://celestrak.com/NORAD/elements/stations.txt",
     "XEphem*WebDB*URL1.value: http://celestrak.com/NORAD/elements/science.txt",
     "XEphem*WebDB*URL2.value: http://celestrak.com/NORAD/elements/tle-new.txt",
-    "XEphem*WebDB*URL3.value: http://www.cfa.harvard.edu/iau/Ephemerides/Comets/Soft03Cmt.txt",
-    "XEphem*WebDB*URL4.value: http://www.cfa.harvard.edu/iau/Ephemerides/CritList/Soft03CritList.txt",
-    "XEphem*WebDB*URL5.value: http://www.cfa.harvard.edu/iau/Ephemerides/Distant/Soft03Distant.txt",
-    "XEphem*WebDB*URL6.value: http://www.cfa.harvard.edu/iau/Ephemerides/Unusual/Soft03Unusual.txt",
+    "XEphem*WebDB*URL3.value: http://www.minorplanetcenter.org/iau/Ephemerides/Comets/Soft03Cmt.txt",
+    "XEphem*WebDB*URL4.value: http://www.minorplanetcenter.org/iau/Ephemerides/CritList/Soft03CritList.txt",
+    "XEphem*WebDB*URL5.value: http://www.minorplanetcenter.org/iau/Ephemerides/Distant/Soft03Distant.txt",
+    "XEphem*WebDB*URL6.value: http://www.minorplanetcenter.org/iau/Ephemerides/Unusual/Soft03Unusual.txt",
     "XEphem*WebDB*URL7.value:",
     "XEphem*WebDB.x: 200",
     "XEphem*WebDB.y: 200",
@@ -796,7 +796,7 @@ String fallbacks[] = {
     "XEphem*XmToggleButton.foreground: #fffafa",
     "XEphem*XmToggleButtonGadget.fontList: *-lucidatypewriter*medium*-10-*-iso8859-1",
     "XEphem*XmToggleButtonGadget.foreground: #fffafa",
-    "XEphem*background: #446155",
+    "XEphem*background: #3B4F47",
     "XEphem*blinkRate: 0",
     "XEphem*fontList: *-lucidatypewriter*medium*-10-*-iso8859-1",
     "XEphem*foreground: black",
@@ -824,6 +824,7 @@ String fallbacks[] = {
     "XEphem.EarthHereColor: #ffff60",
     "XEphem.EarthSiteColor: #ff9b00",
     "XEphem.EarthSunColor: #d3ce8c",
+    "XEphem.EarthGreatCircleColor: #cc0000",
     "XEphem.EarthTrailState: 1 1 1 3 2 0 20 0 0.0",
     "XEphem.EarthViewUp: 0",
     "XEphem.Elevation: 0",
@@ -957,4 +958,4 @@ String fallbacks[] = {
     NULL
 };
 
-/* $Id: fallbacks.c,v 1.303 2009/04/10 21:56:46 ecdowney Exp $ */
+/* $Id: fallbacks.c,v 1.307 2011/05/16 02:48:16 ecdowney Exp $ */
