@@ -51,6 +51,8 @@ static void saveFavs (char *filename);
 static void favRmAll(void);
 static void bldExport(void);
 static Obj *srchRealDB (char *name);
+static void del_i(void);
+static void chg_i(void);
 
 /* the list */
 static Favorite *favs;			/* malloced list of all favorites */
@@ -70,6 +72,9 @@ static char favcategory[] = "Favorites";                /* Save category */
 static char fav_suffix[] = ".fav";	/* suffix of Favorites files */
 static char deffavfn[] = "favorites.fav";	/* default favorites to load */
 
+/* used by query callbacks */
+static int deli;
+static Obj newo;
 
 /* show, create if first time, the favorites window */
 void
@@ -94,7 +99,7 @@ fav_add (Obj *op)
 	    fav_create();
 
 	/* check for dup */
-	if (fav_already (op->o_name)) {
+	if (fav_already (op->o_name) >= 0) {
 	    watch_cursor(0);
 	    return;
 	}
@@ -113,7 +118,7 @@ fav_add (Obj *op)
 	watch_cursor(0);
 }
 
-/* return 1 if given name is on favorites list, else 0 */
+/* return index into favs entry if given name is on favorites list, else -1 */
 int
 fav_already (char *name)
 {
@@ -121,8 +126,8 @@ fav_already (char *name)
 
 	for (i = 0; i < nfavs; i++)
 	    if (strcmp (favs[i].o.o_name, name) == 0)
-		return (1);
-	return (0);
+		return (i);
+	return (-1);
 }
 
 /* return a list of Favorite Obj* to be used.
@@ -276,7 +281,7 @@ fav_create()
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	pb_w = XmCreatePushButton (favform_w, "FATB", args, n);
 	set_xmstring (pb_w, XmNlabelString, "Add edb:");
-	wtip (pb_w, "Add a Favorite defined by edb format entry at right");
+	wtip (pb_w, "Add or change a Favorite defined by the edb format entry at right");
 	XtAddCallback (pb_w, XmNactivateCallback, fav_add_cb, NULL);
 	XtManageChild (pb_w);
 
@@ -290,7 +295,7 @@ fav_create()
 	add_w = XmCreateTextField (favform_w, "edb", args, n);
 	XtAddCallback (add_w, XmNactivateCallback, fav_add_cb, NULL);
 	sr_reg (add_w, NULL, favcategory, 0);
-	wtip (add_w, "edb format definition of new Favorite");
+	wtip (add_w, "edb format definition of new or changed Favorite");
 	XtManageChild (add_w);
 
 	/* favorites file save control */
@@ -534,7 +539,7 @@ XtPointer call;
 	XtFree (txt);
 }
 
-/* callback to Add a new favorite 
+/* callback to Add a new favorite or possibly modify an existing fav.
  * N.B. do not use call arg, this is used both by PB and TF
  */
 /* ARGSUSED */
@@ -544,19 +549,27 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	char whynot[1024];
-	Obj newo;
+	char buf[1024];
 	char *txt;
 
-	/* get edb dfn, crack and add if ok */
+	/* get edb dfn, crack and add/change if ok */
 	txt = XmTextFieldGetString (add_w);
-	if (db_crack_line (txt, &newo, NULL, 0, whynot) < 0) {
-	    xe_msg (1, whynot);
-	} else if (fav_already (newo.o_name)) {
-	    xe_msg (1, "%s is already a favorite", newo.o_name);
+	if (db_crack_line (txt, &newo, NULL, 0, buf) < 0) {
+	    /* unknown */
+	    xe_msg (1, buf);
 	} else if (srchRealDB (newo.o_name)) {
+	    /* already in db */
 	    xe_msg (1, "%s is already in loaded database", newo.o_name);
+	} else if ((deli = fav_already (newo.o_name)) >= 0) {
+	    /* already in favs list */
+	    if (confirm()) {
+		(void) sprintf (buf, "Change \"%s\"?", favs[deli].o.o_name);
+		query (favshell_w, buf, "Yes -- change", "No -- ignore", NULL,
+							    chg_i, NULL, NULL);
+	    } else
+		chg_i();
 	} else {
+	    /* add to favs */
 	    Favorite *newfp = favGrow();
 	    newfp->o = newo;
 	    newfp->op = NULL;
@@ -572,7 +585,6 @@ XtPointer call;
 }
 
 /* used to delete favorite entry deli */
-static int deli;
 static void
 del_i()
 {
@@ -587,6 +599,22 @@ del_i()
 	watch_cursor (0);
 }
 
+/* used to change favorite[deli] to newo */
+static void
+chg_i()
+{
+	/* update entry */
+	memcpy (&favs[deli].o, &newo, sizeof(Obj));
+	favs[deli].op = NULL;	/* local copy now for sure */
+
+	/* update display */
+	watch_cursor (1);
+	bldExport();
+	showFavorites();
+	all_newfavs();
+	watch_cursor (0);
+}
+
 /* called when a Use toggle changes.
  * client is index into favs[]
  */
@@ -825,5 +853,5 @@ srchRealDB (char *name)
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: favmenu.c,v $ $Date: 2009/01/05 20:55:54 $ $Revision: 1.25 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: favmenu.c,v $ $Date: 2010/06/14 02:30:31 $ $Revision: 1.26 $ $Name:  $"};
 
