@@ -3,11 +3,8 @@
 
 #include <stdio.h>
 #include <math.h>
-
-#if defined(__STDC__)
 #include <stdlib.h>
 #include <string.h>
-#endif
 
 #include <X11/Xlib.h>
 #include <Xm/Xm.h>
@@ -23,58 +20,28 @@
 #include <Xm/Separator.h>
 #include <Xm/ScrolledW.h>
 
-#include "P_.h"
-#include "astro.h"
-#include "circum.h"
-#include "dm.h"
-#include "ps.h"
+#include "xephem.h"
 
-
-extern Widget toplevel_w;
-extern XtAppContext xe_app;
-#define XtD XtDisplay(toplevel_w)
-extern Colormap xe_cm;
-
-extern Now *mm_get_now P_((void));
-extern Obj *db_basic P_((int id));
-extern Obj *db_scan P_((DBScan *sp));
-extern char *mm_getsite P_((void));
-extern int gray_ramp P_((Display *dsp, Colormap cm, Pixel **pix));
-extern int isUp P_((Widget shell));
-extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
-extern void db_update P_((Obj *op));
-extern void dm_riset P_((Now *np, Obj *op, RiseSet *rsp));
-extern void fs_time P_((char out[], double t));
-extern void get_something P_((Widget w, char *resource, XtArgVal value));
-extern void get_xmstring P_((Widget w, char *resource, char **txtp));
-extern void get_xmlabel_font P_((Widget w, XFontStruct **f));
-extern void get_views_font P_((Display *dsp, XFontStruct **fspp));
-extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
-extern void obj_pickgc P_((Obj *op, Widget w, GC *gcp));
-extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
-extern void set_something P_((Widget w, char *resource, XtArgVal value));
-extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
-extern void timestamp P_((Now *np, Widget w));
-extern void watch_cursor P_((int want));
-extern void wtip P_((Widget w, char *tip));
-extern void xe_msg P_((char *msg, int app_modal));
-
-static int ng_ison P_((void));
-static void ng_create_shell P_((void));
-static void ng_create_popup P_((void));
-static void ng_print_cb P_((Widget w, XtPointer client, XtPointer call));
-static void ng_da_input_cb P_((Widget w, XtPointer client, XtPointer call));
-static void ng_print P_((void));
-static void ng_ps_annotate P_((void));
-static void ng_popdown_cb P_((Widget w, XtPointer client, XtPointer call));
-static void ng_close_cb P_((Widget w, XtPointer client, XtPointer call));
-static void ng_init_gcs P_((void));
-static void ng_help_cb P_((Widget w, XtPointer client, XtPointer call));
-static void ng_exp_cb P_((Widget w, XtPointer client, XtPointer call));
-static void ng_redraw P_((void));
-static void ng_refresh P_((XExposeEvent *ep));
-static void ng_drawpm P_((void));
+static int ng_ison (void);
+static void ng_create_shell (void);
+static void ng_create_popup (void);
+static void ng_1col_cb (Widget w, XtPointer client, XtPointer call);
+static void ng_print_cb (Widget w, XtPointer client, XtPointer call);
+static void ng_da_input_cb (Widget w, XtPointer client, XtPointer call);
+static void ng_print (void);
+static void ng_ps_annotate (void);
+static void ng_settime_cb (Widget w, XtPointer client, XtPointer call);
+static void ng_popdown_cb (Widget w, XtPointer client, XtPointer call);
+static void ng_close_cb (Widget w, XtPointer client, XtPointer call);
+static void ng_mloop_cb (Widget w, XtPointer client, XtPointer call);
+static void ng_init_gcs (void);
+static void ng_help_cb (Widget w, XtPointer client, XtPointer call);
+static void ng_exp_cb (Widget w, XtPointer client, XtPointer call);
+static void ng_redraw (void);
+static void ng_refresh (XExposeEvent *ep);
+static void ng_drawpm (void);
+static int ng_ano (double *fracx, double *fracy, int *xp, int *yp, int w2x, 
+    int arg);
 
 /* main's widgets */
 static Widget ngshell_w;	/* main shell */
@@ -83,14 +50,16 @@ static Widget pu_w;		/* popup shell */
 static Widget puname_w;		/* popup objects name label */
 static Pixmap ng_pm;		/* image view staging pixmap */
 static Widget dt_w;		/* main date/time stamp widget */
+static Widget ngsolid_w;	/* set when want to use a solid color */
 
 static GC ng_gc;		/* misc GC */
+static Pixel ngsolid_p;		/* color of ngsolid_tb set */
 static XFontStruct *ng_vfs;	/* views font */
 static XFontStruct *ng_lfs;	/* label font */
 static int ng_w, ng_h;		/* current ngda_w size */
 static int skyh;		/* sky region height, pixels */
 static int midx;		/* window x @ local midnight */
-static int leftday, rightday;	/* days +/1 wrt midx */
+static int nowx;		/* window x @ Now.n_mjd */
 
 static Pixel *ngcolors;		/* gray-scale ramp */
 static int ncolors;		/* actual number in ngcolors[] */
@@ -101,20 +70,27 @@ static int ncolors;		/* actual number in ngcolors[] */
 /* info about stuff to put in the popup */
 typedef struct {
     char *prompt;		/* prompt, or NULL for gap */
-    int id;			/* one of *_ID from dm.h */
+    DMCol id;			/* one of *_ID from dm.h */
     char *tip;			/* helpful tip */
     Widget w;			/* label widget */
 } PopupInfo;
 static PopupInfo puinfo[] = {
-    {"Rise:", RSTIME_ID,
-	"Rise time during date below, as per time zone Preference"},
-    {"Transit:", TRTIME_ID,
-	"Transit time during date below, as per time zone Preference"},
-    {"Transit Alt:", TRALT_ID,
-	"Transit altitude during date below, DD:MM"},
-    { "Set:", SETTIME_ID,
-	"Set time during date below, as per time zone Preference"},
+    {"Rise time:", RSTIME_ID,
+	"Time at which this object rises during this 24-hour period, in Preference time zone"},
+    {"Rise az:", RSAZ_ID,
+	"Azimuth at which this object rises, degrees E of N"},
+    {"Transit time:", TRTIME_ID,
+	"Time at which this object transits during this 24-hour period, in Preference time zone"},
+    {"Transit alt:", TRALT_ID,
+	"Altitude at which this object transits during this 24-hour period, DD:MM"},
+    {"Set time:", SETTIME_ID,
+	"Time at which this object sets during this 24-hour period, in Preference time zone"},
+    {"Set Az:", SETAZ_ID,
+	"Azimuth at which this object set, degrees E of N"},
 };
+static Now punow;		/* Now at popup cursor loc */
+static Widget punow_w;		/* PB to set XEphem time to punow */
+static Widget punowsep_w;	/* separator about punowpb_w */
 
 static char naagcategory[] = "Night at a Glance";	/* Save category */
 
@@ -126,12 +102,13 @@ ng_manage ()
 {
 	if (!ngshell_w) {
 	    ng_create_shell();
-	    ng_create_popup();
 	    ng_init_gcs();
 	}
 
 	XtPopup (ngshell_w, XtGrabNone);
 	set_something (ngshell_w, XmNiconic, (XtArgVal)False);
+
+	setXRes (ng_viewupres(), "1");
 }
 
 /* commanded from main to update with a new set of circumstances */
@@ -147,6 +124,16 @@ int how_much;
 	ng_redraw();
 }
 
+/* list of favorites has changed */
+void
+ng_newfavs()
+{
+	/* only if we're up */
+	if (!ng_ison())
+	    return;
+	ng_redraw();
+}
+
 /* called when basic resources change.
  * rebuild and redraw.
  */
@@ -159,18 +146,6 @@ ng_newres()
 	ng_update (mm_get_now(), 1);
 }
 
-/* called when a user-defined object has changed.
- */
-void
-ng_newobj(dbidx)
-int dbidx;		/* OBJXYZ */
-{
-	/* only if we're up */
-	if (!ng_ison())
-	    return;
-	ng_redraw();
-}
-
 /* called to put up or remove the watch cursor.  */
 void
 ng_cursor (c)
@@ -187,6 +162,14 @@ Cursor c;
 	}
 }
 
+/* return the name of the resource containing whether this view is up */
+char *
+ng_viewupres()
+{
+        return ("NaaGViewUp");
+}
+
+
 static int
 ng_ison()
 {
@@ -212,11 +195,13 @@ ng_create_shell()
 	ngshell_w = XtCreatePopupShell ("NaaGlance", topLevelShellWidgetClass,
 							toplevel_w, args, n);
 	set_something (ngshell_w, XmNcolormap, (XtArgVal)xe_cm);
+	setup_icon (ngshell_w);
 	XtAddCallback (ngshell_w, XmNpopdownCallback, ng_popdown_cb, 0);
 	sr_reg (ngshell_w, "XEphem*NaaGlance.width", naagcategory, 0);
 	sr_reg (ngshell_w, "XEphem*NaaGlance.height", naagcategory, 0);
 	sr_reg (ngshell_w, "XEphem*NaaGlance.x", naagcategory, 0);
 	sr_reg (ngshell_w, "XEphem*NaaGlance.y", naagcategory, 0);
+	sr_reg (NULL, ng_viewupres(), naagcategory, 0);
 
 	n = 0;
 	XtSetArg (args[n], XmNhorizontalSpacing, 5); n++;
@@ -247,6 +232,19 @@ ng_create_shell()
 	    set_xmstring (cb_w, XmNlabelString, "Control");
 	    XtManageChild (cb_w);
 
+	    /* the "OneColor" push button */
+
+	    n = 0;
+	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
+	    XtSetArg (args[n], XmNmarginHeight, 0); n++;
+	    XtSetArg (args[n], XmNindicatorType, XmN_OF_MANY); n++;
+	    ngsolid_w = XmCreateToggleButton (pd_w, "OneColor", args, n);
+	    XtAddCallback (ngsolid_w, XmNvalueChangedCallback, ng_1col_cb,NULL);
+	    set_xmstring (ngsolid_w, XmNlabelString, "One color");
+	    wtip (ngsolid_w, "Use same color for all objects");
+	    XtManageChild (ngsolid_w);
+	    sr_reg (ngsolid_w, NULL, naagcategory, 0);
+
 	    /* the "Print" push button */
 
 	    n = 0;
@@ -256,6 +254,34 @@ ng_create_shell()
 	    wtip (w, "Print the current map");
 	    XtManageChild (w);
 
+	    /* the "Favorites" push button */
+
+	    n = 0;
+	    w = XmCreatePushButton (pd_w, "NF", args, n);
+	    set_xmstring (w, XmNlabelString, "Favorites...");
+	    XtAddCallback (w, XmNactivateCallback,(XtCallbackProc)fav_manage,0);
+	    wtip (w, "Print the current map");
+	    XtManageChild (w);
+
+	    /* the "annotation" push button */
+
+	    n = 0;
+	    w = XmCreatePushButton (pd_w, "MAnn", args, n);
+	    set_xmstring (w, XmNlabelString, "User annotation...");
+	    XtAddCallback (w, XmNactivateCallback, ano_cb, 0);
+	    wtip (w, "Open window to create and manage your own annotation");
+	    XtManageChild (w);
+
+	    /* the "movie" push button */
+
+	    n = 0;
+	    n += ml_addacc (args, n);
+	    w = XmCreatePushButton (pd_w, "ML", args, n);
+	    set_xmstring (w, XmNlabelString, "Add to movie...");
+	    XtAddCallback (w, XmNactivateCallback, ng_mloop_cb, 0);
+	    wtip (w, "Add this chart to the movie loop");
+	    XtManageChild (w);
+
 	    /* add a separator */
 	    n = 0;
 	    w = XmCreateSeparator (pd_w, "CtS", args, n);
@@ -351,6 +377,23 @@ ng_create_popup()
 	    if (!pip->prompt)
 		set_xmstring (w, XmNlabelString, " ");
 	}
+
+	/* create the Set time PB */
+	n = 0;
+	punowsep_w = XmCreateSeparator (pu_w, "Sep", args, n);
+	XtManageChild (punowsep_w);
+
+	n = 0;
+	w = XmCreateLabel (pu_w, "ST", args, n);
+	set_xmstring (w, XmNlabelString, "Set Main time to:");
+	XtManageChild (w);
+
+	n = 0;
+	punow_w = XmCreatePushButton (pu_w, "Set", args, n);
+	XtAddCallback (punow_w, XmNactivateCallback, ng_settime_cb, NULL);
+	wtip (punow_w,
+		"Set main XEphem time to moment at mouse click, shown here");
+	XtManageChild (punow_w);
 }
 
 /* callback from the Print PB */
@@ -364,6 +407,17 @@ XtPointer call;
         XPSAsk ("Night-at-a-Glance", ng_print);
 }
 
+/* callback from the One Color TB */
+/* ARGSUSED */
+static void
+ng_1col_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+        ng_redraw();
+}
+
 /* callback from the inputCallback*/
 /* ARGSUSED */
 static void
@@ -374,56 +428,62 @@ XtPointer call;
 {
 	XmDrawingAreaCallbackStruct *c = (XmDrawingAreaCallbackStruct *)call;
 	Now *np = mm_get_now();
-	Now daynow;
-	RiseSet rs;
+	int nlfavs;
+	char buf[1024];
+	Obj **lfavs;
 	XEvent *ev;
-	Obj *op;
-	int obji;
 	int i;
 
-	/* confirm button 3 press event within sky portion */
+	/* create popup first time */
+	if (!pu_w)
+	    ng_create_popup();
+
+	/* confirm button 3 press event */
 	if (c->reason != XmCR_INPUT)
 	    return;
 	ev = c->event;
 	if (ev->xany.type != ButtonPress || ev->xbutton.button != 3)
 	    return;
-	if (ev->xbutton.y >= skyh)
-	    return;
 
-	/* get Obj */
-	obji = ev->xbutton.y*NOBJ/skyh;
-	op = db_basic (obji);
-	if (op->o_type == UNDEFOBJ)
-	    return;
+	/* set punow to time under cursor */
+	memcpy ((void *)&punow, (void *)np, sizeof(Now));
+	np = &punow;
+	mjd += (double)(ev->xbutton.x-nowx)/(2*midx);	/* 1 day wide */
 
-	/* set np to proper day */
-	if (ev->xbutton.x < midx) {
-	    if (leftday) {
-		daynow = *np;
-		daynow.n_mjd += leftday;
-		np = &daynow;
+	/* set time string in label */
+	fs_timestamp (np, buf);
+	set_xmstring (punow_w, XmNlabelString, buf);
+
+	/* show stats too if over an object */
+	nlfavs = fav_get_loaded (&lfavs);
+	if (ev->xbutton.y < skyh && ev->xbutton.y >= 0 && nlfavs > 0) {
+	    Obj *op = lfavs[ev->xbutton.y*nlfavs/skyh];
+	    RiseSet rs;
+
+	    /* need separator */
+	    XtManageChild (punowsep_w);
+
+	    /* set name */
+	    set_xmstring (puname_w, XmNlabelString, op->o_name);
+	    XtManageChild (puname_w);
+
+	    /* find and set info */
+	    dm_riset (np, op, &rs);
+	    for (i = 0; i < XtNumber(puinfo); i++) {
+		PopupInfo *pip = &puinfo[i];
+		if (pip->prompt) {
+		    char fmt[64];
+		    dm_colFormat (np, op, &rs, pip->id, fmt);
+		    sprintf (buf, "%s %s", pip->prompt, fmt);
+		    set_xmstring (pip->w, XmNlabelString, buf);
+		    XtManageChild (pip->w);
+		}
 	    }
 	} else {
-	    if (rightday) {
-		daynow = *np;
-		daynow.n_mjd += rightday;
-		np = &daynow;
-	    }
-	}
-
-	/* set name */
-	set_xmstring (puname_w, XmNlabelString, op->o_name);
-
-	/* find and set info */
-	dm_riset (np, op, &rs);
-	for (i = 0; i < XtNumber(puinfo); i++) {
-	    PopupInfo *pip = &puinfo[i];
-	    if (pip->prompt) {
-		char fmt[64], buf[64];
-		dm_colFormat (np, op, &rs, pip->id, fmt);
-		sprintf (buf, "%s %s", pip->prompt, fmt);
-		set_xmstring (pip->w, XmNlabelString, buf);
-	    }
+	    XtUnmanageChild (punowsep_w);
+	    XtUnmanageChild (puname_w);
+	    for (i = 0; i < XtNumber(puinfo); i++)
+		XtUnmanageChild(puinfo[i].w);
 	}
 
 	/* popup */
@@ -439,7 +499,7 @@ ng_print ()
 {
 	/* must be up */
 	if (!ng_ison()) {
-	    xe_msg ("N-ata-G must be open to print.", 1);
+	    xe_msg (1, "N-ata-G must be open to print.");
 	    XPSClose();
 	    return;
 	}
@@ -497,7 +557,7 @@ ng_ps_annotate ()
 
 	/* date and time on the left */
 	y = AROWY(9);
-	fs_date (buf, mjd_day(mjd));
+	fs_date (buf, pref_get(PREF_DATE_FORMAT), mjd_day(mjd));
 	(void) sprintf (dir, "(UTC Date:) %d %d rstr (%s) %d %d lstr\n",
 							lx, y, buf, lx+10, y);
 	XPSDirect (dir);
@@ -523,6 +583,17 @@ ng_ps_annotate ()
 	XPSDirect (dir);
 }
 
+/* callback from PB in popup to set XEphem time to punow. */
+/* ARGSUSED */
+static void
+ng_settime_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	mm_newcaldate (punow.n_mjd);
+}
+
 /* callback from ngshell_w being popped down. */
 /* ARGSUSED */
 static void
@@ -535,6 +606,9 @@ XtPointer call;
 	    XFreePixmap (XtD, ng_pm);
 	    ng_pm = 0;
 	}
+
+	/* record we are now down */
+	setXRes (ng_viewupres(), "0");
 }
 
 /* called from Close button */
@@ -548,6 +622,16 @@ XtPointer call;
 	/* popping down ngshell_w will do all the real work */
 }
 
+/* called to add graph to movie loop */
+static void
+ng_mloop_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	ml_add (ng_pm, dt_w);
+}
+
 /* callback from the Help all button
  */
 /* ARGSUSED */
@@ -604,7 +688,7 @@ XtPointer call;
 	    }
 	default:
 	    printf ("Unexpected ngda_w event. type=%d\n", c->reason);
-	    exit(1);
+	    abort();
 	}
 
 	XGetGeometry (dsp, win, &root, &x, &y, &wid, &hei, &bw, &d);
@@ -685,6 +769,9 @@ ng_init_gcs()
 
 	/* build gray ramp for night fade */
 	ncolors = gray_ramp (dsp, xe_cm, &ngcolors);
+
+	/* color for solid, when/if enabled */
+	(void) get_color_resource (ngshell_w, "NaaGOneColor", &ngsolid_p);
 }
 
 /* find x positions of line for given rise/set circumstances.
@@ -707,31 +794,33 @@ int *xr, *xs;
 
 /* find x positions of line for given rise/set circumstances.
  * we position across ng_w based on local time where midnight is at midx.
+ * N.B. do not use this for EARTHSAT, use rs_es().
  */
 static int
-rs_xrs (np, op, midx, nowx, xr, xs)
+rs_xrs (np, op, xr, xs)
 Now *np;
 Obj *op;
-int midx, nowx;
 int *xr, *xs;
 {
 	RiseSet rs;
 	int nowright;
+	int badrise, badset;
 
 	/* find x's */
 	if (op->o_type == UNDEFOBJ)
 	    return (-1);
-	db_update (op);
 	dm_riset (np, op, &rs);
 	if (rs.rs_flags & (RS_RISERR|RS_SETERR|RS_NEVERUP))
 	    return (-1);
 	rs_x (np, &rs, xr, xs);
 
-	/* if event occurs on opposite side of midnight as now, redo for
-	 * its day
+	/* if event occurs on opposite side of midnight as now or does not
+	 * occur at all today, redo for opposite day
 	 */
 	nowright = (nowx > midx);
-	if ((*xr > midx) != nowright || (*xs > midx) != nowright) {
+	badrise = (rs.rs_flags & RS_NORISE) || (*xr > midx) != nowright;
+	badset  = (rs.rs_flags & RS_NOSET)  || (*xs > midx) != nowright;
+	if (badrise || badset) {
 	    Now daynow;
 	    RiseSet dayrs;
 	    int dayxr, dayxs;
@@ -740,10 +829,16 @@ int *xr, *xs;
 	    daynow.n_mjd += nowright ? -1 : 1;
 	    dm_riset (&daynow, op, &dayrs);
 	    rs_x (&daynow, &dayrs, &dayxr, &dayxs);
-	    if ((*xr > midx) != nowright)
+	    if (rs.rs_flags & (RS_RISERR|RS_SETERR|RS_NEVERUP))
+		return (-1);
+	    if (badrise) {
 		*xr = dayxr;
-	    if ((*xs > midx) != nowright)
+		rs.rs_flags &= ~RS_NORISE;	/* definitely known now */
+	    }
+	    if (badset) {
 		*xs = dayxs;
+		rs.rs_flags &= ~RS_NOSET;	/* definitely known now */
+	    }
 	}
 
 	/* catch a few odd cases */
@@ -751,9 +846,64 @@ int *xr, *xs;
 	    *xr = 0;	/* already up */
 	if (rs.rs_flags & (RS_CIRCUMPOLAR|RS_NOSET))
 	    *xs = ng_w;	/* still up */
+
 	return (0);
 }
 
+/* plot all visible periods for EARTHSAT op.
+ * set xrp to left-most rise, xsp to right-most set.
+ */
+static int
+rs_es (np, op, y, xrp, xsp)
+Now *np;
+Obj *op;
+int y;
+int *xrp, *xsp;
+{
+	Display *dsp = XtD;
+	RiseSet rs;
+	Now daynow;
+	double emjd;
+	int xr, xs;
+	int n;
+
+	/* start at beginning of window and go for 1 day */
+	memcpy (&daynow, np, sizeof(daynow));
+	daynow.n_mjd = mjd_day(mjd - tz/24.0) + tz/24.0 + 0.5;
+	if (nowx > midx)
+	    daynow.n_mjd -= 1;
+	*xrp = 2*midx;
+	*xsp = 0;
+	n = 0;
+	for (emjd = daynow.n_mjd + 1; daynow.n_mjd < emjd;
+					daynow.n_mjd = rs.rs_settm+60./SPD) {
+	    dm_riset (&daynow, op, &rs);
+	    if (rs.rs_flags&(RS_ERROR|RS_RISERR|RS_SETERR|RS_NEVERUP|RS_NORISE))
+		break;
+	    rs_x (&daynow, &rs, &xr, &xs);
+	    if (rs.rs_risetm > emjd)
+		break;				/* off right end */
+	    if (rs.rs_flags & RS_NOSET) {
+		xs = 2*midx;			/* up through end */
+		rs.rs_settm = emjd;
+	    }
+	    if (rs.rs_settm < rs.rs_risetm)
+		continue;			/* wrong dir to use */
+	    if (rs.rs_settm > emjd)
+		xs = 2*midx;			/* sets after end */
+	    if (xs < xr+2)
+		xs = xr+2;			/* min seeable */
+	    XPSDrawLine (dsp, ng_pm, ng_gc, xr, y, xs, y);
+	    n++;
+	    if (xr < *xrp)
+		*xrp = xr;
+	    if (xs < *xsp)
+		*xsp = xs;
+	}
+
+	return (n ? 0 : -1);
+}
+
 /* draw the entire scene in ng_pm, which is known to be ng_w x ng_h.
  * N.B. this just fills the pixmap; call ng_refresh() to copy to the screen.
  */
@@ -767,6 +917,8 @@ ng_drawpm ()
 	int risex, setx;
 	int dawnx, duskx;
 	double dawn, dusk;
+	int nlfavs;
+	Obj **lfavs;
 	int hw, n;
 	int status;
 	Obj *op;
@@ -775,7 +927,6 @@ ng_drawpm ()
 	int dir, asc, des;
 	XCharStruct overall;
 	int fw, fh;
-	int nowx;
 	int lmarg;
 	int x1, x2;
 	int i;
@@ -791,7 +942,9 @@ ng_drawpm ()
 	y = strlen(utclabel) + 1;	/* " */
 	lmarg = fw * (x > y ? x : y);
 
-	/* find graph height sans room for scale and labeling at bottom */
+	/* find graph height sans room for scale and labeling at bottom.
+	 * N.B. this is copied in ng_ano()
+	 */
 	skyh = ng_h - 4*fh;
 
 	/* black-to-white is too harsh, use grays */
@@ -799,13 +952,18 @@ ng_drawpm ()
 	maxcol = 4*ncolors/5;			/* bright */
 	ncols = maxcol-mincol;			/* rails, not posts */
 
-	/* find times of twilight and sunrise/set */
+	/* find x of local midnight and now */
+	rs.rs_risetm = tz/24.+.5;	/* local midnight as an mjd */
+	rs.rs_settm = mjd;		/* now */
+	rs_x (np, &rs, &midx, &nowx);
+
+	/* find x of twilight and sunrise/set */
 	op = db_basic (SUN);
-	db_update (op);
 	twilight_cir (np, dip, &dawn, &dusk, &status);
-	rs.rs_risetm = dawn;
-	rs.rs_settm = dusk;
-	rs.rs_flags = status;
+	/* local midnight close enough if diurnal cycle lands beyond today */
+	rs.rs_risetm = (status&RS_NORISE) ? tz/24+.5 : dawn;
+	rs.rs_settm =  (status&RS_NOSET)  ? tz/24+.5 : dusk;
+	rs.rs_flags = status & ~(RS_NORISE|RS_NOSET);
 	if (rs.rs_flags) {
 	    dm_riset (np, op, &rs);
 	    if (rs.rs_flags & RS_CIRCUMPOLAR) {
@@ -839,16 +997,17 @@ ng_drawpm ()
 		rs_x (np, &rs, &risex, &setx);
 	}
 
-	/* canonize to window size */
+	/* modulo window size */
 	dawnx %= ng_w;
 	risex %= ng_w;
 	setx %= ng_w;
 	duskx %= ng_w;
 
-	/* find x of local midnight and now */
-	rs.rs_risetm = tz/24.+.5;	/* local midnight as an mjd */
-	rs.rs_settm = mjd;		/* now */
-	rs_x (np, &rs, &midx, &nowx);
+	/* beware sun dip set higher than sun rise/set value */
+	if (dawnx > risex)
+	    dawnx = risex;
+	if (duskx < setx)
+	    duskx = setx;
 
 	/* fill block at bottom for annotations -- screen only */
 	XSetForeground (dsp, ng_gc, BlackPixel (dsp, DefaultScreen(dsp)));
@@ -903,44 +1062,53 @@ ng_drawpm ()
 	XPSDrawLine (dsp, ng_pm, ng_gc, nowx, 0, nowx, skyh+fh);
 
 	/* overlay the names and thick bars */
+	nlfavs = fav_get_loaded (&lfavs);
 	XSetLineAttributes (dsp, ng_gc, UPLT, LineSolid, CapButt, JoinMiter);
 	XSetFont (dsp, ng_gc, ng_vfs->fid);
-	for (i = 0; i < NOBJ; i++) {
-	    unsigned long gcm;
-	    XGCValues gcv;
+	for (i = 0; i < nlfavs; i++) {
 	    char *name;
-	    GC gc;
+	    int esat;
 	    int y;
 
 	    /* get rise and set x values, or skip if undef or never up */
-	    op = db_basic(i);
-	    if (rs_xrs (np, op, midx, nowx, &risex, &setx) < 0)
+	    op = lfavs[i];
+	    db_update (op);
+	    esat = op->o_type == EARTHSAT;
+	    if (!esat && rs_xrs (np, op, &risex, &setx) < 0)
 		continue;
 
-	    /* get the standard color for this object type */
-	    obj_pickgc (op, ngda_w, &gc);
-
-	    /* name a little above the line, avoid far right */
-	    y = (i+1)*skyh/NOBJ-3*UPLT;
-	    name = op->o_name;
-	    if (risex > setx && ng_w-risex < 50)
-		XPSDrawString (dsp, ng_pm, gc, 10, y, name, strlen(name));
-	    else
-		XPSDrawString (dsp, ng_pm, gc, risex, y, name, strlen(name));
-
-	    /* grab the color but use our thick line gc */
-	    gcm = GCForeground;
-	    XGetGCValues (dsp, gc, gcm, &gcv);
-	    XSetForeground (dsp, ng_gc, gcv.foreground);
-	    y += 3*UPLT/2;
+	    /* set the desired color */
+	    if (XmToggleButtonGetState (ngsolid_w)) {
+		XSetForeground (dsp, ng_gc, ngsolid_p);
+	    } else {
+		unsigned long gcm;
+		XGCValues gcv;
+		GC gc;
+		obj_pickgc (op, ngda_w, &gc);
+		gcm = GCForeground;
+		XGetGCValues (dsp, gc, gcm, &gcv);
+		XSetForeground (dsp, ng_gc, gcv.foreground);
+	    }
 
 	    /* draw line .. beware of wrap */
-	    if (risex < setx) {
+	    y = (i+1)*skyh/nlfavs-3*UPLT/2;
+	    if (esat) {
+		if (rs_es (np, op, y, &risex, &setx) < 0)
+		    continue;
+	    } else if (risex < setx) {
 		XPSDrawLine (dsp, ng_pm, ng_gc, risex, y, setx, y);
 	    } else {
 		XPSDrawLine (dsp, ng_pm, ng_gc, 0, y, setx, y);
 		XPSDrawLine (dsp, ng_pm, ng_gc, risex, y, ng_w, y);
 	    }
+
+	    /* name a little above the line, avoid far right */
+	    y -= 3*UPLT/2;
+	    name = op->o_name;
+	    if (risex > setx && ng_w-risex < 50)
+		XPSDrawString (dsp, ng_pm, ng_gc, 10, y, name, strlen(name));
+	    else
+		XPSDrawString (dsp, ng_pm, ng_gc, risex, y, name, strlen(name));
 	}
 
 	/* hours and tick marks */
@@ -999,21 +1167,48 @@ ng_drawpm ()
 	XSetForeground (dsp, ng_gc, WhitePixel (dsp, DefaultScreen(dsp)));
 	y = skyh + 4*fh-2;
 	if (nowx < midx) {
-	    fs_date (buf, mjd_day(mjd-tz/24.));
-	    fs_date (buf2, mjd_day(mjd-tz/24.+1));
-	    leftday = 0;
-	    rightday = 1;
+	    fs_date (buf, pref_get(PREF_DATE_FORMAT), mjd_day(mjd-tz/24.));
+	    fs_date (buf2, pref_get(PREF_DATE_FORMAT), mjd_day(mjd-tz/24.+1));
 	} else {
-	    fs_date (buf, mjd_day(mjd-tz/24.-1));
-	    fs_date (buf2, mjd_day(mjd-tz/24.));
-	    leftday = -1;
-	    rightday = 0;
+	    fs_date (buf, pref_get(PREF_DATE_FORMAT), mjd_day(mjd-tz/24.-1));
+	    fs_date (buf2, pref_get(PREF_DATE_FORMAT), mjd_day(mjd-tz/24.));
 	}
 	n = strlen (buf);
 	XPSDrawString (dsp, ng_pm, ng_gc, (midx-fw*n)/2, y, buf, n);
 	n = strlen (buf2);
 	XPSDrawString (dsp, ng_pm, ng_gc, (ng_w+midx-fw*n)/2, y, buf2, n);
+
+	/* draw any annotation */
+	ano_draw (ngda_w, ng_pm, ng_ano, 0);
+}
+
+/* convert map fractions to/from X Windows coords depending on w2x.
+ * return whether visible
+ */
+static int
+ng_ano (double *fracx, double *fracy, int *xp, int *yp, int w2x, int arg)
+{
+	int dir, asc, des;
+	XCharStruct overall;
+	int fh, skyh;
+
+	/* get size of a typical string in the label font */
+	XTextExtents (ng_lfs, "5", 1, &dir, &asc, &des, &overall);
+	fh = 2*(overall.ascent + overall.descent);	/* room for gap */
+
+	/* find graph height sans room for scale and labeling at bottom */
+	skyh = ng_h - 4*fh;
+
+	if (w2x) {
+	    *xp = (int)floor(*fracx*ng_w);
+	    *yp = (int)floor(*fracy*skyh);
+	} else {
+	    *fracx = (double)*xp/ng_w;
+	    *fracy = (double)*yp/skyh;
+	}
+
+	return (1);
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: glance.c,v $ $Date: 2000/10/07 03:48:18 $ $Revision: 1.23 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: glance.c,v $ $Date: 2009/03/25 02:12:10 $ $Revision: 1.49 $ $Name:  $"};
