@@ -8,10 +8,7 @@
 #include <string.h>
 #include <math.h>
 #include <time.h>
-
-#if defined(__STDC__)
 #include <stdlib.h>
-#endif
 
 #include <X11/Xlib.h>
 #include <Xm/Xm.h>
@@ -23,70 +20,7 @@
 #include <Xm/Separator.h>
 #include <Xm/SelectioB.h>
 
-#include "P_.h"
-#include "astro.h"
-#include "circum.h"
-#include "sites.h"
-#include "preferences.h"
-
-extern Widget toplevel_w;
-extern XtAppContext xe_app;
-extern Colormap xe_cm;
-#define	XtD XtDisplay(toplevel_w)
-
-extern FILE *fopenh P_((char *name, char *how));
-extern Obj *db_basic P_((int id));
-extern Widget calm_create P_((Widget parent));
-extern char *getPrivateDir P_((void));
-extern char *getXRes P_((char *name, char *def));
-extern char *syserrstr P_((void));
-extern double atod P_((char *buf));
-extern int any_ison P_((void));
-extern int listing_ison P_((void));
-extern int plot_ison P_((void));
-extern int srch_eval P_((double Mjd, double *tmincp));
-extern int tz_fromsys P_((Now *np));
-extern void all_update P_((Now *np, int how_much));
-extern void buttonAsButton P_((Widget w, int whether));
-extern void calm_set P_((Now *np));
-extern void db_invalidate P_((void));
-extern void db_loadinitial P_((void));
-extern void db_update P_((Obj *op));
-extern void defaultTextFN P_((Widget w, int setcols, char *x, char *y));
-extern void dm_riset P_((Now *np, Obj *op, RiseSet *rsp));
-extern void f_date P_((Widget w, double jd));
-extern void f_dms_angle P_((Widget w, double a));
-extern void f_double P_((Widget w, char *fmt, double f));
-extern void f_mtime P_((Widget w, double t));
-extern void f_off P_((void));
-extern void f_on P_((void));
-extern void f_sexa P_((Widget wid, double a, int w, int fracbase));
-extern void f_showit P_((Widget w, char *s));
-extern void f_string P_((Widget w, char *s));
-extern void f_time P_((Widget w, double t));
-extern void fs_create P_((void));
-extern void get_something P_((Widget w, char *resource, XtArgVal value));
-extern void get_xmstring P_((Widget w, char *resource, char **txtp));
-extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
-extern void inc_mjd P_((Now *np, double inc, int rev, int rtcflag));
-extern void listing P_((void));
-extern void mm_go_cb P_((Widget w, XtPointer client, XtPointer call));
-extern void moonnf P_((double Mjd, double *Mjdn, double *Mjdf));
-extern void ng_update P_((Now *np, int how_much));
-extern void plot P_((void));
-extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
-extern void redraw_screen P_((int how_much));
-extern void register_selection P_((char *name));
-extern void set_something P_((Widget w, char *resource, XtArgVal value));
-extern void set_t0 P_((Now *np));
-extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
-extern void time_fromsys P_((Now *np));
-extern void twilight_cir P_((Now *np, double dis, double *dawn, double *dusk,
-    int *status));
-extern void watch_cursor P_((int want));
-extern void wtip P_((Widget w, char *tip));
-extern void xe_msg P_((char *msg, int app_modal));
+#include "xephem.h"
 
 /* Category for our widgets in the Save system */
 char maincategory[] = "Main -- Basics";
@@ -107,51 +41,51 @@ typedef struct {
 
 #define	MAXSITENL	25	/* max site name after abbreviation */
 
-static void create_main_form P_((Widget mainrc));
-static void create_pixmaps P_((void));
-static void make_prompts P_((Widget p_w, int pc, char *title, char *tip,
-    Field *fp, int nfp));
-static Widget fw P_((int fid));
-static void mm_step_now P_((int rev));
-static void mm_set_step_code P_((char *bp));
-static void mm_set_buttons P_((int whether));
-static void mm_activate_cb P_((Widget w, XtPointer client, XtPointer call));
-static void mm_initres P_((void));
-static void mm_pms P_((void));
-static void mm_set_alt_prompts P_((Field *fp));
-static void mm_timer_cb P_((XtPointer client, XtIntervalId *id));
-static void mm_stop P_((void));
-static void mm_step_cb P_((Widget w, XtPointer client, XtPointer call));
-static void mm_go_action P_((Widget w, XEvent *e, String *args,
-    Cardinal *nargs));
-static void keepnow_cb P_((Widget w, XtPointer client, XtPointer call));
-static void mm_gostop P_((int dir));
-static void mm_go P_((int dir));
-static void print_tminc P_((void));
-static void print_updating P_((void));
-static void print_idle P_((void));
-static void print_running P_((int rev));
-static void print_extrunning P_((void));
-static void print_status P_((char *s));
-static void print_nstep P_((void));
-static void print_mspause P_((void));
-static int chg_fld P_((char *bp, Field *fp));
-static void prompt_ok_cb P_((Widget w, XtPointer client, XtPointer call));
-static void prompt P_((Field *fp));
-static Widget create_prompt_w P_((Widget *wp));
-static void mm_now P_((int all));
-static void mm_twilight P_((void));
-static void mm_newcir P_((int y));
-static void mm_newcir_cb P_((XtPointer client, XtIntervalId *id));
-static void ext_fileask P_((void));
-static int ext_readnext P_((void));
-static void ext_stop P_((void));
-static void ext_create_w P_((void));
-static void ext_ok_cb P_((Widget w, XtPointer client, XtPointer call));
-static void ext_help_cb P_((Widget w, XtPointer client, XtPointer call));
+static void create_main_form (Widget mainrc);
+static void create_pixmaps (void);
+static void make_prompts (Widget p_w, int pc, char *title, char *tip,
+    Field *fp, int nfp);
+static Widget fw (int fid);
+static void mm_step_now (int rev);
+static void mm_set_step_code (char *bp);
+static void mm_set_buttons (int whether);
+static void mm_activate_cb (Widget w, XtPointer client, XtPointer call);
+static void mm_initres (void);
+static void mm_pms (void);
+static void mm_set_alt_prompts (Field *fp);
+static void mm_timer_cb (XtPointer client, XtIntervalId *id);
+static void mm_stop (void);
+static void mm_step_cb (Widget w, XtPointer client, XtPointer call);
+static void mm_go_action (Widget w, XEvent *e, String *args, Cardinal *nargs);
+static void keepnow_cb (Widget w, XtPointer client, XtPointer call);
+static void mm_gostop (int dir);
+static void mm_go (int dir);
+static void print_magdecl (void);
+static void print_tminc (void);
+static void print_updating (void);
+static void print_idle (void);
+static void print_running (int rev);
+static void print_extrunning (void);
+static void print_status (char *s);
+static void print_nstep (void);
+static void print_mspause (void);
+static int chg_fld (char *bp, Field *fp);
+static void prompt_ok_cb (Widget w, XtPointer client, XtPointer call);
+static void prompt (Field *fp);
+static Widget create_prompt_w (Widget *wp);
+static void mm_now (int all);
+static void mm_twilight (void);
+static void mm_newcir (int y);
+static void mm_newcir_cb (XtPointer client, XtIntervalId *id);
+static void ext_fileask (void);
+static int ext_readnext (void);
+static void ext_stop (void);
+static void ext_create_w (void);
+static void ext_ok_cb (Widget w, XtPointer client, XtPointer call);
+static void ext_help_cb (Widget w, XtPointer client, XtPointer call);
 
 /* shorthands for fields of a Now structure, now.
- * first undo the ones for a Now pointer from circum.h.
+ * first undo the ones for a Now pointer from astro.h.
  */
 #undef mjd
 #undef lat
@@ -181,6 +115,7 @@ static Now now;			/* where when and how, right now */
 static double tminc;		/* hrs to inc time each loop (see StepOptions)*/
 static int nstep;		/* steps to go before stopping */
 static int mspause;		/* msecs to pause between steps */
+static int startnstep;		/* nstep at start of 0-pause loop */
 static int newcir = 1;		/* new circumstances - don't inc time */
 static int movie;		/* true while a movie is running */
 static FILE *ext_fp;		/* if set, file to read time/loc from */
@@ -194,6 +129,7 @@ static int mm_selecting;        /* set while our fields are being selected */
 #define	NO_PAUSE_DELAY	25	/* min ms delay to use, regardless of mspause */
 #define	NC_BLINK_DELAY	200	/* ms between NEW CIRCUMSTANCES state changes */
 #define	MANYSTEPS	1000000	/* "many" steps */
+#define	MAXPNSTEP	1000	/* max steps for auto showing progress meter */
 #define	NOWPAUSE	10000	/* default Pause when staying in sync, ms */
 
 /* field ids
@@ -203,7 +139,7 @@ typedef enum {
     JD_FID, UD_FID, UT_FID, LST_FID,TZN_FID, TZONE_FID, LD_FID, LT_FID, DT_FID,
     DIP_FID, DAWN_FID, DUSK_FID, LON_FID, LSTM_FID, GAP, STPSZ_FID, NSTEP_FID,
     PAUSE_FID, SITE_FID, LAT_FID, LONG_FID, ELEV_FID, TEMP_FID, PRES_FID,
-    EPOCH_FID
+    EPOCH_FID, MAG_FID
 } FID;
 
 /* array of label/button pairs.
@@ -268,16 +204,17 @@ or press Auto to use model: ",
 	"\
 Select one of the above shortcuts\n\
 or enter any time step as follows:\n\
-    h:m:s, or\n\
+    h m s, or\n\
     <x>d for x days, or\n\
-    <x>s for x sidereal days",				"Step:", NULL},
+    <x>s for x sidereal days, or\n\
+    <x>y for x tropical years",				"Step:", NULL},
     {"NSteps", NSTEP_FID, 0, "Number of steps to run on next Update",
 	"Number of steps to run: ",			"N Steps:", NULL},
     {"Pause", PAUSE_FID, 0, "Seconds to pause after each Update",
 	"Seconds to pause between steps: ",		"Pause:", NULL},
     {"SiteNm", SITE_FID, 1, "Name of current site.",
 	"Search:",					NULL, NULL},
-    {"Lat", LAT_FID, 1, "Local geographic latitude, degrees, + north",
+    {"Lat", LAT_FID, 1,"Local geographic (geodetic) latitude, degrees, + north",
 	"Geographic Latitude (+ north) (d:m:s): ", 	"Latitude:", "Lat"},
     {"Long", LONG_FID, 1, "Local longitude, degrees, + west",
 	"Longitude (+ west) (d:m:s): ",			"Longitude:","Long"},
@@ -296,12 +233,15 @@ or enter any time step as follows:\n\
     {"Pres", PRES_FID, 1, "Local atmospheric pressure",
 	dummy,						"Atm Pres:","AtmPr",
 	{   "Atmospheric pressure (inches of Mercury):",
-	    "Atmospheric pressure (mBar):"
+	    "Atmospheric pressure (hPa):"
 	}
     },
     {"Equinox", EPOCH_FID, 1,
 	"Precession epoch: EOD for full apparent, fixed year for astrometric",
 	"Precession Epoch (decimal year): ",		"Equinox:", NULL},
+    {"MagDec", MAG_FID, 0,
+	"Magnetic bearing = true az + mag declination.",
+	NULL, "Mag decl:", "MagDec"},
 };
 
 #define	NFM	XtNumber(mm_field_map)
@@ -322,7 +262,7 @@ static Widget stepF_w, stepB_w;
  */
 typedef enum {
     FIXED_SS, STEPDAWN_SS, STEPDUSK_SS, STEPSUNRISE_SS, STEPSUNSET_SS,
-    STEPFULLMOON_SS, STEPNEWMOON_SS, RTC_SS
+    STEPFULLMOON_SS, STEPNEWMOON_SS, RTC_SS, STEPMOONRISE_SS, STEPMOONSET_SS
 } StepCode;
 
 typedef struct {
@@ -333,14 +273,17 @@ typedef struct {
 } StepOption;
 /* N.B. leave RT Clock first!! see the RTC_SS_OPTION macro */
 static StepOption step_options[] = {
-    {"RT Clock",	RTC_SS,		 0.0,	"Advance by elapsed time"},
+    {"Clock",		RTC_SS,		 0.0,	"Advance by elapsed time"},
     {"24:00:00",	FIXED_SS,	24.0,	"Step by 1 day"},
     {" 1:00:00",	FIXED_SS,	1.0,	"Step by 1 hour"},
+    {" 0:05:00",	FIXED_SS,	5./60.,	"Step by 5 minutes"},
     {" 0:01:00",	FIXED_SS,	1./60.,	"Step by 1 minute"},
     {"Dawn",		STEPDAWN_SS,	 0.0,	"Step to next dawn"},
     {"Dusk",		STEPDUSK_SS,	 0.0,	"Step to next dusk"},
     {"Sun rise",	STEPSUNRISE_SS,	 0.0,	"Step to next sun rise"},
     {"Sun set",		STEPSUNSET_SS,	 0.0,	"Step to next sun set"},
+    {"Moon rise",	STEPMOONRISE_SS, 0.0,	"Step to next moon rise"},
+    {"Moon set",	STEPMOONSET_SS,	 0.0,	"Step to next moon set"},
     {"Full moon",	STEPFULLMOON_SS, 0.0,	"Step to next full moon"},
     {"New moon",	STEPNEWMOON_SS,	 0.0,	"Step to next new moon"},
     {"Sidereal Day",	FIXED_SS,  24.*SIDRATE,	"Step by 1 Earth rotation"},
@@ -365,6 +308,8 @@ static StepOption *mm_step_option = RTC_SS_OPTION;
  */
 static char *msitename;
 static char absitename[MAXSITENL];
+static char snres[] = "Sitename";		/* name of X resource */
+static char nosite[] = "<No site defined>";	/* display/save if !msitename */
 
 /* map the XeGo action to the mm_go_action() handler function */
 static XtActionsRec mm_actions[] = {
@@ -431,6 +376,15 @@ static unsigned char sun_bits[] = {
 static Pixmap sun_pm;		/* pixmap when sun is up */
 static Widget sun_w;		/* label for sun_pm */
 
+/* empty bitmap */
+#define empty_width 13
+#define empty_height 13
+static unsigned char empty_bits[] = {
+   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+   0x00, 0x00};
+static Pixmap empty_pm;		/* empty pixmap */
+
 
 /* called exactly once when the main form is made.
  * create and manage all the widgets as children of the mainrc.
@@ -440,17 +394,25 @@ mm_create (mainrc)
 Widget mainrc;
 {
 	/* create the main form */
+	splashMsg ("Building main form\n");
 	create_main_form(mainrc);
+	splashMsg ("Building pixmaps\n");
 	create_pixmaps();
 
 	/* init resources and Now */
+	splashMsg ("Initializing resources\n");
 	mm_initres();
 
 	/* load the initial set of objects and set checkpoint there. */
+	splashMsg ("Loading data\n");
 	db_loadinitial();
 
 	/* setup the initial field star info */
+	splashMsg ("Setting up field stars\n");
 	fs_create();
+
+	/* resource used to record whether to initially start RT */
+	sr_reg (NULL, mm_autortres(), maincategory, 0);
 }
 
 /* connect up an action routine to handle XeUpdate */
@@ -501,7 +463,7 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	mm_gostop ((int)client);
+	mm_gostop ((long int)client);
 }
 
 /* called by the calendar subsystem to set a new UT date.
@@ -545,9 +507,11 @@ void
 mm_sitename (name)
 char *name;
 {
+	setXRes (snres, name);
 	if (msitename)
 	    XtFree(msitename);
 	msitename = name ? XtNewString(name) : NULL; /* yes, NULL is OK */
+	setXRes (snres, msitename ? msitename : nosite);
 	sites_abbrev (msitename, absitename, MAXSITENL);
 	mm_now (1);
 }
@@ -566,7 +530,6 @@ int update;
 	/* update everything */
 	lat = l;
 	lng = L;
-	elev = 0;
 	mm_sitename (NULL);
 
 	/* guess a timezone from longitude */
@@ -595,7 +558,7 @@ mm_setsite (sp, update)
 Site *sp;
 int update;
 {
-	static char envbuf[32]; /* putenv() wants stable string */
+	static char envbuf[64]; /* putenv() wants stable string */
 
 	/* update everything */
 	lat = sp->si_lat;
@@ -695,7 +658,7 @@ double stepsz;	/* step size, hours */
 	movie = 1;
 
 	/* init nstep and mspause to reasonable for long fast looping */
-	nstep = MANYSTEPS;
+	startnstep = nstep = MANYSTEPS;
 	print_nstep();
 	mspause = 0;
 	print_mspause();
@@ -756,6 +719,39 @@ int how_much;
 	watch_cursor(0);
 }
 
+/* name of resource containg whether to start running in real time */
+char *
+mm_autortres()
+{
+	return ("AutoRT");
+}
+
+/* keep XEphem time in sync with computer */
+void
+mm_startrt()
+{
+	/* RT Clock stepping */
+	set_t0 (&now);
+	mm_step_option = RTC_SS_OPTION;
+
+	/* many steps */
+	nstep = MANYSTEPS;
+
+	/* use same pause unless 0 */
+	if (mspause == 0)
+	    mspause = NOWPAUSE;
+
+	/* set to computer time */
+	time_fromsys (&now);
+	mm_newcaldate (mjd);
+
+	print_running(0);
+	mm_go(0);
+
+	/* record we are running RT */
+	setXRes (mm_autortres(), "1");
+}
+
 /* (re)create the pixmaps.
  * N.B. we use the colors from status_w.
  */
@@ -825,6 +821,12 @@ create_pixmaps()
 	    XFreePixmap (dsp, sun_pm);
 	sun_pm = XCreatePixmapFromBitmapData (dsp, win, (char *)sun_bits,
 					sun_width, sun_height, fg, bg, d); 
+
+	/* empty pixmap */
+	if (empty_pm)
+	    XFreePixmap (dsp, empty_pm);
+	empty_pm = XCreatePixmapFromBitmapData (dsp, win, (char *)empty_bits,
+					empty_width, empty_height, fg, bg, d); 
 }
 
 static void
@@ -1048,7 +1050,7 @@ int nfp;
 		XtSetArg (args[n], XmNmarginBottom, 0); n++;
 		XtSetArg (args[n], XmNmarginLeft, 0); n++;
 		XtSetArg (args[n], XmNmarginRight, 0); n++;
-		keepnow_w = XmCreatePushButton (f_w, "Now", args, n);
+		keepnow_w = XmCreatePushButton (f_w, "RT", args, n);
 		XtAddCallback (keepnow_w, XmNactivateCallback, keepnow_cb, 0);
 		wtip (keepnow_w,
 		    "Set looping to stay in sync with computer clock @ Pause intervals");
@@ -1173,7 +1175,7 @@ int fid;
 
 	if (fid < 0 || fid >= NFM || (fp = &mm_field_map[fid])->id != fid) {
 	    printf ("mainmenu:fw(): bad field id: %d\n", fid);
-	    exit(1);
+	    abort();
 	}
 	return (fp->pb_w);
 }
@@ -1235,23 +1237,7 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	/* RT Clock stepping */
-	set_t0 (&now);
-	mm_step_option = RTC_SS_OPTION;
-
-	/* many steps */
-	nstep = MANYSTEPS;
-
-	/* use same pause unless 0 */
-	if (mspause == 0)
-	    mspause = NOWPAUSE;
-
-	/* set to computer time */
-	time_fromsys (&now);
-	mm_newcaldate (mjd);
-
-	print_running(0);
-	mm_go(0);
+	mm_startrt();
 }
 
 /* set things to their initial resource settings */
@@ -1277,7 +1263,7 @@ mm_initres()
 	    {TEMP_FID,	"Temp"},
 	    {TZONE_FID,	"TZone"},
 	    {TZN_FID,	"TZName"},
-	    {SITE_FID,	"Sitename"},	/* overrides others if valid */
+	    {SITE_FID,	snres},		/* overrides others if valid */
 
 #if 0	    /* doing this works but how to /remove/ from res file once saved? */
 	    {JD_FID,	"JD"},		/* overrides Now if valid */
@@ -1288,23 +1274,32 @@ mm_initres()
 	    {LST_FID,	"LST"},		/* overrides Now if valid */
 #endif
 	};
+	char *sn;
 	int i;
 
 	/* init to now first */
 	time_fromsys (&now);
 
-	/* then scan other fields so they can override */
+	/* then scan other fields so they can override.
+	 * N.B. save Sitename early because Lat/Long will set it to null.
+	 */
+	sn = XtNewString(getXRes (snres, 0));
 	for (i = 0; i < XtNumber(imap); i++) {
-	    char *cp = getXRes (imap[i].res, NULL);
+	    char *cp = imap[i].fid == SITE_FID ? sn : getXRes (imap[i].res, 0);
 	    if (cp)
 		chg_fld (cp, &mm_field_map[imap[i].fid]);
 	}
+	XtFree (sn);
 
-	/* go from resources to real widget, then capture in save system */
+	/* go from resources to real widget, then capture in save system.
+	 * N.B. track Sitename directly; its display widget might be abbrev.
+	 */
 	redraw_screen (1);
-	for (i = 0; i < XtNumber(imap); i++)
-	    sr_reg (mm_field_map[imap[i].fid].pb_w, imap[i].res,
-			    maincategory, mm_field_map[imap[i].fid].autosav);
+	for (i = 0; i < XtNumber(imap); i++) {
+	    FID fid = imap[i].fid;
+	    sr_reg (fid == SITE_FID ? 0 : mm_field_map[fid].pb_w, imap[i].res,
+			    maincategory, mm_field_map[fid].autosav);
+	}
 
 	/* button info */
 	mm_set_buttons (mm_selecting);
@@ -1359,6 +1354,9 @@ mm_stop()
 	    mm_interval_id = 0;
 	}
 
+	/* close progress meter, if up */
+	pm_down();
+
 	/* do final screen update if last was partial */
 	if (nstep > 1 && mspause == 0)
 	    redraw_screen (1);
@@ -1410,15 +1408,19 @@ Cardinal *n;
 	/* assert */
 	if (!(n && *n && p)) {
 	    printf ("Bad go_action: %p %d %p\n", n, n?*n:0, p);
-	    exit(1);
+	    abort();
 	}
 
+	/* avoid runaway auto reps */
+	XSync (XtD, True);
+
 	/* ok */
 	mm_gostop (atoi(*p));
 }
 
 /* common routine to go or stop time.
  * dir is number of steps to go +step (+) or -step (-), or 0 to just GO.
+ * bring up pm if running fast and not a crazy number of steps.
  */
 static void
 mm_gostop(dir)
@@ -1426,9 +1428,15 @@ int dir;
 {
 	if (mm_interval_id) {
 	    mm_stop();
+	    /* record we have stopped (running RT for sure) */
+	    setXRes (mm_autortres(), "0");
 	} else {
-	    if (!dir && nstep > 1)
+	    if (!dir && nstep > 1) {
 		print_running(0);
+		startnstep = nstep;
+		if (mspause < NO_PAUSE_DELAY && nstep <= MAXPNSTEP)
+		    pm_up();
+	    }
 	    if (!newcir && any_ison())
 		mm_now (1);	/* update time fields in case of listing/etc */
 	    mm_go(dir);
@@ -1484,6 +1492,9 @@ int dir;
 	    /* update all fields if last didn't already */
 	    if (!drawall)
 		redraw_screen (1);
+
+	    /* close progress meter, if up */
+	    pm_down();
 	} else {
 	    long ms;
 
@@ -1503,6 +1514,10 @@ int dir;
 	    if (mm_interval_id != 0)
 		XtRemoveTimeOut (mm_interval_id);
 	    mm_interval_id = XtAppAddTimeOut (xe_app, ms, mm_timer_cb, 0);
+
+	    /* progress meter, if still running fast */
+	    if (mspause < NO_PAUSE_DELAY && nstep > 1)
+		pm_set (100*(startnstep-nstep)/startnstep);
 	}
 
 	/* XSync (XtD, False); */
@@ -1517,7 +1532,7 @@ mm_step_now(reverse)
 int reverse;
 {
 #define	RSACC	(30./3600./24.)	/* nominal rise/set accuracy, days */
-#define	BADRS (RS_NORISE|RS_NOSET|RS_ERROR|RS_CIRCUMPOLAR|RS_NEVERUP)
+#define	BADRS	(RS_ERROR|RS_CIRCUMPOLAR|RS_NEVERUP)
 	StepCode stepcode= mm_step_option ? mm_step_option->stepcode : FIXED_SS;
 	double dawn, dusk;
 	RiseSet rs;
@@ -1529,18 +1544,22 @@ int reverse;
 	switch (stepcode) {
 	case STEPDAWN_SS:
 	    twilight_cir (&now, dip, &dawn, &dusk, &status);
-	    if (status & BADRS) {
-		xe_msg ("No dawn.", 1);
-		nstep = 0;
-	    } else if ((reverse && mjd < dawn + RSACC) ||
-					(!reverse && mjd > dawn - RSACC)) {
+	    if ((status & (BADRS|RS_NORISE))
+					|| (!reverse && mjd > dawn - RSACC)
+					|| (reverse && mjd < dawn + RSACC)) {
+		/* try another few days */
 		Now n;
 		(void) memcpy ((void *)&n, (void *)&now, sizeof(Now));
 		n.n_mjd += reverse ? -1 : 1;
 		twilight_cir (&n, dip, &dawn, &dusk, &status);
-		if (status & BADRS) {
-		    xe_msg ("No dawn.", 1);
-		    nstep = 0;
+		if (status & (BADRS|RS_NORISE)) {
+		    n.n_mjd += reverse ? -1 : 1;
+		    twilight_cir (&n, dip, &dawn, &dusk, &status);
+		    if (status & (BADRS|RS_NORISE)) {
+			xe_msg (1, "No dawn within two days.");
+			nstep = 0;
+		    } else
+			mjd = dawn;
 		} else
 		    mjd = dawn;
 	    } else
@@ -1549,18 +1568,22 @@ int reverse;
 
 	case STEPDUSK_SS:
 	    twilight_cir (&now, dip, &dawn, &dusk, &status);
-	    if (status & BADRS) {
-		xe_msg ("No dusk.", 1);
-		nstep = 0;
-	    } else if ((reverse && mjd < dusk + RSACC) ||
-					(!reverse && mjd > dusk - RSACC)) {
+	    if ((status & (BADRS|RS_NOSET))
+					|| (!reverse && mjd > dusk - RSACC)
+					|| (reverse && mjd < dusk + RSACC)) {
+		/* try another few days */
 		Now n;
 		(void) memcpy ((void *)&n, (void *)&now, sizeof(Now));
 		n.n_mjd += reverse ? -1 : 1;
 		twilight_cir (&n, dip, &dawn, &dusk, &status);
-		if (status & BADRS) {
-		    xe_msg ("No dusk.", 1);
-		    nstep = 0;
+		if (status & (BADRS|RS_NOSET)) {
+		    n.n_mjd += reverse ? -1 : 1;
+		    twilight_cir (&n, dip, &dawn, &dusk, &status);
+		    if (status & (BADRS|RS_NOSET)) {
+			xe_msg (1, "No dusk within two days.");
+			nstep = 0;
+		    } else
+			mjd = dusk;
 		} else
 		    mjd = dusk;
 	    } else
@@ -1570,18 +1593,22 @@ int reverse;
 	case STEPSUNRISE_SS:
 	    op = db_basic(SUN);
 	    dm_riset (&now, op, &rs);
-	    if (rs.rs_flags & BADRS) {
-		xe_msg ("No sunrise.", 1);
-		nstep = 0;
-	    } else if ((reverse && mjd < rs.rs_risetm + RSACC) ||
-				(!reverse && mjd > rs.rs_risetm - RSACC)) {
+	    if ((rs.rs_flags & (BADRS|RS_NORISE))
+				|| (!reverse && mjd > rs.rs_risetm - RSACC)
+				|| (reverse && mjd < rs.rs_risetm + RSACC)) {
+		/* try another few days */
 		Now n;
 		(void) memcpy ((void *)&n, (void *)&now, sizeof(Now));
 		n.n_mjd += reverse ? -1 : 1;
 		dm_riset (&n, op, &rs);
-		if (rs.rs_flags & BADRS) {
-		    xe_msg ("No sunrise.", 1);
-		    nstep = 0;
+		if (rs.rs_flags & (BADRS|RS_NORISE)) {
+		    n.n_mjd += reverse ? -1 : 1;
+		    dm_riset (&n, op, &rs);
+		    if (rs.rs_flags & (BADRS|RS_NORISE)) {
+			xe_msg (1, "No sunrise within two days.");
+			nstep = 0;
+		    } else
+			mjd = rs.rs_risetm;
 		} else
 		    mjd = rs.rs_risetm;
 	    } else
@@ -1591,18 +1618,72 @@ int reverse;
 	case STEPSUNSET_SS:
 	    op = db_basic(SUN);
 	    dm_riset (&now, op, &rs);
-	    if (rs.rs_flags & BADRS) {
-		xe_msg ("No sunset.", 1);
-		nstep = 0;
-	    } else if ((reverse && mjd < rs.rs_settm + RSACC) ||
-				(!reverse && mjd > rs.rs_settm - RSACC)) {
+	    if ((rs.rs_flags & (BADRS|RS_NOSET))
+				|| (!reverse && mjd > rs.rs_settm - RSACC)
+				|| (reverse && mjd < rs.rs_settm + RSACC)) {
+		/* try another few days */
 		Now n;
 		(void) memcpy ((void *)&n, (void *)&now, sizeof(Now));
 		n.n_mjd += reverse ? -1 : 1;
 		dm_riset (&n, op, &rs);
-		if (rs.rs_flags & BADRS) {
-		    xe_msg ("No sunset.", 1);
-		    nstep = 0;
+		if (rs.rs_flags & (BADRS|RS_NOSET)) {
+		    n.n_mjd += reverse ? -1 : 1;
+		    dm_riset (&n, op, &rs);
+		    if (rs.rs_flags & (BADRS|RS_NOSET)) {
+			xe_msg (1, "No sunset within two days.");
+			nstep = 0;
+		    } else
+			mjd = rs.rs_settm;
+		} else
+		    mjd = rs.rs_settm;
+	    } else
+		mjd = rs.rs_settm;
+	    break;
+
+	case STEPMOONRISE_SS:
+	    op = db_basic(MOON);
+	    dm_riset (&now, op, &rs);
+	    if ((rs.rs_flags & (BADRS|RS_NORISE))
+				|| (!reverse && mjd > rs.rs_risetm - RSACC)
+				|| (reverse && mjd < rs.rs_risetm + RSACC)) {
+		/* try another few days */
+		Now n;
+		(void) memcpy ((void *)&n, (void *)&now, sizeof(Now));
+		n.n_mjd += reverse ? -1 : 1;
+		dm_riset (&n, op, &rs);
+		if (rs.rs_flags & (BADRS|RS_NORISE)) {
+		    n.n_mjd += reverse ? -1 : 1;
+		    dm_riset (&n, op, &rs);
+		    if (rs.rs_flags & (BADRS|RS_NORISE)) {
+			xe_msg (1, "No moonrise within two days.");
+			nstep = 0;
+		    } else
+			mjd = rs.rs_risetm;
+		} else
+		    mjd = rs.rs_risetm;
+	    } else
+		mjd = rs.rs_risetm;
+	    break;
+
+	case STEPMOONSET_SS:
+	    op = db_basic(MOON);
+	    dm_riset (&now, op, &rs);
+	    if ((rs.rs_flags & (BADRS|RS_NOSET))
+				|| (!reverse && mjd > rs.rs_settm - RSACC)
+				|| (reverse && mjd < rs.rs_settm + RSACC)) {
+		/* try another few days */
+		Now n;
+		(void) memcpy ((void *)&n, (void *)&now, sizeof(Now));
+		n.n_mjd += reverse ? -1 : 1;
+		dm_riset (&n, op, &rs);
+		if (rs.rs_flags & (BADRS|RS_NOSET)) {
+		    n.n_mjd += reverse ? -1 : 1;
+		    dm_riset (&n, op, &rs);
+		    if (rs.rs_flags & (BADRS|RS_NOSET)) {
+			xe_msg (1, "No moonset within two days.");
+			nstep = 0;
+		    } else
+			mjd = rs.rs_settm;
 		} else
 		    mjd = rs.rs_settm;
 	    } else
@@ -1665,7 +1746,7 @@ int reverse;
 
 	default:
 	    printf ("mm_step_now(): bogus stepcode: %d\n", stepcode);
-	    exit (1);
+	    abort();
 	}
 
 	if (autotz)
@@ -1679,7 +1760,27 @@ int reverse;
 #undef	BADRS
 }
 
-/* ARGSUSED */
+static void
+print_magdecl()
+{
+	static int lasts = -100;
+        double yr, md;
+	char msg[1024];
+	char dir[1024];
+	int s;
+
+	mjd_year (mjd, &yr);
+	sprintf (dir, "%s/auxil", getShareDir());
+	s = magdecl (lat, lng, elev*ERAD, yr, dir, &md, msg);
+	if (s < 0) {
+	    md = 0;
+	    if (s != lasts)
+		xe_msg (0, "%s", msg);
+	}
+	lasts = s;
+	f_dms_angle (fw(MAG_FID), md);
+}
+
 static void
 print_tminc()
 {
@@ -1694,6 +1795,8 @@ print_tminc()
 	case STEPDUSK_SS:	/* same as .. */
 	case STEPSUNRISE_SS:	/* same as .. */
 	case STEPSUNSET_SS:	/* same as .. */
+	case STEPMOONRISE_SS:	/* same as .. */
+	case STEPMOONSET_SS:	/* same as .. */
 	case STEPFULLMOON_SS:	/* same as .. */
 	case STEPNEWMOON_SS:	/* same as .. */
 	case RTC_SS:
@@ -1711,7 +1814,7 @@ print_tminc()
 
 	default:
 	    printf ("print_tminc(): bogus stepcode: %d\n", stepcode);
-	    exit (1);
+	    abort();
 	}
 }
 
@@ -1801,7 +1904,7 @@ Field *fp;
 
 	if (!bp) {
 	    printf ("Bug! NULL bp for field %s\n", fp->iname);
-	    exit(1);
+	    abort();
 	}
 
 	switch (fp->id) {
@@ -1821,24 +1924,19 @@ Field *fp;
 						    &month, &day, &year);
 		cal_mjd (month, day, year, &newmjd0);
 
-		/* if don't give a fractional part to days
-		 * then retain current hours.
-		 */
-		if ((long)day == day)
-		    mjd = newmjd0 + mjd_hr(mjd)/24.0;
-		else
+		/* if don't see a decimal retain current hours */
+		if (strchr(bp,'.'))
 		    mjd = newmjd0;
+		else
+		    mjd = newmjd0 + mjd_hr(mjd)/24.0;
 	    }
 	    set_t0 (&now);
 	    new = 1;
 	    break;
 
 	case UT_FID:
-	    {
-		double newutc = (mjd-mjd_day(mjd)) * 24.0;
-		f_scansex (newutc, bp, &newutc);
-		mjd = mjd_day(mjd) + newutc/24.0;
-	    }
+	    f_scansexa (bp, &tmp);
+	    mjd = mjd_day(mjd) + tmp/24.0;
 	    set_t0 (&now);
 	    new = 1;
 	    break;
@@ -1853,13 +1951,11 @@ Field *fp;
 						    &month, &day, &year);
 		cal_mjd (month, day, year, &newlmjd0);
 
-		/* if don't give a fractional part to days
-		 * then retain current hours.
-		 */
-		if ((long)day == day)
-		    mjd = newlmjd0 + mjd_hr(mjd-tz/24.0)/24.0;
-		else
+		/* if don't see a decimal retain current hours */
+		if (strchr(bp,'.'))
 		    mjd = newlmjd0;
+		else
+		    mjd = newlmjd0 + mjd_hr(mjd-tz/24.0)/24.0;
 		mjd += tz/24.0;
 	    }
 	    set_t0 (&now);
@@ -1867,12 +1963,8 @@ Field *fp;
 	    break;
 
 	case LT_FID:
-	    {
-		double newlt = (mjd-mjd_day(mjd)) * 24.0 - tz;
-		range (&newlt, 24.0);
-		f_scansex (newlt, bp, &newlt);
-		mjd = mjd_day(mjd-tz/24.0) + (newlt + tz)/24.0;
-	    }
+	    f_scansexa (bp, &tmp);
+	    mjd = mjd_day(mjd-tz/24.0) + (tmp + tz)/24.0;
 	    set_t0 (&now);
 	    new = 1;
 	    break;
@@ -1893,8 +1985,7 @@ Field *fp;
 		double lst, gst, utc;
 
 		/* read goal */
-		now_lst (&now, &lst);
-		f_scansex (lst, bp, &lst);
+		f_scansexa (bp, &lst);
 
 		/* compute new mjd based on basic transform */
 		gst = lst - radhr(lng); /* convert to gst */
@@ -1921,7 +2012,7 @@ Field *fp;
 
 	case TZONE_FID:
 	    /* TZONE_FID */
-	    f_scansex (tz, bp, &tz);
+	    f_scansexa (bp, &tz);
 	    autotz = 0;
 	    new = 1;
 	    break;
@@ -1941,16 +2032,14 @@ Field *fp;
 	    break;
 
 	case LONG_FID:
-	    tmp = -raddeg(lng);
-	    f_scansex (tmp, bp, &tmp);
+	    f_scansexa (bp, &tmp);
 	    lng = degrad (-tmp); 	/* want <0 radians west */
 	    mm_setll (lat, lng, 0);
 	    new = 1;
 	    break;
 
 	case LAT_FID:
-	    tmp = raddeg(lat);
-	    f_scansex (tmp, bp, &tmp);
+	    f_scansexa (bp, &tmp);
 	    lat = degrad (tmp);
 	    mm_setll (lat, lng, 0);
 	    new = 1;
@@ -1967,9 +2056,9 @@ Field *fp;
 	    break;
 
 	case DIP_FID:
-	    if (sscanf (bp, "%lf", &tmp) == 1) {
+	    if (f_scansexa (bp, &tmp) == 0) {
 		if (tmp < 0)
-		    xe_msg ("Twilight dip must be at least 0", 1);
+		    xe_msg (1, "Twilight dip must be at least 0");
 		else {
 		    dip = degrad(tmp);
 		    mm_twilight ();
@@ -1993,10 +2082,7 @@ Field *fp;
 		if (pref_get(PREF_UNITS) == PREF_ENGLISH)
 		    temp = 5./9.*(temp - 32.0);	/* want degs C */
 		if (temp < -100 || temp > 100) {
-		    char msg[128];
-		    (void) sprintf (msg,
-				"NOTICE: very unusual temperature: %g C", temp);
-		    xe_msg (msg, 0);
+		    xe_msg (0, "NOTICE: very unusual temperature: %g C", temp);
 		}
 		new = 1;
 	    }
@@ -2005,13 +2091,11 @@ Field *fp;
 	case PRES_FID:
 	    if (sscanf (bp, "%lf", &pressure) == 1) {
 		if (pref_get(PREF_UNITS) == PREF_ENGLISH) 
-		    pressure *= 33.86;		/* want mBar */
+		    pressure *= 33.86;		/* want hPa */
 		if (pressure < 0 || pressure > 4000) {
-		    char msg[128];
-		    (void) sprintf (msg,
-			    "NOTICE: very unusual atmospheric pressure: %g mB",
+		    xe_msg (0,
+			    "NOTICE: very unusual atmospheric pressure: %g hPa",
 								    pressure);
-		    xe_msg (msg, 0);
 		}
 		new = 1;
 	    }
@@ -2036,7 +2120,7 @@ Field *fp;
 
 	default:
 	    printf ("chg_fld: unknown id: %d\n", fp->id);
-	    exit (1);
+	    abort();
 	}
 
 	return (new);
@@ -2069,7 +2153,7 @@ XtPointer call;
 	    break;
 	default:
 	    printf ("main prompt_ok_cb: unknown reason: %d\n", s->reason);
-	    exit(1);
+	    abort();
 	}
 
 	get_something (w, XmNuserData, (XtArgVal)&fp);
@@ -2105,7 +2189,7 @@ Field *fp;
 
 	/* sites input is handled all separately */
 	if (fp->id == SITE_FID) {
-	    sites_query();
+	    sites_manage();
 	    return;
 	}
 
@@ -2292,8 +2376,10 @@ char *bp;
 		tminc = atod(bp) * 24.0;
 	    else if (strpbrk(bp, "sS"))
 		tminc = atod(bp) * 24.0 * SIDRATE;
+	    else if (strpbrk(bp, "yY"))
+		tminc = atod(bp) * 365.2425 * 24.0; /* mean gregorian */
 	    else
-		f_scansex (0.0, bp, &tminc);
+		f_scansexa (bp, &tminc);
 	}
 }
 
@@ -2305,21 +2391,18 @@ mm_pms()
 	Pixmap moonpm;
 
 	/* dst */
-	set_something (dst_w, XmNlabelPixmap,
-				    autotz ? dst_pm : XmUNSPECIFIED_PIXMAP);
+	set_something (dst_w, XmNlabelPixmap, autotz ? dst_pm : empty_pm);
 
 	/* moon up/phase */
 	op = db_basic(MOON);
 	db_update (op);
 	moonpm = moon_pm[((int)((op->s_elong+202)/45))%NMOONBM];
-	set_something (moon_w, XmNlabelPixmap,
-				op->s_alt > 0 ? moonpm : XmUNSPECIFIED_PIXMAP);
+	set_something (moon_w, XmNlabelPixmap, op->s_alt>0 ? moonpm : empty_pm);
 
 	/* sun up */
 	op = db_basic(SUN);
 	db_update (op);
-	set_something (sun_w, XmNlabelPixmap,
-				op->s_alt > 0 ? sun_pm : XmUNSPECIFIED_PIXMAP);
+	set_something (sun_w, XmNlabelPixmap, op->s_alt>0 ? sun_pm : empty_pm);
 }
 
 /* print all the time/date/where related stuff: the Now structure.
@@ -2353,9 +2436,10 @@ int all;
 	now_lst (&now, &tmp);
 	f_time (fw(LST_FID), tmp);
 
-	f_showit(fw(SITE_FID), msitename ? absitename : "<No site defined>");
+	f_showit(fw(SITE_FID), msitename ? absitename : nosite);
 	f_dms_angle (fw(LAT_FID), lat);
 	f_dms_angle (fw(LONG_FID), -lng);	/* + west */
+	print_magdecl ();
 	if (pref_get(PREF_UNITS) == PREF_ENGLISH) {
 	    tmp = elev * (ERAD*FTPM);	/* want ft, not earth radii*/
 	    f_double (fw(ELEV_FID), "%7.1f ft", tmp);
@@ -2377,10 +2461,10 @@ int all;
 
 	    tmp = pressure;
 	    if (pref_get(PREF_UNITS) == PREF_ENGLISH) {
-		tmp /= 33.86;    /* want to see in. Hg, not mBar */
+		tmp /= 33.86;    /* want to see in. Hg, not hPa */
 		f_double (fw(PRES_FID), "%5.2f in", tmp);
 	    } else {
-		f_double (fw(PRES_FID), "%5.0f mB", tmp);
+		f_double (fw(PRES_FID), "%5.0f hPa", tmp);
 	    }
 
 	    if (epoch == EOD)
@@ -2423,14 +2507,28 @@ mm_twilight ()
 	    f_string (fw(DUSK_FID), nope);
 	else
 	    f_mtime (fw(DUSK_FID), mjd_hr(dusk));
+
+	/* length of night */
 	if (status & RS_NEVERUP)
 	    f_mtime (fw(LON_FID), 24.0);
 	else if (status & RS_CIRCUMPOLAR)
 	    f_mtime (fw(LON_FID), 0.0);
-	else if (status & (RS_NORISE|RS_NOSET|RS_ERROR))
+	else if (status & RS_ERROR)
+	    f_string (fw(LON_FID), nope);
+	else if ((status & RS_NORISE) && (status & RS_NOSET))
 	    f_string (fw(LON_FID), nope);
 	else {
-	    double tmp = (dawn - dusk)*24.0;
+	    /* at least one of dusk and dawn.
+	     * if only rise, assume set at prior midnight.
+	     * if only set, assume rises at next midnight.
+	     * N.B. dawn+dusk are mjd
+	     */
+	    double tmp;
+	    if (status & RS_NORISE)
+		dawn = .5;
+	    else if (status & RS_NOSET)
+		dusk = -.5;
+	    tmp = (dawn - dusk)*24.0;
 	    range (&tmp, 24.0);
 	    f_mtime (fw(LON_FID), tmp);
 	}
@@ -2571,13 +2669,12 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	char msg[1024];
 	char *name;
 
 	/* get the file name */
 	get_xmstring(w, XmNtextString, &name);
 	if (strlen(name) == 0) {
-	    xe_msg ("Please enter the name of an external input file.", 1);
+	    xe_msg (1, "Please enter the name of an external input file.");
 	    XtFree (name);
 	    return;
 	}
@@ -2585,16 +2682,14 @@ XtPointer call;
 	/* open it */
 	ext_fp = fopenh (name, "r");
 	if (!ext_fp) {
-	    (void) sprintf (msg, "%s: %s", name, syserrstr());
-	    xe_msg (msg, 1);
+	    xe_msg (1, "%s: %s", name, syserrstr());
 	    XtFree (name);
 	    return;
 	}
 
 	/* read one entry just to test */
 	if (ext_readnext() < 0) {
-	    (void) sprintf (msg, "%s: contains no valid entries", name);
-	    xe_msg (msg, 1);
+	    xe_msg (1, "%s: contains no valid entries", name);
 	    XtFree (name);
 	    return;
 	}
@@ -2636,4 +2731,4 @@ static char *hlp[] = {
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: mainmenu.c,v $ $Date: 2001/10/15 02:56:26 $ $Revision: 1.59 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: mainmenu.c,v $ $Date: 2009/01/05 20:55:54 $ $Revision: 1.90 $ $Name:  $"};
