@@ -181,7 +181,7 @@ httpGET (char *host, char *GETcmd, char msg[])
 	int fd;
 	int n;
 
-	/* fill buf[] with GET command */
+	/* open connection */
 	if (proxy_on) {
 	    fd = mkconnection (proxy_host, proxy_port, msg);
 	    if (fd < 0)
@@ -193,8 +193,8 @@ httpGET (char *host, char *GETcmd, char msg[])
 		return (-1);
 	}
 
-	/* use absoluteURL in lieu of HTTP1.1 for now */
-	(void) sprintf (buf, "GET http://%s%s", host, GETcmd);
+	/* fill buf */
+	(void) sprintf (buf, "%s", GETcmd);
 
 	/* add proxy auth if enabled */
 	if (!auth_w)
@@ -202,6 +202,7 @@ httpGET (char *host, char *GETcmd, char msg[])
 	if (XmToggleButtonGetState (auth_w))
 	    addAuth(buf);
 
+	/* log it */
 	xe_msg (0, "http: %s", buf);
 
 	/* send it */
@@ -423,7 +424,7 @@ recvlineb (int sock, char *buf, int size)
 {
 	char *origbuf = buf;		/* save to prevent overfilling buf */
 	char c = '\0';
-	int nr = 0;
+	int ok = 1;
 
 	/* always leave room for trailing \n */
 	size -= 1;
@@ -433,13 +434,20 @@ recvlineb (int sock, char *buf, int size)
 
 	    if (rb_next >= rb_unk) {
 		/* linebuf is empty -- refill */
+
+		int nr;
+
 		if (tout (TOUT, sock, 0) < 0) {
 		    nr = -1;
 		    break;
 		}
 		nr = read (sock, rb_linebuf, sizeof(rb_linebuf));
-		if (nr <= 0)
+		if (nr <= 0) {
+		    ok = nr;
+		    rb_next = 0;
+		    rb_unk = 0;
 		    break;
+		}
 		rb_next = 0;
 		rb_unk = nr;
 	    }
@@ -450,12 +458,12 @@ recvlineb (int sock, char *buf, int size)
 	} while (buf-origbuf < size && c != '\n');
 
 	/* always give back a real line regardless, else status */
-	if (c == '\n') {
+	if (ok > 0) {
 	    *buf = '\0';
-	    nr = buf - origbuf;
+	    ok = buf - origbuf;
 	}
 
-	return (nr);
+	return (ok);
 }
 
 static void
@@ -1027,4 +1035,4 @@ int addrlen;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: netmenu.c,v $ $Date: 2009/01/05 20:55:54 $ $Revision: 1.28 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: netmenu.c,v $ $Date: 2010/10/06 21:12:16 $ $Revision: 1.30 $ $Name:  $"};
