@@ -4,9 +4,8 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <math.h>
-#if defined(__STDC__)
 #include <stdlib.h>
-#endif
+
 #include <X11/Xlib.h>
 #include <Xm/Xm.h>
 #include <Xm/Form.h>
@@ -17,27 +16,14 @@
 #include <Xm/Text.h>
 
 
-#include "P_.h"
-#include "astro.h"
-#include "ps.h"
+#include "xephem.h"
 
-extern int get_color_resource P_((Widget w, char *cname, Pixel *p));
-extern int tickmarks P_((double min, double max, int numdiv, double ticks[]));
-extern void fs_date P_((char out[], double jd));
-extern void get_something P_((Widget w, char *resource, XtArgVal value));
-extern void get_views_font P_((Display *dsp, XFontStruct **fspp));
-extern void xe_msg P_((char *msg, int app_modal));
-
-static void mk_gcs P_((Widget w));
-static int x_ticks P_((int asdate, double minx, double maxx, int maxticks,
-    double ticks[]));
-static int draw_x_label P_((Display *dsp, Window win, int asdate, double v,
-    int x, int y, int w));
-
-/* maximum number of unique functions (ie tags) in the plot file.
- * to be fair, it should be at least as large as the value in plot.
- */
-#define	MAXPLTLINES	10
+static void mk_gcs (Widget w);
+static double binsplit (int i);
+static int x_ticks (int asdate, double minx, double maxx, int maxticks,
+    double ticks[]);
+static int draw_x_label (Display *dsp, Window win, int asdate, double v,
+    int x, int y, int w);
 
 static GC plt_gc;		/* the GC to use for everything */
 static Pixel plt_p[MAXPLTLINES];/* colors, one per category */
@@ -50,13 +36,13 @@ static Pixel da_p;		/* the foreground color of the drawing area */
  * return 0 if ok, else give a message about trouble and return -1.
  */
 int
-plot_cartesian (pfp, widget, nx, ny, flipx, flipy, grid, xasdate)
-FILE *pfp;
-Widget widget;
-unsigned int nx, ny;
-int flipx, flipy;	/* respectively true to flip their axis */
-int grid;		/* whether to include a grid with the tick marks */
-int xasdate;		/* whether to show x axis as dates */
+plot_cartesian (
+FILE *pfp,
+Widget widget,
+unsigned int nx, unsigned int ny,
+int flipx, int flipy,	/* respectively true to flip their axis */
+int grid,		/* whether to include a grid with the tick marks */
+int xasdate)		/* whether to show x axis as dates */
 {
 #define	TL	5	/* tick mark length, in pixels */
 #define	RB	30	/* right border */
@@ -66,12 +52,12 @@ int xasdate;		/* whether to show x axis as dates */
 	double xticks[NT+2];
 	double yticks[NT+2];
 	int nxt, nyt;
-	static char fmt[] = "%c,%lf,%lf";
+	static char fmt[] = "%[A-Za-z0-9 ],%lf,%lf";
 	double x, y;	/* N.B. be sure these match what scanf's %lf wants*/
 	double minx=0, maxx=0, miny=0, maxy=0, xscale, yscale;
 	char buf[128];
 	int lx[MAXPLTLINES], ly[MAXPLTLINES], one[MAXPLTLINES];
-	char c, tags[MAXPLTLINES];
+	char tag[MAXTAG+1], tags[MAXPLTLINES][MAXTAG+1];
 	XCharStruct overall;
 	int sawtitle;
 	int ylblw;	/* label width */
@@ -89,23 +75,22 @@ int xasdate;		/* whether to show x axis as dates */
 	/* find ranges and number of and tags for each unique line */
 	nlines = 0;
 	while (fgets (buf, sizeof(buf), pfp)) {
-	    if (sscanf (buf, fmt, &c, &x, &y) != 3)
+	    if (sscanf (buf, fmt, tag, &x, &y) != 3)
 		continue;
 	    if (nlines == 0) {
 		maxx = minx = x;
 		maxy = miny = y;
 	    }
 	    for (i = 0; i < nlines; i++)
-		if (c == tags[i])
+		if (strcmp(tag, tags[i]) == 0)
 		    break;
 	    if (i == nlines) {
 		if (nlines == MAXPLTLINES) {
-		   (void) sprintf (buf,
-		     "Plot file contains more than %d functions.", MAXPLTLINES);
-		   xe_msg (buf, 1);
+		   xe_msg (1, "Plot file contains more than %d functions.",
+								   MAXPLTLINES);
 		   return(-1);
 		}
-		tags[nlines++] = c;
+		strcpy (tags[nlines++], tag);
 	    }
 	    if (x > maxx) maxx = x;
 	    else if (x < minx) minx = x;
@@ -114,12 +99,12 @@ int xasdate;		/* whether to show x axis as dates */
 	}
 
 	if (nlines == 0) {
-	    xe_msg ("Plot file appears to be empty.", 1);
+	    xe_msg (1, "Plot file appears to be empty.");
 	    return(-1);
 	}
 #define	SMALL	(1e-6)
 	if (fabs(minx-maxx) < SMALL || fabs(miny-maxy) < SMALL) {
-	    xe_msg ("Plot file values contain insufficient spread.", 1);
+	    xe_msg (1, "Plot file values contain insufficient spread.");
 	    return(-1);
 	}
 
@@ -201,7 +186,7 @@ int xasdate;		/* whether to show x axis as dates */
 	for (i = 0; i < nlines; i++)
 	    one[i] = 0;
 	while (fgets (buf, sizeof(buf), pfp)) {
-	    if (sscanf (buf, fmt, &c, &x, &y) != 3) {
+	    if (sscanf (buf, fmt, tag, &x, &y) != 3) {
 		/* a title line ? */
 		int l;
 
@@ -217,7 +202,7 @@ int xasdate;		/* whether to show x axis as dates */
 		continue;
 	    }
 	    for (i = 0; i < nlines; i++)
-		if (c == tags[i])
+		if (strcmp(tag, tags[i]) == 0)
 		    break;
 	    ix = XCORD(x);
 	    iy = YCORD(y);
@@ -227,7 +212,7 @@ int xasdate;		/* whether to show x axis as dates */
 	    else {
 		int ytop = y0 - h + asc;
 		XPSDrawString (dsp, win, plt_gc, ix+2, (iy<ytop ? ytop : iy)-2,
-									&c, 1);
+							    tag, strlen(tag));
 	    }
 	    lx[i] = ix;
 	    ly[i] = iy;
@@ -254,19 +239,43 @@ Widget w;
 	get_views_font (dsp, &plt_fs);
 
 	/* fill in plt_p array with pixels to use for function plotting.
-	 * Use colors defined in the plotColor[0-9] resources, else reuse fg.
-	 * they all use the same font.
 	 */
 	for (i = 0; i < MAXPLTLINES; i++) {
-	    Pixel *p = &plt_p[i];
-	    char cname[32];
+	    double h, s, v, r, g, b;
+	    XColor xc;
 
-	    (void) sprintf (cname, "plotColor%d", i);
-	    if (get_color_resource (w, cname, p) < 0 || *p == da_bgp)
-		*p = da_p;
+	    /* bounce around color wheel */
+	    h = binsplit (i);
+	    s = i < MAXPLTLINES/2 ? 1.0 : 0.5;
+	    v = 1.0;
+
+	    toRGB (h, s, v, &r, &g, &b);
+	    xc.red = (int)(65535 * r);
+	    xc.green = (int)(65535 * g);
+	    xc.blue = (int)(65535 * b);
+
+	    if (!XAllocColor (dsp, xe_cm, &xc))
+		xc.pixel = WhitePixel(dsp,DefaultScreen(dsp));
+	    plt_p[i] = xc.pixel;
 	}
 }
 
+/* return a unique number between 0 and 1 for every non-neg value of i.
+ * arrange for them to move along ever finer binary steps.
+ */
+static double
+binsplit (int i)
+{
+	/* 1-based */
+	double oneb = i + 1;
+
+	/* divide into base power of two parts */
+	double ndiv = pow (2.0, floor(log(oneb)/log(2.0)));
+
+	/* add remainder, divide last evenly on both ends */
+	return ((0.5 + oneb - ndiv)/ndiv);
+}
+
 static int
 x_ticks (asdate, minx, maxx, maxticks, ticks)
 int asdate;
@@ -315,7 +324,7 @@ double ticks[];
 	    /* put ticks on day boundaries */
 	    int nd, dt;
 
-	    nd = (int)(maxjd - minjd + 1);		/* period */
+	    nd = (int)(maxjd - minjd + 1);	/* period */
 	    dt = nd/maxticks + 1;		/* round up */
 	    nt = (nd+(dt-1))/dt + 1;		/* n ticks */
 	    jd = mjd_day (minjd);
@@ -349,7 +358,7 @@ int w;
 	    double jd;
 
 	    year_mjd (v, &jd);
-	    fs_date (buf, jd);
+	    fs_date (buf, pref_get(PREF_DATE_FORMAT), jd);
 	} else
 	    (void) sprintf (buf, "%g", v);
 	l = strlen(buf);
@@ -368,4 +377,4 @@ int w;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: plot_aux.c,v $ $Date: 2001/01/12 22:53:25 $ $Revision: 1.7 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: plot_aux.c,v $ $Date: 2006/04/10 09:00:06 $ $Revision: 1.11 $ $Name:  $"};
