@@ -4,10 +4,8 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <math.h>
-
-#if defined(__STDC__)
 #include <stdlib.h>
-#endif
+#include <unistd.h>
 
 #include <X11/Xlib.h>
 #include <Xm/Xm.h>
@@ -23,113 +21,95 @@
 #include <Xm/Separator.h>
 #include <Xm/TextF.h>
 
-#include "P_.h"
-#include "ps.h"
+#include "xephem.h"
+#include "lilxml.h"
 
-extern Widget	toplevel_w;
-#define	XtD	XtDisplay(toplevel_w)
-extern Colormap xe_cm;
+static void plt_select (int whether);
+static void plot_create_shell (void);
+static void plt_active_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_select_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_show_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_savecfg_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_loadcfg_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_popdown_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_close_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_help_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_undo_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_reset (void);
+static void plt_stop_selecting (void);
+static void plt_turn_off (void);
+static void set_x (char *name);
+static void set_y (char *name);
+static void add_field (char *name);
+static void init_row (void);
+static void plt_try_append (void);
+static void plt_try_overwrite (void);
+static void plt_try_cancel (void);
+static void plt_try_turn_on (void);
+static void plt_turn_on (char *how);
+static void plt_da_manage (char *fn);
+static void plt_da_destroy (Widget da_w);
+static void plt_print (void);
+static void plt_da_print_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_da_mloop_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_da_close_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_da_unmap_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_da_flipx_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_da_flipy_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_da_grid_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_da_xasdate_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_da_exp_cb (Widget da_w, XtPointer client, XtPointer call);
+static void plt_drawall (Widget da_w);
+static int plt_ano (double *sx, double *sy, int *xp, int *yp, int w2x,int arg);
 
-extern FILE *fopenh P_((char *name, char *how));
-extern char *getPrivateDir P_((void));
-extern char *getShareDir P_((void));
-extern char *syserrstr P_((void));
-extern int confirm P_((void));
-extern int existsh P_((char *filename));
-extern int isUp P_((Widget shell));
-extern int plot_cartesian P_((FILE *pfp, Widget widget, unsigned int nx,
-    unsigned int ny, int flipx, int flipy, int grid, int xasdate));
-extern void all_selection_mode P_((int whether));
-extern void defaultTextFN P_((Widget w, int setcols, char *x, char *y));
-extern void f_string P_((Widget w, char *s));
-extern void get_something P_((Widget w, char *resource, XtArgVal value));
-extern void get_xmstring P_((Widget w, char *resource, char **txtp));
-extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
-extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
-extern void query P_((Widget tw, char *msg, char *label1, char *label2,
-    char *label3, void (*func1)(void), void (*func2)(void),
-    void (*func3)(void)));
-extern void set_something P_((Widget w, char *resource, XtArgVal value));
-extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
-extern void sr_unreg P_((Widget w));
-extern void watch_cursor P_((int want));
-extern void wtip P_((Widget w, char *string));
-extern void xe_msg P_((char *msg, int app_modal));
-
-static void plt_select P_((int whether));
-static void plot_create_shell P_((void));
-static void plt_activate_cb P_((Widget w, XtPointer client, XtPointer call));
-static void plt_popdown_cb P_((Widget w, XtPointer client, XtPointer call));
-static void plt_browse_cb P_((Widget w, XtPointer client, XtPointer call));
-static void plt_fsbcancel_cb P_((Widget w, XtPointer client, XtPointer call));
-static void plt_fsbok_cb P_((Widget w, XtPointer client, XtPointer call));
-static void plt_close_cb P_((Widget w, XtPointer client, XtPointer call));
-static void plt_help_cb P_((Widget w, XtPointer client, XtPointer call));
-static void plt_reset P_((void));
-static void plt_stop_selecting P_((void));
-static void plt_turn_off P_((void));
-static void init_next_row P_((void));
-static void plt_try_append P_((void));
-static void plt_try_overwrite P_((void));
-static void plt_try_cancel P_((void));
-static void plt_try_turn_on P_((void));
-static void plt_turn_on P_((char *how));
-static void plt_da_manage P_((void));
-static void plt_da_destroy P_((Widget da_w));
-static void plt_print P_((void));
-static void plt_da_print_cb P_((Widget w, XtPointer client, XtPointer call));
-static void plt_da_close_cb P_((Widget w, XtPointer client, XtPointer call));
-static void plt_da_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
-static void plt_da_flipx_cb P_((Widget w, XtPointer client, XtPointer call));
-static void plt_da_flipy_cb P_((Widget w, XtPointer client, XtPointer call));
-static void plt_da_grid_cb P_((Widget w, XtPointer client, XtPointer call));
-static void plt_da_xasdate_cb P_((Widget w, XtPointer client, XtPointer call));
-static void plt_da_exp_cb P_((Widget da_w, XtPointer client, XtPointer call));
-static void plt_drawall P_((Widget da_w));
-
-
-/* max number of things we can keep track of at once to plot */
-#define	MAXPLTLINES	10
 #define	MAXFLDNAM	32	/* longest allowed field name */
 
+/* column indeces */
+typedef enum {
+    T=0		/* tag label text field */,
+    X		/* X label */,
+    Y		/* Y label */,
+    FORM	/* the form that holds the three above */,
+    NPC		/* number of columns */
+} PCols;
+
 static Widget plotshell_w;	/* main shell */
-static Widget fsb_w;		/* FSB */
-static Widget select_w, active_w, prompt_w;
-static Widget title_w, filename_w;
-static Widget table_w[MAXPLTLINES][4];	/* column indeces follow.. */
-#define	T	0		/* tag label text field */
-#define	X	1		/* X label */
-#define	Y	2		/* Y label */
-#define	FORM	3		/* the form that holds the three above */
+static Widget select_w;		/* TB for selecting plot fields */
+static Widget active_w;		/* TB for building plot file */
+static Widget prompt_w;		/* what to do next label */
+static Widget title_w;		/* TF for user's plot title */
+static Widget pfn_w;		/* TF for new plot file name */
+static Widget cfn_w;		/* TF for new config file name */
+static Widget undo_w;		/* PB for undoing a field selection */
 static int selecting_xy;	/* one of X or Y */
 
 static FILE *plt_fp;            /* the plot file; == 0 means don't plot */
 
 static Widget da_w_save;	/* used to set up for a print action */
-
-/* plt_activate_cb client values. */
-enum { SELECT, ACTIVE, SHOW};
+static char config_suffix[] = ".ptc";	/* file ext for plot config files */
+static char plot_suffix[] = ".plt";	/* file ext for plot files */
 
 static char plotcategory[] = "Tools -- Plot";	/* Save category */
+static char xml_plotele[] = "XEphemPlotConfig";
 
 /* store the name of each x and y line to track and their values.
- * we get the label straight from the Text widget in the table as needed.
+ * we get the tag straight from the Text widget in the table as needed.
  */
 typedef struct {
     char pl_xn[MAXFLDNAM];	/* name of x field, or 0 if none */
     double pl_xv;		/* last known value of x field */
     char pl_yn[MAXFLDNAM];	/* name of y field, or 0 if none */
     double pl_yv;		/* last known value of x field */
+    Widget c_w[NPC];		/* column displays */
 } PltLine;
 static PltLine pltlines[MAXPLTLINES];
-static int npltlines;		/* number of pltlines[] in actual use */
+static int npltlines;		/* number of completely defined plots */
 
 /* one of these gets malloced and passed to the drawing area expose callback via
  * its client parameter. be sure to free it when the parent FormDialog goes
  * away too.
  * by doing this, we can have lots of different plots up at once and yet we
- * don't have to keep track of them - they track themselves.
+ * don't have to keep track of them - they clean up after themselves.
  */
 typedef struct {
     char *filename;	/* name of file being plotted (also malloced) */
@@ -162,29 +142,10 @@ void
 plt_selection (name)
 char *name;
 {
-	Widget tw;
-
 	if (!isUp(plotshell_w) || !XmToggleButtonGetState(select_w))
 	    return;
 
-	tw = table_w[npltlines][selecting_xy];
-	set_xmstring (tw, XmNlabelString, name);
-	XtManageChild (tw);
-
-	if (selecting_xy == X) {
-	    (void) strncpy (pltlines[npltlines].pl_xn, name, MAXFLDNAM-1);
-	    selecting_xy = Y;
-	    XtManageChild (prompt_w);
-	    f_string (prompt_w, "Select quantity for Y..");
-	} else {
-	    (void) strncpy (pltlines[npltlines].pl_yn, name, MAXFLDNAM-1);
-	    if (++npltlines == MAXPLTLINES)
-		plt_stop_selecting();
-	    else {
-		selecting_xy = X;
-		init_next_row();
-	    }
-	}
+	add_field (name);
 }
 
 /* called as each different field is written -- just save in pltlines[]
@@ -217,8 +178,8 @@ plot()
 	    /* plot in order of original selection */
 	    PltLine *plp;
 	    for (plp = pltlines; plp < &pltlines[npltlines]; plp++) {
-		char *lbl = XmTextFieldGetString (table_w[plp-pltlines][T]);
-		(void) fprintf (plt_fp, "%c,%.12g,%.12g\n", lbl[0],
+		char *lbl = XmTextFieldGetString(pltlines[plp-pltlines].c_w[T]);
+		(void) fprintf (plt_fp, "%s,%.12g,%.12g\n", lbl,
 						    plp->pl_xv, plp->pl_yv);
 		XtFree (lbl);
 	    }
@@ -260,23 +221,12 @@ int whether;
 static void
 plot_create_shell()
 {
-	static struct {
-	    char *title;
-	    int cb_data;
-	    Widget *wp;
-	    char *tip;
-	} tbs[] = {
-	    {"Select fields", SELECT, &select_w,
-	    	"When on, data fields eligible for plotting are selectable buttons"},
-	    {"Create plot file", ACTIVE, &active_w,
-	    	"When on, selected fields are written to the named file at each main Update"},
-	    {"Show plot file", SHOW, NULL, "Display the named plot file"},
-	};
 	XmString str;
 	Widget f_w, rc_w;
-	Widget plotform_w;
-	Widget w;
+	Widget pf_w, oms_w, omc_w;
+	Widget w, cfg_w;
 	Arg args[20];
+	char *s[1];
 	int i, n;
 
 	/* create the main dialog */
@@ -289,6 +239,7 @@ plot_create_shell()
 	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
 	plotshell_w = XtCreatePopupShell ("Plot", topLevelShellWidgetClass,
 							toplevel_w, args, n);
+	setup_icon (plotshell_w);
 	set_something (plotshell_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (plotshell_w, XmNpopdownCallback, plt_popdown_cb, NULL);
 	sr_reg (plotshell_w, "XEphem*Plot.x", plotcategory, 0);
@@ -299,64 +250,110 @@ plot_create_shell()
 	XtSetArg (args[n], XmNmarginWidth, 10); n++;
 	XtSetArg (args[n], XmNverticalSpacing, 10); n++;
 	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_ANY); n++;
-	plotform_w = XmCreateForm (plotshell_w, "PlotF", args, n);
-	XtAddCallback (plotform_w, XmNhelpCallback, plt_help_cb, 0);
-	XtManageChild (plotform_w);
+	pf_w = XmCreateForm (plotshell_w, "PlotF", args, n);
+	XtAddCallback (pf_w, XmNhelpCallback, plt_help_cb, 0);
+	XtManageChild (pf_w);
 
-	/* make a RowColumn to hold everything */
+	/* control to allow selecting an existing plot file to display */
 
+	s[0] = plot_suffix;
+	oms_w = createFSM (pf_w, s, 1, "auxil", plt_show_cb);
+	wtip (oms_w, "Select a plot configuration file to load");
+	set_xmstring (oms_w, XmNlabelString, "Show plot file:     ");
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	XtSetValues (oms_w, args, n);
+
+	/* control to allow selecting a config file to load */
+
+	s[0] = config_suffix;
+	omc_w = createFSM (pf_w, s, 1, "auxil", plt_loadcfg_cb);
+	set_xmstring (omc_w, XmNlabelString, "Load config file:   ");
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, oms_w); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	XtSetValues (omc_w, args, n);
+
+	/* save config TB and its TF */
+
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, omc_w); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	cfg_w = XmCreateToggleButton(pf_w, "PSTB", args, n);
+	XtAddCallback(cfg_w, XmNvalueChangedCallback, plt_savecfg_cb, NULL);
+	set_xmstring (cfg_w, XmNlabelString, "Save config file: ");
+	wtip (cfg_w, "Save current plot definition to file");
+	XtManageChild (cfg_w);
+
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, omc_w); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNleftWidget, cfg_w); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	cfn_w = XmCreateTextField (pf_w, "ConfigFile", args, n);
+	defaultTextFN (cfn_w, 0, "myconfig.ptc", NULL);
+	wtip (cfn_w,
+		"Enter name of file in which to save current plot definition");
+	XtManageChild (cfn_w);
+	sr_reg (cfn_w, NULL, plotcategory, 0);
+
+	/* field Select PB */
+
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, cfn_w); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	select_w = XmCreateToggleButton(pf_w, "PSTB", args, n);
+	XtAddCallback(select_w, XmNvalueChangedCallback, plt_select_cb, NULL);
+	set_xmstring (select_w, XmNlabelString, "Select fields to plot");
+	wtip (select_w, "Build plot functions from selectable buttons");
+	XtManageChild (select_w);
+
+	/* create plot TB and its TF */
+
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, select_w); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	active_w = XmCreateToggleButton(pf_w, "PCTB", args, n);
+	XtAddCallback(active_w, XmNvalueChangedCallback, plt_active_cb, NULL);
+	set_xmstring (active_w, XmNlabelString, "Create plot file: ");
+	wtip (active_w,
+		"Selected fields are written to the named file at each Update");
+	XtManageChild (active_w);
+
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, select_w); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNleftWidget, active_w); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	pfn_w = XmCreateTextField (pf_w, "PlotFile", args, n);
+	defaultTextFN (pfn_w, 0, "myplot.plt", NULL);
+	wtip (pfn_w, "Enter name of plot file to create");
+	XtManageChild (pfn_w);
+	sr_reg (pfn_w, NULL, plotcategory, 0);
+
+	/* make a RowColumn to everything else */
+
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNtopWidget, pfn_w); n++;
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNspacing, 6); n++;
 	XtSetArg (args[n], XmNisAligned, False); n++;
 	XtSetArg (args[n], XmNadjustMargin, False); n++;
-	rc_w = XmCreateRowColumn(plotform_w, "PlotRC", args, n);
+	rc_w = XmCreateRowColumn(pf_w, "PlotRC", args, n);
 	XtManageChild (rc_w);
 
-	/* make the control toggle buttons */
-
-	for (i = 0; i < XtNumber(tbs); i++) {
-	    str = XmStringCreate(tbs[i].title, XmSTRING_DEFAULT_CHARSET);
-	    n = 0;
-	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
-	    XtSetArg (args[n], XmNlabelString, str); n++;
-	    w = XmCreateToggleButton(rc_w, "PlotTB", args, n);
-	    XmStringFree (str);
-	    XtAddCallback(w, XmNvalueChangedCallback, plt_activate_cb,
-						    (XtPointer)tbs[i].cb_data);
-	    if (tbs[i].wp)
-		*tbs[i].wp = w;
-	    if (tbs[i].tip)
-		wtip (w, tbs[i].tip);
-	    XtManageChild (w);
-	}
-
-	/* create filename text area and its label */
-
-	n = 0;
-	str = XmStringCreate("File name:", XmSTRING_DEFAULT_CHARSET);
-	XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
-	XtSetArg (args[n], XmNlabelString, str); n++;
-	w = XmCreateLabel (rc_w, "PlotFnL", args, n);
-	XmStringFree (str);
-	XtManageChild (w);
-
-	n = 0;
-	w = XmCreatePushButton (rc_w, "Browse...", args, n);
-	XtAddCallback (w, XmNactivateCallback, plt_browse_cb, NULL);
-	wtip (w, "Open a file selection box for hunting .plt files");
-	XtManageChild (w);
-
-	n = 0;
-	filename_w = XmCreateTextField (rc_w, "Filename", args, n);
-	defaultTextFN (filename_w, 1, getShareDir(), "auxil/analemma.plt");
-	wtip (filename_w, "Enter name of file to write");
-	XtManageChild (filename_w);
-	sr_reg (filename_w, NULL, plotcategory, 1);
-
 	/* create title text area and its label */
 
 	n = 0;
@@ -387,14 +384,13 @@ plot_create_shell()
 
 	for (i = -1; i < MAXPLTLINES; i++) {
 	    n = 0;
-	    XtSetArg (args[n], XmNfractionBase, 9); n++;
 	    f_w = XmCreateForm (rc_w, "PlotTF", args, n);
 
 	    /* save the form unless we are just making the column headings */
 	    if (i == -1)
 		XtManageChild (f_w);
 	    else
-		table_w[i][FORM] = f_w;
+		pltlines[i].c_w[FORM] = f_w;
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
@@ -402,51 +398,51 @@ plot_create_shell()
 	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNleftPosition, 0); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 1); n++;
+	    XtSetArg (args[n], XmNrightPosition, 20); n++;
 	    if (i == -1) {
 		XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 		w = XmCreateLabel (f_w, "Tag", args, n);
 		wtip (w, "Column of tags for each X/Y pair to plot");
 		XtManageChild (w);
 	    } else {
-		XtSetArg (args[n], XmNmaxLength, 1); n++;
-		XtSetArg (args[n], XmNcolumns, 1); n++;
-		table_w[i][T] = XmCreateTextField (f_w, "PlotTag", args, n);
-		XtManageChild (table_w[i][T]);
+		XtSetArg (args[n], XmNmaxLength, MAXTAG); n++;
+		XtSetArg (args[n], XmNcolumns, MAXTAG); n++;
+		pltlines[i].c_w[T] = XmCreateTextField (f_w, "PlotTag", args,n);
+		XtManageChild (pltlines[i].c_w[T]);
 	    }
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 2); n++;
+	    XtSetArg (args[n], XmNleftPosition, 30); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 5); n++;
+	    XtSetArg (args[n], XmNrightPosition, 65); n++;
 	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	    if (i == -1) {
 		w = XmCreateLabel (f_w, "_X_", args, n);
 		wtip (w, "Column of X fields for each X/Y pair to plot");
 		XtManageChild (w);
 	    } else {
-		table_w[i][X] = XmCreateLabel (f_w, "PlotX", args, n);
-		XtManageChild (table_w[i][X]);
+		pltlines[i].c_w[X] = XmCreateLabel (f_w, "PlotX", args, n);
+		XtManageChild (pltlines[i].c_w[X]);
 	    }
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 6); n++;
+	    XtSetArg (args[n], XmNleftPosition, 65); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 9); n++;
+	    XtSetArg (args[n], XmNrightPosition, 100); n++;
 	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	    if (i == -1) {
 		w = XmCreateLabel (f_w, "_Y_", args, n);
 		wtip (w, "Column of Y fields for each X/Y pair to plot");
 		XtManageChild (w);
 	    } else {
-		table_w[i][Y] = XmCreateLabel (f_w, "PlotY", args, n);
-		XtManageChild (table_w[i][Y]);
+		pltlines[i].c_w[Y] = XmCreateLabel (f_w, "PlotY", args, n);
+		XtManageChild (pltlines[i].c_w[Y]);
 	    }
 	}
 
@@ -459,7 +455,7 @@ plot_create_shell()
 	/* make a form to hold the close and help buttons evenly */
 
 	n = 0;
-	XtSetArg (args[n], XmNfractionBase, 7); n++;
+	XtSetArg (args[n], XmNfractionBase, 10); n++;
 	f_w = XmCreateForm (rc_w, "PlotCF", args, n);
 	XtManageChild(f_w);
 
@@ -482,121 +478,198 @@ plot_create_shell()
 	    XtSetArg (args[n], XmNleftPosition, 4); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNrightPosition, 6); n++;
+	    XtSetArg (args[n], XmNsensitive, False); n++;
+	    undo_w = XmCreatePushButton (f_w, "Undo", args, n);
+	    wtip (undo_w, "Undo last choice)");
+	    XtAddCallback (undo_w, XmNactivateCallback, plt_undo_cb, 0);
+	    XtManageChild (undo_w);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
+	    XtSetArg (args[n], XmNleftPosition, 7); n++;
+	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
+	    XtSetArg (args[n], XmNrightPosition, 9); n++;
 	    w = XmCreatePushButton (f_w, "Help", args, n);
 	    wtip (w, "More detailed usage information");
 	    XtAddCallback (w, XmNactivateCallback, plt_help_cb, 0);
 	    XtManageChild (w);
 }
 
-/* callback from any of the plot menu toggle buttons being activated.
+/* callback from one of the file names in the Show list.
+ * file name is label of this widget
  */
 /* ARGSUSED */
 static void
-plt_activate_cb (w, client, call)
+plt_show_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
 {
-	XmToggleButtonCallbackStruct *t = (XmToggleButtonCallbackStruct *) call;
-	int what = (int) client;
+	char *fn;
 
-	switch (what) {
-	case SELECT:
-	    if (t->set) {
-		/* first turn off plotting, if on, while we change things */
-		if (XmToggleButtonGetState(active_w))
-		    XmToggleButtonSetState(active_w, False, True);
-		plt_reset();	/* reset pltlines array and unmanage the table*/
-		plt_select(1);	/* inform other menus to inform us of fields */
-		init_next_row();/* set first tag to something and show it */
+	/* turn off plotting, if on, to make sure file is complete. */
+	if (XmToggleButtonGetState(active_w))
+	    XmToggleButtonSetState(active_w, False, True);
+
+	get_xmstring (w, XmNlabelString, &fn);
+	plt_da_manage(fn);
+	XtFree (fn);
+}
+
+/* callback from one of the config file names in load.
+ * file name is label of this widget
+ */
+/* ARGSUSED */
+static void
+plt_loadcfg_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	char buf[1024];
+	FILE *fp;
+	char *fn;
+
+	/* turn off plotting and selecting, if on */
+	if (XmToggleButtonGetState(active_w))
+	    XmToggleButtonSetState(active_w, False, True);
+	if (XmToggleButtonGetState(select_w))
+	    XmToggleButtonSetState(select_w, False, True);
+
+	get_xmstring (w, XmNlabelString, &fn);
+	fp = fopend (fn, NULL, "r");
+	if (fp) {
+	    LilXML *lp = newLilXML();
+	    XMLEle *pele = readXMLFile (fp, lp, buf);
+
+	    if (!pele)
+		xe_msg (1, "%s:\n%s", fn, buf);
+	    else if (strcmp (tagXMLEle(pele), xml_plotele)) {
+		xe_msg (1, "Not a valid plot config file");
+	    } else {
+		XMLEle *ep;
+
+		plt_reset();
+		init_row();	/* set first tag to something and show it */
 		selecting_xy = X;
-	    } else
+		for (ep= nextXMLEle(pele,1); ep != NULL; ep=nextXMLEle(pele,0)){
+		    if (strcmp (tagXMLEle(ep), "plot") == 0) {
+			XmTextFieldSetString (pltlines[npltlines].c_w[T],
+					findXMLAttValu (ep, "tag"));
+			add_field (findXMLAttValu (ep, "x"));
+			add_field (findXMLAttValu (ep, "y"));
+		    }
+		}
+
+		XmTextFieldSetString (title_w,
+					pcdataXMLEle(findXMLEle(pele,"title")));
 		plt_stop_selecting();
-	    break;
-	case ACTIVE:
-	    if (t->set) {
-		/* first turn off selecting, if on */
-		if (XmToggleButtonGetState(select_w))
-		    XmToggleButtonSetState(select_w, False, True);
-		plt_try_turn_on();
-	    } else
-		plt_turn_off();
-	    break;
-	case SHOW:
-	    /* turn off plotting, if on, to make sure file is complete. */
+	    }
+	    delXMLEle (pele);
+	    delLilXML (lp);
+	    fclose (fp);
+	}
+
+	XtFree (fn);
+}
+
+/* callback to save current configuration to file named by cfn_w. */
+/* ARGSUSED */
+static void
+plt_savecfg_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	FILE *fp;
+	char *txt, *fn;
+	char buf[1024];
+	int i;
+
+	/* ignore if turning off */
+	if (!XmToggleButtonGetState(w))
+	    return;
+
+	/* stop selecting if on */
+	if (XmToggleButtonGetState (select_w))
+	    XmToggleButtonSetState (select_w, False, True);
+
+	/* get config file name, add suffix if not already */
+	fn = txt = XmTextFieldGetString (cfn_w);
+	if (!strstr (txt, config_suffix)) {
+	    sprintf (fn = buf, "%s%s", txt, config_suffix);
+	    XmTextFieldSetString (cfn_w, fn);
+	}
+
+	/* save each entry */
+	fp = fopend (fn, NULL, "w");
+	if (fp) {
+	    char *title = XmTextFieldGetString (title_w);
+	    fprintf (fp, "<%s>\n", xml_plotele);
+	    fprintf (fp, "  <title>%s</title>\n", title);
+	    for (i = 0; i < npltlines; i++) {
+		PltLine *p = &pltlines[i];
+		char *tn = XmTextFieldGetString (p->c_w[T]);
+		fprintf (fp, "  <plot tag='%s' x='%s' y='%s' />\n", tn,
+							p->pl_xn, p->pl_yn);
+		XtFree (tn);
+	    }
+	    fprintf (fp, "</%s>\n", xml_plotele);
+	    XtFree (title);
+	    fclose (fp);
+	}
+
+	/* act like a push button - give chance to see change. */
+	if (confirm())
+	    xe_msg (1, "%s saved", fn);
+	XmToggleButtonSetState(w, False, True);
+
+	/* finished */
+	XtFree (txt);
+}
+
+/* callback to allow selecting plot fields */
+/* ARGSUSED */
+static void
+plt_select_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	if (XmToggleButtonGetState(w)) {
+	    /* first turn off plotting, if on, while we change things */
 	    if (XmToggleButtonGetState(active_w))
 		XmToggleButtonSetState(active_w, False, True);
-	    plt_da_manage();
-	    /* we want this to work like it was a pushbutton, really */
-	    XmToggleButtonSetState(w, False, False);
-	    break;
-	}
+	    plt_reset();	/* reset pltlines array and unmanage the table*/
+	    plt_select(1);	/* inform other menus to inform us of fields */
+	    init_row();		/* set first tag to something and show it */
+	    selecting_xy = X;
+	} else
+	    plt_stop_selecting();
 }
 
-/* callback from the FSB Cancel PB */
+/* callback when user wants to start building a plot file */
 /* ARGSUSED */
 static void
-plt_fsbcancel_cb (w, client, call)
+plt_active_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if (fsb_w)
-	    XtUnmanageChild (fsb_w);
-}
-
-/* callback from the FSB Ok PB */
-/* ARGSUSED */
-static void
-plt_fsbok_cb (w, client, call)
-Widget w;
-XtPointer client;
-XtPointer call;
-{
-	if (fsb_w) {
-	    char *fn;
-
-	    get_xmstring (fsb_w, XmNdirSpec, &fn);
-	    XmTextFieldSetString (filename_w, fn);
-	    XtFree (fn);
-	    XtUnmanageChild (fsb_w);
-	}
-}
-
-/* callback from the Browse PB */
-/* ARGSUSED */
-static void
-plt_browse_cb (w, client, call)
-Widget w;
-XtPointer client;
-XtPointer call;
-{
-	if (!fsb_w) {
-	    Widget w;
-	    Arg args[20];
-	    int n;
-
-	    n = 0;
-	    XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
-	    XtSetArg (args[n], XmNtitle, "Plot File browser"); n++;
-	    XtSetArg (args[n], XmNwidth, 400); n++;
-	    XtSetArg (args[n], XmNmarginHeight, 10); n++;
-	    XtSetArg (args[n], XmNmarginWidth, 10); n++;
-	    fsb_w = XmCreateFileSelectionDialog (toplevel_w, "PFSB", args, n);
-	    XtAddCallback (fsb_w, XmNmapCallback, prompt_map_cb, NULL);
-	    XtAddCallback (fsb_w, XmNokCallback, plt_fsbok_cb, NULL);
-	    XtAddCallback (fsb_w, XmNcancelCallback, plt_fsbcancel_cb, NULL);
-	    set_xmstring (fsb_w, XmNcancelLabelString, "Close");
-
-	    /* set default dir and pattern */
-	    set_xmstring (fsb_w, XmNdirectory, getPrivateDir());
-	    set_xmstring (fsb_w, XmNpattern, "*.plt");
-
-	    w = XmFileSelectionBoxGetChild (fsb_w, XmDIALOG_HELP_BUTTON);
-	    XtUnmanageChild (w);
-	}
-
-	XtManageChild (fsb_w);
+	if (XmToggleButtonGetState(w)) {
+	    /* first turn off selecting, if on */
+	    if (XmToggleButtonGetState(select_w))
+		XmToggleButtonSetState(select_w, False, True);
+	    if (npltlines > 0)
+		plt_try_turn_on();
+	    else {
+		xe_msg (1, "Please select at least one pair of values to plot");
+		XmToggleButtonSetState (w, False, False);
+	    }
+	} else
+	    plt_turn_off();
 }
 
 /* callback when the main plot window is closed */
@@ -607,8 +680,6 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if (fsb_w)
-	    XtUnmanageChild (fsb_w);
 }
 
 /* callback from the Close button.
@@ -640,6 +711,29 @@ static char *help_msg[] = {
 	hlp_dialog ("Plot", help_msg, sizeof(help_msg)/sizeof(help_msg[0]));
 }
 
+/* callback from the Undo button.
+ */
+/* ARGSUSED */
+static void
+plt_undo_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	if (selecting_xy == X) {
+	    if (npltlines > 0) {
+		/* delete row and previous y */
+		XtUnmanageChild (pltlines[npltlines--].c_w[FORM]);
+		set_y ("");
+		selecting_xy = Y;
+	    }
+	} else if (selecting_xy == Y) {
+	    /* clear x */
+	    set_x ("");
+	    selecting_xy = X;
+	}
+}
+
 /* forget our list, and unmanage the table.
  */
 static void
@@ -648,7 +742,7 @@ plt_reset()
 	int i;
 
 	for (i = 0; i < npltlines; i++)
-	    XtUnmanageChild (table_w[i][FORM]);
+	    XtUnmanageChild (pltlines[i].c_w[FORM]);
 
 	npltlines = 0;
 }
@@ -660,11 +754,12 @@ static void
 plt_stop_selecting()
 {
 	if (npltlines < MAXPLTLINES)
-	    XtUnmanageChild (table_w[npltlines][FORM]);
+	    XtUnmanageChild (pltlines[npltlines].c_w[FORM]);
 
 	XmToggleButtonSetState (select_w, False, False);
 	plt_select(0);
 	XtUnmanageChild (prompt_w);
+	XtSetSensitive (undo_w, False);
 }
 
 static void
@@ -676,21 +771,63 @@ plt_turn_off ()
 	}
 }
 
+/* set name as X coord on current row and change prompt to ask for Y */
 static void
-init_next_row()
+set_x (name)
+char *name;
+{
+	(void) strncpy (pltlines[npltlines].pl_xn, name, MAXFLDNAM-1);
+	set_xmstring (pltlines[npltlines].c_w[X], XmNlabelString, name);
+	set_xmstring (prompt_w, XmNlabelString, "Choose field for Y ..");
+	XtManageChild (prompt_w);
+}
+
+/* set name as Y coord on current row */
+static void
+set_y (name)
+char *name;
+{
+	(void) strncpy (pltlines[npltlines].pl_yn, name, MAXFLDNAM-1);
+	set_xmstring (pltlines[npltlines].c_w[Y], XmNlabelString, name);
+}
+
+/* init npltlines'th row and change prompt to ask for X */
+static void
+init_row ()
 {
 	char buf[100];
 
+	(void) sprintf (buf, "P%02d", npltlines+1);
+	XmTextFieldSetString (pltlines[npltlines].c_w[T], buf);
+
+	set_xmstring (prompt_w, XmNlabelString, "Choose field for X ..");
 	XtManageChild (prompt_w);
-	f_string (prompt_w, "Select quantity for X..");
 
-	(void) sprintf (buf, "%c", 'A' + npltlines);
-	XmTextFieldSetString (table_w[npltlines][T], buf);
+	set_xmstring (pltlines[npltlines].c_w[X], XmNlabelString, " ");
+	set_xmstring (pltlines[npltlines].c_w[Y], XmNlabelString, " ");
 
-	f_string (table_w[npltlines][X], " ");
-	f_string (table_w[npltlines][Y], " ");
+	XtManageChild (pltlines[npltlines].c_w[FORM]);
 
-	XtManageChild (table_w[npltlines][FORM]);
+	XtSetSensitive (undo_w, True);
+}
+
+/* add the next field to the new plot configuration being build */
+static void
+add_field (char *name)
+{
+	if (selecting_xy == X) {
+	    set_x (name);
+	    selecting_xy = Y;
+	} else {
+	    set_y (name);
+	    ++npltlines;
+	    if (npltlines == MAXPLTLINES)
+		plt_stop_selecting();
+	    else {
+		init_row();
+		selecting_xy = X;
+	    }
+	}
 }
 
 /* called from the query routine when want to append to an existing plot file.*/
@@ -723,14 +860,20 @@ plt_try_cancel()
 static void
 plt_try_turn_on()
 {
-	char *txt = XmTextFieldGetString (filename_w);
-	if (existsh (txt) == 0 && confirm()) {
-	    char *buf;
-	    buf = XtMalloc (strlen(txt)+100);
-	    (void) sprintf (buf, "%s exists:\nAppend or Overwrite?", txt);
-	    query (toplevel_w, buf, "Append", "Overwrite", "Cancel",
+	char *txt;
+	char buf[1024], *fn;
+
+	/* add suffix if not already */
+	fn = txt = XmTextFieldGetString (pfn_w);
+	if (!strstr (txt, plot_suffix))
+	    sprintf (fn = buf, "%s%s", txt, plot_suffix);
+
+	/* create, or modify if ok */
+	if (existd (fn,NULL) == 0 && confirm()) {
+	    char buf[256];
+	    (void) sprintf (buf, "%s exists:\nAppend or Overwrite?", fn);
+	    query (plotshell_w, buf, "Append", "Overwrite", "Cancel",
 			    plt_try_append, plt_try_overwrite, plt_try_cancel);
-	    XtFree (buf);
 	} else
 	    plt_try_overwrite();
 
@@ -744,19 +887,20 @@ static void
 plt_turn_on (how)
 char *how;	/* fopen how argument */
 {
+	char buf[1024], *fn;
 	char *txt;
 
-	/* plotting is on if file opens ok */
-	txt = XmTextFieldGetString (filename_w);
-	plt_fp = fopenh (txt, how);
-	if (!plt_fp) {
-	    char *buf;
-	    XmToggleButtonSetState (active_w, False, False);
-	    buf = XtMalloc (strlen(txt)+100);
-	    (void) sprintf (buf, "%s:\n%s", txt, syserrstr());
-	    xe_msg (buf, 1);
-	    XtFree (buf);
+	/* add suffix if not already */
+	fn = txt = XmTextFieldGetString (pfn_w);
+	if (!strstr (txt, plot_suffix)) {
+	    sprintf (fn = buf, "%s%s", txt, plot_suffix);
+	    XmTextFieldSetString (pfn_w, fn);
 	}
+
+	/* plotting is on if file opens ok */
+	plt_fp = fopend (fn, NULL, how);
+	if (!plt_fp)
+	    XmToggleButtonSetState (active_w, False, False);
 	XtFree (txt);
 	
 	if (plt_fp) {
@@ -779,7 +923,7 @@ char *how;	/* fopen how argument */
  *   or renamed.
  */
 static void
-plt_da_manage()
+plt_da_manage(char *fn)
 {
 	Widget daform_w;
 	Widget da_w, w;
@@ -790,17 +934,11 @@ plt_da_manage()
 	int n;
 	DrawInfo *di;
 	FILE *fp;
-	char *fn;
 
 	/* first make sure we can open the plot file */
-	fn = XmTextFieldGetString (filename_w);
-	fp = fopenh (fn, "r");
+	fn = XtNewString (fn);
+	fp = fopend (fn, "auxil", "r");
 	if (!fp) {
-	    char *buf;
-	    buf = XtMalloc (strlen(fn)+100);
-	    (void) sprintf (buf, "%s:\n%s", fn, syserrstr());
-	    xe_msg (buf, 1);
-	    XtFree (buf);
 	    XtFree (fn);
 	    return;
 	}
@@ -843,7 +981,7 @@ plt_da_manage()
 	 * N.B. be sure this guys parent is the FormDialog so exp_cb can find
 	 *   the DrawInfo by looking there at its userData.
 	 * make this as high as it is wide when it is first mapped.
-	 * N.B. if ever want this is a frame beware that other functions
+	 * N.B. if ever want this in a frame beware that other functions
 	 *   assume that the daform_w is the parent of the DrawingArea.
 	 */
 	n = 0;
@@ -855,7 +993,7 @@ plt_da_manage()
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNmarginWidth, 0); n++;
 	XtSetArg (args[n], XmNmarginHeight, 0); n++;
-	da_w = XmCreateDrawingArea (daform_w, "PlotDA", args, n);
+	da_w = XmCreateDrawingArea (daform_w, "PlotMap", args, n);
 	XtAddCallback (da_w, XmNexposeCallback, plt_da_exp_cb, NULL);
 	XtManageChild (da_w);
 
@@ -883,6 +1021,23 @@ plt_da_manage()
 	    wtip (w, "Print this plot");
 	    XtManageChild (w);
 
+	    /* make the annot button */
+	    n = 0;
+	    w = XmCreatePushButton (pd_w, "Ann", args, n);
+	    XtAddCallback (w, XmNactivateCallback, ano_cb, NULL);
+	    set_xmstring (w, XmNlabelString, "User annotation...");
+	    wtip (w, "Open window to create and manage your own annotation");
+	    XtManageChild (w);
+
+	    /* make the movie loop button */
+	    n = 0;
+	    n += ml_addacc (args, n);
+	    w = XmCreatePushButton (pd_w, "ML", args, n);
+	    XtAddCallback (w, XmNactivateCallback, plt_da_mloop_cb, da_w);
+	    set_xmstring (w, XmNlabelString, "Add to movie...");
+	    wtip (w, "Add this plot to the movie loop");
+	    XtManageChild (w);
+
 	    /* add a separator */
 	    n = 0;
 	    w = XmCreateSeparator (pd_w, "Sep", args, n);
@@ -1037,7 +1192,7 @@ plt_print()
 	    XPSClose();
 	    da_w_save = 0;
 	} else {
-	    xe_msg ("Can not print after plot has been dismissed.", 1);
+	    xe_msg (1, "Can not print after plot has been dismissed.");
 	    XPSClose();
 	}
 }
@@ -1058,6 +1213,21 @@ XtPointer call;
 	XPSAsk ("Plot", plt_print);
 }
 
+/* called to add plot to the movie loop.
+ * use the window of the da_w (passed in client)
+ */
+/* ARGSUSED */
+static void
+plt_da_mloop_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	Widget da_w = (Widget)client;
+	XmUpdateDisplay (da_w);		/* handle expose after pulldown menu */
+	ml_add (XtWindow(da_w), NULL);
+}
+
 /* called when the Close button is pushed on a plot.
  * just unmap the dialog (passed as client).
  */
@@ -1274,7 +1444,7 @@ XtPointer call;
 	    }
 	default:
 	    printf ("Unexpected da_w event. type=%d\n", c->reason);
-	    exit(1);
+	    abort();
 	}
 
 	/* draw the plot from scratch */
@@ -1305,15 +1475,31 @@ Widget da_w;
 							    di->xasdate) < 0) {
 	    /* had trouble, so done with this FormDialog.
 	     */
-	    char buf[128];
-	    (void) sprintf (buf, "Error plotting `%.*s'\n", (int)sizeof(buf)-20,
-								di->filename);
-	    xe_msg (buf, 0);
+	    xe_msg (0, "Error plotting `%s'", di->filename);
 	    plt_da_destroy (da_w);
 	}
 
+	/* user annotation */
+	ano_draw (da_w, XtWindow(da_w), plt_ano, 0);
+
 	watch_cursor (0);
 }
 
+/* support simple pixel based annotation
+ */
+static int
+plt_ano (double *sx, double *sy, int *xp, int *yp, int w2x, int arg)
+{
+	if (w2x) {
+	    *xp = (int)*sx;
+	    *yp = (int)*sy;
+	} else {
+	    *sx = (double)*xp;
+	    *sy = (double)*yp;
+	}
+
+	return (1);
+}
+
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: plotmenu.c,v $ $Date: 2000/12/03 08:33:06 $ $Revision: 1.13 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: plotmenu.c,v $ $Date: 2005/11/14 03:55:44 $ $Revision: 1.37 $ $Name:  $"};
