@@ -6,9 +6,9 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <math.h>
-#if defined(__STDC__)
 #include <stdlib.h>
-#endif
+#include <unistd.h>
+
 #include <X11/Xlib.h>
 #include <Xm/Xm.h>
 #include <Xm/Form.h>
@@ -20,84 +20,76 @@
 #include <Xm/SelectioB.h>
 #include <Xm/ToggleB.h>
 #include <Xm/Text.h>
+#include <Xm/TextF.h>
 
-#include "P_.h"
-#include "astro.h"
-#include "circum.h"
+#include "xephem.h"
+#include "lilxml.h"
 
-extern Widget	toplevel_w;
-#define	XtD	XtDisplay(toplevel_w)
-extern Colormap xe_cm;
-
-extern Now *mm_get_now P_((void));
-extern int any_ison P_((void));
-extern int compile_expr P_((char *ex, char *errbuf));
-extern int execute_expr P_((double *vp, char *errbuf));
-extern int isUp P_((Widget shell));
-extern int prog_isgood P_((void));
-extern void all_selection_mode P_((int whether));
-extern void buttonAsButton P_((Widget w, int whether));
-extern void compiler_log P_((char *name, double value));
-extern void f_double P_((Widget w, char *fmt, double f));
-extern void f_scansex P_((double o, char bp[], double *np));
-extern void f_showit P_((Widget w, char *str));
-extern void f_time P_((Widget w, double t));
-extern void get_xmstring P_((Widget w, char *resource, char **txtp));
-extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
-extern void lst_selection P_((char *name));
-extern void plt_selection P_((char *name));
-extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
-extern void redraw_screen P_((int how_much));
-extern void set_something P_((Widget w, char *resource, XtArgVal value));
-extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
-extern void wtip P_((Widget w, char *string));
-extern void xe_msg P_((char *msg, int app_modal));
-
-static void srch_create_shell P_((void));
-static int srch_isup P_((void));
-static int srching_now P_((void));
-static void srch_set_buttons P_((int whether));
-static void srch_use_cb P_((Widget w, XtPointer client, XtPointer call));
-static void srch_newfunc_cb P_((Widget w, XtPointer client, XtPointer call));
-static void srch_compile_cb P_((Widget w, XtPointer client, XtPointer call));
-static void srch_help_cb P_((Widget w, XtPointer client, XtPointer call));
-static void srch_clear_cb P_((Widget w, XtPointer client, XtPointer call));
-static void srch_fields_cb P_((Widget w, XtPointer client, XtPointer call));
-static void srch_close_cb P_((Widget w, XtPointer client, XtPointer call));
-static void srch_acc_ok_cb P_((Widget w, XtPointer client, XtPointer call));
-static void srch_acc_cb P_((Widget wid, XtPointer client, XtPointer call));
-static void srch_goal_cb P_((Widget w, XtPointer client, XtPointer call));
-static void srch_on_off_cb P_((Widget w, XtPointer client, XtPointer call));
-static int srch_minmax P_((double Mjd, double v, double *tmincp));
-static int srch_solve0 P_((double Mjd, double v, double *tmincp));
-static int srch_binary P_((double Mjd, double v, double *tmincp));
+static void srch_create_shell (void);
+static int srch_isup (void);
+static void get_tmlimit(void);
+static int srching_now (void);
+static void compile_func (void);
+static void srch_set_buttons (int whether);
+static void srch_use_cb (Widget w, XtPointer client, XtPointer call);
+static void srch_loadfn_cb (Widget w, XtPointer client, XtPointer call);
+static void srch_savefn_cb (Widget w, XtPointer client, XtPointer call);
+static void srch_newfunc_cb (Widget w, XtPointer client, XtPointer call);
+static void srch_compile_cb (Widget w, XtPointer client, XtPointer call);
+static void srch_help_cb (Widget w, XtPointer client, XtPointer call);
+static void srch_clear_cb (Widget w, XtPointer client, XtPointer call);
+static void srch_fields_cb (Widget w, XtPointer client, XtPointer call);
+static void srch_close_cb (Widget w, XtPointer client, XtPointer call);
+static void srch_acc_cb (Widget wid, XtPointer client, XtPointer call);
+static void srch_goal_cb (Widget w, XtPointer client, XtPointer call);
+static void srch_on_off_cb (Widget w, XtPointer client, XtPointer call);
+static int srch_min (double Mjd, double v, double *tmincp);
+static int srch_max (double Mjd, double v, double *tmincp);
+static int srch_solve0 (double Mjd, double v, double *tmincp);
+static int srch_binary (double Mjd, double v, double *tmincp);
 
 
 /* the widgets we need direct access to */
-static Widget srchshell_w;
-static Widget help_w;
-static Widget acc_w;
-static Widget field_w;
-static Widget func_w;	/* contains the search function [to be] compiled */
-static Widget err_w;
-static Widget compile_w;
-static Widget valu_w;	/* used to display most recent calulated value */
-static Widget on_w;	/* whether searching is active comes directly from this
-			 * widget's XmToggleButtonGetState()
-			 */
+static Widget srchshell_w;	/* main shell */
+static Widget acc_w;		/* TF to enter desired search accuracy */
+static Widget field_w;		/* TB whether to set buttons to define func */
+static Widget func_w;		/* T contains the search function */
+static Widget err_w;		/* label to show compile errors */
+static Widget valu_w;		/* label to display calulated value */
+static Widget on_w;		/* TB whether searching is active */
+static Widget save_w;		/* TF naming file to save */
 
 /* name is it appears when valu_w is selected for plotting or lising */
 static char srchvname[] = "SolveValue";
 static char solvecategory[] = "Tools -- Solve";	/* Save category */
+static char xml_srchele[] = "XEphemSolver";
 
 static int (*srch_f)();		/* 0 or pointer to one of the search functions*/
 static int srch_tmscalled;	/* number of iterations so far */
 static double tmlimit;		/* search accuracy, hrs */
+static char config_suffix[] = ".svc";	/* file ext for solver config files */
 
 static int srch_selecting;	/* whether our value is currently selectable */
 static int srch_self_selection_mode;	/* flag to prevent self-selection */
 
+typedef struct {
+    char *title;
+    char *name;
+    XtPointer cb_data;
+    char *tip;
+    Widget w;
+} Goal;
+static Goal goals[] = {
+    {"Find Maximum", "FindMax", (XtPointer)srch_max,
+	"Solve for a time at which the Function experiences a local maxima"},
+    {"Find Minimum", "FindMin", (XtPointer)srch_min,
+	"Solve for a time at which the Function experiences a local minima"},
+    {"Find 0", "Find0", (XtPointer)srch_solve0,
+	"Solve for a time at which the Function evaluates to 0"},
+    {"Binary", "Binary", (XtPointer)srch_binary,
+	"Step time along until the truth value of the Function changes"},
+};
+
 /* called when the search menu is activated via the main menu pulldown.
  * if never called before, create and manage all the widgets as a child of a
  * form. otherwise, just go for it.
@@ -188,14 +180,13 @@ double *tmincp;
 	    if (s == 0) {
 		f_double (valu_w, "%g", v);
 		if (srching_now()) {
+		    xe_msg (0, "Solver step %3d @ %13.5f %g", srch_tmscalled,
+		    						Mjd+MJD0, v);
 		    s = (*srch_f)(Mjd, v, tmincp);
 		    srch_tmscalled++;
 		}
 	    } else {
-		char srchmsg[256];
-		(void) sprintf (srchmsg, "Search evaluation error: %.200s",
-								    errbuf);
-		xe_msg (srchmsg, 0);
+		xe_msg (0, "Solver evaluation error: %.200s", errbuf);
 	    }
 	} else
 	    s = 0;
@@ -250,23 +241,13 @@ Cursor c;
 static void
 srch_create_shell()
 {
-	static struct {
-	    char *title;
-	    XtPointer cb_data;
-	    char *tip;
-	} rbtb[] = {
-	    {"Find Extreme", (XtPointer)srch_minmax,
-	    	"Solve for a time at which Function experiences a local minima or maxima"},
-	    {"Find 0", (XtPointer)srch_solve0,
-	    	"Solve for a time at which the Function evaluates to 0"},
-	    {"Binary", (XtPointer)srch_binary,
-	    	"Step time along until the truth value of the Function changes"},
-	};
 	XmString str;
-	char *txt;
 	Widget w, trc_w, brc_w, f_w, rb_w;
+	Widget oms_w, compile_w; 
+	Widget help_w;
 	Widget srchform_w;
 	Arg args[20];
+	char *s[1];
 	int i, n;
 
 	/* create form dialog */
@@ -278,6 +259,7 @@ srch_create_shell()
 	XtSetArg (args[n], XmNtitle, "xephem Solver Control"); n++;
 	srchshell_w = XtCreatePopupShell ("Solve", topLevelShellWidgetClass,
 							toplevel_w, args, n);
+	setup_icon (srchshell_w);
 	set_something (srchshell_w, XmNcolormap, (XtArgVal)xe_cm);
 	sr_reg (srchshell_w, "XEphem*Solve.x", solvecategory, 0);
 	sr_reg (srchshell_w, "XEphem*Solve.y", solvecategory, 0);
@@ -389,6 +371,46 @@ srch_create_shell()
 	w = XmCreateSeparator (brc_w, "Sep2", args, n);
 	XtManageChild (w);
 
+	/* config file load */
+
+	s[0] = config_suffix;
+	oms_w = createFSM (brc_w, s, 1, "auxil", srch_loadfn_cb);
+	wtip (oms_w, "Select existing plot configuration file to load");
+	set_xmstring (oms_w, XmNlabelString, "Load file: ");
+	n = 0;
+
+	/* config file save */
+
+	n = 0;
+	f_w = XmCreateForm (brc_w, "CFF", args, n);
+	XtManageChild (f_w);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	    w = XmCreatePushButton (f_w, "CFL", args, n);
+	    XtAddCallback (w, XmNactivateCallback, srch_savefn_cb, NULL);
+	    wtip (w,"Save search function and settings to file named at right");
+	    set_xmstring (w, XmNlabelString, "Save to:");
+	    XtManageChild (w);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNleftWidget, w); n++;
+	    XtSetArg (args[n], XmNleftOffset, 5); n++;
+	    save_w = XmCreateTextField (f_w, "ConfigFile", args, n);
+	    sr_reg (save_w, NULL, solvecategory, 0);
+	    defaultTextFN (save_w, 0, "mysolver.svc", NULL);
+	    XtAddCallback (save_w, XmNactivateCallback, srch_savefn_cb, NULL);
+	    wtip (save_w, "File name in which to save search function");
+	    XtManageChild (save_w);
+
+	/* create a separator */
+
+	n = 0;
+	w = XmCreateSeparator (brc_w, "Sep2", args, n);
+	XtManageChild (w);
+
 	/* create goal radio box and its toggle buttons */
 
 	str = XmStringCreate ("Goal:", XmSTRING_DEFAULT_CHARSET);
@@ -404,17 +426,19 @@ srch_create_shell()
 	rb_w = XmCreateRadioBox (brc_w, "SrchGoalRadioBox", args, n);
 	XtManageChild (rb_w);
 
-	    for (i = 0; i < XtNumber(rbtb); i++) {
+	    for (i = 0; i < XtNumber(goals); i++) {
 		n = 0;
-		str = XmStringCreate(rbtb[i].title, XmSTRING_DEFAULT_CHARSET);
+		str = XmStringCreate(goals[i].title, XmSTRING_DEFAULT_CHARSET);
 		XtSetArg (args[n], XmNlabelString, str); n++;
-		w = XmCreateToggleButton (rb_w, "SrchRBTB", args, n);
+		w = XmCreateToggleButton (rb_w, goals[i].name, args, n);
 		XtAddCallback (w, XmNvalueChangedCallback, srch_goal_cb,
-						    (XtPointer)rbtb[i].cb_data);
-		if (rbtb[i].tip)
-		    wtip (w, rbtb[i].tip);
-		XtManageChild (w);
+						(XtPointer)goals[i].cb_data);
 		XmStringFree(str);
+		if (goals[i].tip)
+		    wtip (w, goals[i].tip);
+		XtManageChild (w);
+		goals[i].w = w;
+		sr_reg (w, NULL, solvecategory, 1);
 	    }
 
 	/* create a separator */
@@ -433,15 +457,13 @@ srch_create_shell()
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	    acc_w = XmCreatePushButton (f_w, "Accuracy", args, n);
+	    XtSetArg (args[n], XmNcolumns, 8); n++;
+	    acc_w = XmCreateTextField (f_w, "Accuracy", args, n);
 	    XtAddCallback (acc_w, XmNactivateCallback, srch_acc_cb, 0);
-	    wtip (acc_w, "Set desired time accuracy of solution");
+	    wtip (acc_w, "Set desired time accuracy of solution, hours");
 	    XtManageChild (acc_w);
 	    sr_reg (acc_w, NULL, solvecategory, 1);
-
-	    get_xmstring (acc_w, XmNlabelString, &txt);
-	    f_scansex (0.0, txt, &tmlimit);
-	    XtFree (txt);
+	    get_tmlimit();
 
 	    str = XmStringCreate("Desired accuracy:", XmSTRING_DEFAULT_CHARSET);
 	    n = 0;
@@ -546,6 +568,15 @@ srch_create_shell()
 	sr_reg (func_w, NULL, solvecategory, 1);
 }
 
+/* set tmlimit from user's widget */
+static void
+get_tmlimit()
+{
+	char *str = XmTextFieldGetString (acc_w);
+	f_scansexa (str, &tmlimit);
+	XtFree (str);
+}
+
 /* return True whenever the srch control menu is up */
 static int
 srch_isup()
@@ -583,15 +614,12 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	static char msg[] =
-	    "You must first successfully compile a search\nfunction before value may be selected.";
-
 	if (srch_selecting) {
 	    if (prog_isgood()) {
 		plt_selection (srchvname);
 		lst_selection (srchvname);
 	    } else
-		xe_msg (msg, 0);
+		xe_msg (0, "You must first successfully compile a search\nfunction before value may be selected.");
 	}
 }
 
@@ -607,6 +635,118 @@ XtPointer call;
 	f_showit (err_w, "New function not compiled");
 }
 
+/* callback to load a function from a file.
+ * file name is label of this widget.
+ */
+/* ARGSUSED */
+static void
+srch_loadfn_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	char msg[1024];
+	char *g, *f, *a;
+	char *fn;
+	FILE *fp;
+
+	get_xmstring (w, XmNlabelString, &fn);
+	fp = fopend (fn, NULL, "r");
+	if (fp) {
+	    LilXML *lp = newLilXML ();
+	    XMLEle *sele = readXMLFile (fp, lp, msg);
+	    int i;
+
+	    if (!sele)
+		xe_msg (1, "%s", msg);
+	    else if (strcmp (tagXMLEle(sele), xml_srchele)
+	    		|| !(g = pcdataXMLEle (findXMLEle (sele, "goal")))
+	    		|| !(f = pcdataXMLEle (findXMLEle (sele, "function")))
+	    		|| !(a = pcdataXMLEle (findXMLEle (sele, "accuracy"))))
+		xe_msg (1, "Not a valid solver file");
+	    else {
+		for (i = 0; i < XtNumber(goals); i++) {
+		    if (!strcmp(goals[i].title, g)) {
+			XmToggleButtonSetState (goals[i].w, True, True);
+			break;
+		    }
+		}
+		if (i == XtNumber(goals)) {
+		    xe_msg (1, "Unknown goal: %s", g);
+		} else {
+		    /* ok, all looks good */
+		    XmTextSetString (func_w, f);
+		    XmTextFieldSetString (acc_w, a);
+		    get_tmlimit();
+		    compile_func();
+		}
+	    }
+
+	    delXMLEle (sele);
+	    delLilXML (lp);
+	    fclose (fp);
+	}
+
+	XtFree (fn);
+}
+
+/* save the current search function to a file.
+ * N.B. don't use call, used by both TF and PB 
+ */
+/* ARGSUSED */
+static void
+srch_savefn_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	char buf[1024], *txt;
+	char *goal;
+	char *fn;
+	FILE *fp;
+	int i;
+
+	/* find current goal setting */
+	goal = NULL;
+	for (i = 0; i < XtNumber(goals); i++) {
+	    if (XmToggleButtonGetState(goals[i].w)) {
+		goal = goals[i].title;
+		break;
+	    }
+	}
+	if (!goal) {
+	    xe_msg (1, "Goal must be selected before saving");
+	    return;
+	}
+
+	/* get file name */
+	fn = txt = XmTextFieldGetString (save_w);
+	if (!strstr(txt, config_suffix)) {
+	    sprintf (fn = buf, "%s%s", txt, config_suffix);
+	    XmTextFieldSetString (save_w, fn);
+	}
+
+	/* create file */
+	fp = fopend (fn, NULL, "w");
+	if (fp) {
+	    char *func = XmTextGetString (func_w);
+	    char *acc = XmTextFieldGetString (acc_w);
+	    fprintf (fp, "<%s>\n", xml_srchele);
+		fprintf (fp, "  <function>%s</function>\n", entityXML(func));
+		fprintf (fp, "  <accuracy>%s</accuracy>\n", acc);
+		fprintf (fp, "  <goal>%s</goal>\n", goal);
+	    fprintf (fp, "</%s>\n", xml_srchele);
+	    XtFree (func);
+	    XtFree (acc);
+	    fclose (fp);
+	}
+
+	/* clean up */
+	if (confirm())
+	    xe_msg (1, "%s saved", fn);
+	XtFree (txt);
+}
+
 /* callback from the compile button.
  */
 /* ARGSUSED */
@@ -616,30 +756,7 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	char *newexp;
-	char errbuf[256];
-
-	newexp = XmTextGetString (func_w);
-	if (compile_expr (newexp, errbuf) <  0)
-	    set_xmstring (err_w, XmNlabelString, errbuf);
-	else
-	    set_xmstring (err_w, XmNlabelString, "No compile errors");
-
-	XtFree (newexp);
-
-	/* as a courtesy, turn off searching if it's currently active */
-	if (XmToggleButtonGetState(on_w))
-	    XmToggleButtonSetState(on_w, False, True/*invoke cb*/);
-
-	/* compute the new function.
-	 * must update everything else first because our function uses that
-	 *   stuff in all likelihood.
-	 */
-	if (prog_isgood()) {
-	    Now *np = mm_get_now();
-	    redraw_screen(1);
-	    (void) srch_eval (mjd, (double *)NULL);
-	}
+	compile_func();
 }
 
 /* callback from the help button.
@@ -714,71 +831,15 @@ XtPointer call;
 	XtPopdown (srchshell_w);
 }
 
-/* user typed OK to the accuracy prompt. get his new value and use it */
+/* user typed Return in accuracy field. get his new value and use it */
 /* ARGSUSED */
 static void
-srch_acc_ok_cb (w, client, call)
+srch_acc_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
 {
-	char *text;
-	
-	get_xmstring(w, XmNtextString, &text);
-	f_scansex (tmlimit, text, &tmlimit);
-	XtFree (text);
-	f_time (acc_w, tmlimit);
-	XtUnmanageChild (w);
-}
-
-/* callback from the accuracy-change-request pushbutton.
- * put up a prompt dialog to ask for a new value.
- */
-/* ARGSUSED */
-static void
-srch_acc_cb (wid, client, call)
-Widget wid;
-XtPointer client;
-XtPointer call;
-{
-	static Widget dw;
-	Widget w;
-	
-	if (!dw) {
-	    XmString str, title;
-	    Arg args[20];
-	    int n;
-
-	    str = XmStringCreate ("Accuracy (h:m:s):",
-						    XmSTRING_DEFAULT_CHARSET);
-	    title = XmStringCreate ("xephem Solver Accuracy",
-						    XmSTRING_DEFAULT_CHARSET);
-	    n = 0;
-	    XtSetArg(args[n], XmNselectionLabelString, str);  n++;
-	    XtSetArg(args[n], XmNdialogTitle, title);  n++;
-	    XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	    XtSetArg(args[n], XmNmarginWidth, 10);  n++;
-	    XtSetArg(args[n], XmNmarginHeight, 10);  n++;
-	    dw = XmCreatePromptDialog(toplevel_w, "Accuracy", args, n);
-	    set_something (dw, XmNcolormap, (XtArgVal)xe_cm);
-	    XtAddCallback (dw, XmNokCallback, srch_acc_ok_cb, NULL);
-	    XtAddCallback (dw, XmNmapCallback, prompt_map_cb, NULL);
-	    XmStringFree (str);
-	    XmStringFree (title);
-
-	    w = XmSelectionBoxGetChild (dw, XmDIALOG_HELP_BUTTON);
-	    XtUnmanageChild (w);
-	}
-
-	set_xmstring(dw, XmNtextString, "");
-
-	XtManageChild (dw);
-
-#if XmVersion >= 1001
-	w = XmSelectionBoxGetChild (dw, XmDIALOG_TEXT);
-	XmProcessTraversal (w, XmTRAVERSE_CURRENT);
-	XmProcessTraversal (w, XmTRAVERSE_CURRENT); /* yes, twice!! */
-#endif
+	get_tmlimit();
 }
 
 /* callback from the search goal selection radio buttons.
@@ -820,108 +881,145 @@ XtPointer call;
 	     */
 	    if (!prog_isgood()) {
 		XmToggleButtonSetState(on_w, False, True/* invoke cb */);
-		xe_msg (msg, 1);
+		xe_msg (1, "%s", msg);
 		 
 	    } else if (srch_f == 0) {
 		XmToggleButtonSetState(on_w, False, True/* invoke cb */);
-		xe_msg (msg, 1);
+		xe_msg (1, "%s", msg);
 	    } else {
 		srch_tmscalled = 0;
 		/* turning on searhing so as a courtesy turn off the
 		 * field selection mechanism if it's on now.
+		 * get fresh desired acc.
 		 */
 		if (XmToggleButtonGetState(field_w))
 		    XmToggleButtonSetState(field_w, False, True);
+		get_tmlimit();
 	    }
 	}
 }
 
-/* use successive paraboloidal fits to find when expression is at a
- * local minimum or maximum.
+static void
+compile_func()
+{
+	char *newexp;
+	char errbuf[256];
+
+	newexp = XmTextGetString (func_w);
+	if (compile_expr (newexp, errbuf) <  0)
+	    set_xmstring (err_w, XmNlabelString, errbuf);
+	else
+	    set_xmstring (err_w, XmNlabelString, "Function compiled successfully");
+
+	XtFree (newexp);
+
+	/* as a courtesy, turn off searching if it's currently active */
+	if (XmToggleButtonGetState(on_w))
+	    XmToggleButtonSetState(on_w, False, True/*invoke cb*/);
+
+	/* compute the new function.
+	 * must update everything else first because our function uses that
+	 *   stuff in all likelihood.
+	 */
+	if (prog_isgood()) {
+	    Now *np = mm_get_now();
+	    redraw_screen(1);
+	    (void) srch_eval (mjd, (double *)NULL);
+	}
+}
+
+/* search for time when expression is at a local maximum.
+ * uses globals srch_tmscalled and tmlimit.
+ * return -1 when finished.
  */
 static int
-srch_minmax(Mjd, v, tmincp)
+srch_max(Mjd, v, tmincp)
 double Mjd;
 double v;
 double *tmincp;
 {
-	static double base;		/* for better stability */
-	static double x_1, x_2, x_3;	/* keep in increasing order */
-	static double y_1, y_2, y_3;
-	double xm, a, b;
+	static double v0, lastv0, j0, v1, lastv1, j1;
+	static double lastv;
 
+	/* save first time and keep going */
 	if (srch_tmscalled == 0) {
-	    base = Mjd;
-	    x_1 = 0.0;
-	    y_1 = v;
+	    j0 = -1e100;
+	    j1 =  1e100;
+	    lastv = v;
 	    return (0);
 	}
-	Mjd -= base;
-	if (srch_tmscalled == 1) {
-	    /* put in one of first two slots */
-	    if (Mjd < x_1) {
-	        x_2 = x_1;  y_2 = y_1;
-		x_1 = Mjd; y_1 = v;
-	    } else {
-		x_2 = Mjd; y_2 = v;
-	    }
-	    return (0);
-	}
-	if (srch_tmscalled == 2 || fabs(Mjd - x_1) < fabs(Mjd - x_3)) {
-	    /* closer to x_1 so discard x_3.
-	     * or if it's our third value we know to "discard" x_3.
-	     */
-	    if (Mjd > x_2) {
-		x_3 = Mjd; y_3 = v;
-	    } else {
-		x_3 = x_2;  y_3 = y_2;
-		if (Mjd > x_1) {
-		    x_2 = Mjd; y_2 = v;
-		} else {
-		    x_2 = x_1;  y_2 = y_1;
-		    x_1 = Mjd; y_1 = v;
-		}
-	    }
-	    if (srch_tmscalled == 2)
-		return (0);
-	} else {
-	    /* closer to x_3 so discard x_1 */
-	    if (Mjd < x_2) {
-		x_1 = Mjd;  y_1 = v;
-	    } else {
-		x_1 =  x_2;  y_1 = y_2;
-		if (Mjd < x_3) {
-		    x_2 = Mjd; y_2 = v;
-		} else {
-		    x_2 =  x_3; y_2 = y_3;
-		    x_3 = Mjd; y_3 = v;
-		}
-	    }
-	}
 
-#ifdef TRACEMM
-	{ char buf[NC];
-	  (void) sprintf (buf, "x_1=%g y_1=%g x_2=%g y_2=%g x_3=%g y_3=%g",
-						x_1, y_1, x_2, y_2, x_3, y_3);
-	  xe_msg (buf,0);
+	/* record and turn around at half speed whenever value shrinks again */
+	if (v < lastv) {
+	    if (*tmincp > 0) {
+		j1 = Mjd;
+		lastv1 = v1;
+		v1 = v;
+	    } else {
+		j0 = Mjd;
+		lastv0 = v0;
+		v0 = v;
+	    }
+	    if (lastv0 == v0 && lastv1 == v1) {
+		xe_msg (1, "Function is too broad to find max so precisely");
+		return (-1);
+	    }
+
+	    *tmincp *= -0.5;
 	}
-#endif
-	a = y_1*(x_2-x_3) - y_2*(x_1-x_3) + y_3*(x_1-x_2);
-	if (fabs(a) < 1e-10) {
-	    /* near-0 zero denominator, ie, curve is pretty flat here,
-	     * so assume we are done enough.
-	     * signal this by forcing a 0 tminc.
-	     */
-	    *tmincp = 0.0;
-	    return (-1);
+
+	lastv = v;
+
+	return (24*(j1 - j0) < tmlimit ? -1 : 0);
+}
+
+/* search for time when expression is at a local minimum.
+ * uses globals srch_tmscalled and tmlimit.
+ * return -1 when finished.
+ */
+static int
+srch_min(Mjd, v, tmincp)
+double Mjd;
+double v;
+double *tmincp;
+{
+	static double v0, lastv0, j0, v1, lastv1, j1;
+	static double lastv;
+
+	/* save first time and keep going */
+	if (srch_tmscalled == 0) {
+	    j0 = -1e100;
+	    j1 =  1e100;
+	    lastv = v;
+	    return (0);
 	}
-	b = (x_1*x_1)*(y_2-y_3) - (x_2*x_2)*(y_1-y_3) + (x_3*x_3)*(y_1-y_2);
-	xm = -b/(2.0*a);
-	*tmincp = (xm - Mjd)*24.0;
-	return (fabs (*tmincp) < tmlimit ? -1 : 0);
+
+	/* record and turn around at half speed whenever value grows again */
+	if (v > lastv) {
+	    if (*tmincp > 0) {
+		j1 = Mjd;
+		lastv1 = v1;
+		v1 = v;
+	    } else {
+		j0 = Mjd;
+		lastv0 = v0;
+		v0 = v;
+	    }
+	    if (lastv0 == v0 && lastv1 == v1) {
+		xe_msg (1, "Function is too broad to find min so precisely");
+		return (-1);
+	    }
+
+	    *tmincp *= -0.5;
+	}
+
+	lastv = v;
+
+	return (24*(j1 - j0) < tmlimit ? -1 : 0);
 }
 
 /* use secant method to solve for time when expression passes through 0.
+ * uses globals srch_tmscalled and tmlimit.
  */
 static int
 srch_solve0(Mjd, v, tmincp)
@@ -955,6 +1053,7 @@ double *tmincp;
 }
 
 /* binary search for time when expression changes from its initial state.
+ * uses globals srch_tmscalled and tmlimit.
  * if the change is outside the initial tminc range, then keep searching in that
  *    direction by tminc first before starting to divide down.
  */
@@ -991,11 +1090,8 @@ double *tmincp;
 		ub = Mjd;
 	    *tmincp = ((lb + ub)/2.0 - Mjd)*24.0;
 #ifdef TRACEBIN
-	    { char buf[NC];
-	      (void) sprintf (buf, "lb=%g ub=%g tminc=%g Mjd=%g is=%d ts=%d",
+	    xe_msg (0, "lb=%g ub=%g tminc=%g Mjd=%g is=%d ts=%d",
 			    lb, ub, *tmincp, Mjd, initial_state, this_state);
-	      xe_msg(buf, 0);
-	    }
 #endif
 	    /* signal to stop if asking for time change less than TMLIMIT */
 	    return (fabs (*tmincp) < tmlimit ? -1 : 0);
@@ -1020,4 +1116,4 @@ double *tmincp;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: srchmenu.c,v $ $Date: 2000/07/20 07:37:43 $ $Revision: 1.7 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: srchmenu.c,v $ $Date: 2008/01/31 16:40:52 $ $Revision: 1.25 $ $Name:  $"};
