@@ -899,7 +899,7 @@ getLatestFN(char fn[])
 	    return (-1);
 
 	/* build http command for page that lists latest fn */
-	sprintf (get, "/data/realtime/realtime-%s-%s.html HTTP/1.0\r\nUser-Agent: xephem/%s\r\n\r\n", stype[t].file, ssize[s].file, PATCHLEVEL);
+	sprintf (get, "GET http://%s/data/realtime/realtime-%s-%s.html HTTP/1.0\r\nUser-Agent: xephem/%s\r\n\r\n", sohohost, stype[t].file, ssize[s].file, PATCHLEVEL);
 
 	/* open */
 	socket = httpGET (sohohost, get, msg);
@@ -960,8 +960,8 @@ readSOHOImage (char fn[])
 	int fd, nr;
 
 	/* build GET command */
-	sprintf (get, "%s HTTP/1.0\r\nUser-Agent: xephem/%s\r\n\r\n", fn,
-								    PATCHLEVEL);
+	sprintf (get, "GET http://%s%s HTTP/1.0\r\nUser-Agent: xephem/%s\r\n\r\n",
+						    sohohost, fn, PATCHLEVEL);
 
 	/* query server */
 	fd = httpGET (sohohost, get, buf);
@@ -1224,5 +1224,5 @@ carrington(Now *np)
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: sunmenu.c,v $ $Date: 2009/01/05 20:55:54 $ $Revision: 1.21 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: sunmenu.c,v $ $Date: 2010/10/06 21:12:16 $ $Revision: 1.22 $ $Name:  $"};
 
