@@ -16,13 +16,11 @@
 #include <unistd.h>
 
 
-#include "P_.h"
-#include "astro.h"
-#include "circum.h"
+#include "xephem.h"
 
 
-extern int handle_request (char *req);
-extern int db_read (FILE *fp, int nodups);
+extern int handle_request (char *req, int nofs);
+extern void db_read (char *fn);
 extern int fs_setup (char catdir[], char msg[]);
 
 static void usage (void);
@@ -31,18 +29,20 @@ static void mklkfile (void);
 static void newstdin (void);
 static void daemonize (void);
 static void find_edb (void);
-static void read_edb (char *name);
 static void go(void);
 static void new_process (char *req);
 static void rmlkfile(void);
 
-static char catdir_def[] = "/usr/local/telescope/archive/catalogs";
+static char catdir_def[] = "/usr/local/xephem/catalogs";
+static char moddir_def[] = "/usr/local/xephem/auxil";
 static char lkfile_def[] = "/tmp/xephemdbd.pid";
 
 static int dflag;
 static int vflag;
+static int fflag;
 static char *pname;
 static char *catdir = catdir_def;
+static char *moddir = moddir_def;
 static char *lkfile = lkfile_def;
 static char *ifile;
 
@@ -72,6 +72,9 @@ main (int ac, char *av[])
 		case 'd':
 		    dflag++;
 		    break;
+		case 'f':
+		    fflag++;
+		    break;
 		case 'i':
 		    if (ac < 2)
 			usage ();
@@ -84,6 +87,12 @@ main (int ac, char *av[])
 		    lkfile = *++av;
 		    ac--;
 		    break;
+		case 'm':
+		    if (ac < 2)
+			usage ();
+		    moddir = *++av;
+		    ac--;
+		    break;
 		case 't':
 		    if (ac < 2)
 			usage ();
@@ -135,8 +144,10 @@ usage()
 	fprintf(fp," -c <dir>    alternate catalogs directory;\n");
 	fprintf(fp,"             default is %s\n", catdir_def);
 	fprintf(fp," -d          do not fork/exit as a daemon process\n");
+	fprintf(fp," -f          do not try to use field star catalogs\n");
 	fprintf(fp," -i <file>   open file R/W for input; default is stdin\n");
 	fprintf(fp," -l <file>   lock file; default is %s\n",lkfile_def);
+	fprintf(fp," -m <file>   dir of moon models; default is %s\n",moddir_def);
 	fprintf(fp," -t <secs>   max idle minutes, 0=forever. default is %d\n",
 								    IDLET_DEF);
 	fprintf(fp," -v          verbose to stderr\n");
@@ -146,7 +157,8 @@ usage()
 	fprintf(fp," outputmode  1:column; 2:topo; 4:apparent; 8:header\n");
 	fprintf(fp," objtypes    15:sol sys; 16:br stars; 224:deep sky; 768:field\n");
 	fprintf(fp," year        time of ephemerides, decimal year\n");
-	fprintf(fp," RA,Dec,FOV  position and size of center, rads\n");
+	fprintf(fp," RA,Dec      position of center, rads\n");
+	fprintf(fp," FOV         field of view, rads\n");
 	fprintf(fp," Mag         limiting magnitude\n");
 	fprintf(fp," [lt,lg,el]  location, if topocentric\n");
 
@@ -236,6 +248,7 @@ static void
 daemonize()
 {
 	int i;
+	long n;
 
 	switch (fork()) {
 	case -1:
@@ -244,7 +257,8 @@ daemonize()
 	    break;
 	case 0:
 	    /* close all, but preserve out/err for messages */
-	    for (i = 3; i < OPEN_MAX; i++)
+	    n = sysconf (_SC_OPEN_MAX);
+	    for (i = 3; i < n; i++)
 		(void) close (i);
 	    (void) setsid();
 	    break;
@@ -263,12 +277,18 @@ find_edb()
 	char buf[1024];
 	DIR *dir;
 
-	if (vflag)
-	    fprintf (stderr, "Checking GSC and PPM\n");
-	if (fs_setup(catdir, buf) < 0) {
-	    fprintf (stderr, "%s\n", buf);
-	    rmlkfile();
-	    exit (1);
+	/* register moon model dir */
+	setMoonDir (moddir);
+
+	/* check for field star catalogs unless disabled */
+	if (!fflag) {
+	    if (vflag)
+		fprintf (stderr, "Checking GSC and PPM\n");
+	    if (fs_setup(catdir, buf) < 0) {
+		fprintf (stderr, "%s\n", buf);
+		rmlkfile();
+		exit (1);
+	    }
 	}
 
 	/* open dir */
@@ -291,7 +311,7 @@ find_edb()
 		(void) sprintf (buf, "%s/%s", catdir, name);
 		if (vflag)
 		    fprintf (stderr, "    %s\n", name);
-		read_edb (buf);
+		db_read (buf);
 	    }
 	}
 
@@ -301,30 +321,6 @@ find_edb()
 	(void) closedir (dir);
 }
 
-/* open and read in a .edb file.
- * exit if any trouble.
- */
-static void
-read_edb (char *name)
-{
-	FILE *fp;
-
-	fp = fopen (name, "r");
-	if (!fp) {
-	    perror (name);
-	    rmlkfile();
-	    exit(1);
-	}
-
-	if (db_read (fp, 1) < 0) {
-	    rmlkfile();
-	    exit (1);	/* reason already printed */
-	}
-
-	/* ok */
-	fclose (fp);
-}
-
 static void
 on_alarm (int signo)
 {
@@ -349,7 +345,7 @@ go()
 	signal (SIGALRM, on_alarm);
 
 	if (vflag)
-	    fprintf (stderr, "%6d: Master daemon\n", getpid());
+	    fprintf (stderr, "%6d: Master daemon pid\n", getpid());
 
 	while (fgets (request, sizeof(request), stdin) != NULL) {
 	    alarm (0);	/* cancel timeout */
@@ -360,7 +356,7 @@ go()
 	    if (request[0] == '>')
 		new_process (request);
 	    else
-		(void) handle_request (request);
+		(void) handle_request (request, fflag);
 	    alarm (60*idlet);	/* arm timeout */
 	}
 }
@@ -396,7 +392,7 @@ new_process (char *req)
 	    setbuf (stdout, NULL);
 
 	    /* handle remainder of string as the request as usual */
-	    (void) handle_request (fnp + 1);
+	    (void) handle_request (fnp + 1, fflag);
 	    exit(0);
 
 	} else if (pid < 0) {
@@ -410,4 +406,4 @@ new_process (char *req)
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: xephemdbd.c,v $ $Date: 1998/04/05 20:58:30 $ $Revision: 1.1 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: xephemdbd.c,v $ $Date: 2003/12/05 06:27:38 $ $Revision: 1.6 $ $Name:  $"};
