@@ -1,19 +1,15 @@
 /* main() for xephem.
- * Copyright (c) 1990-2000 by Elwood Charles Downey
- * Permission is granted to make and distribute copies of this program free of
- * charge, provided the copyright notice and this permission notice are
- * preserved on all copies.  All other rights reserved.  No representation is
- * made about the suitability of this software for any purpose.  It is provided
- * "as is" without express or implied warranty.
+ * Copyright (c) 1990-2007 by Elwood Charles Downey
  */
 
 #include <stdio.h>
 #include <signal.h>
-#if defined(__STDC__)
 #include <stdlib.h>
-#else
-extern char *malloc();
-extern char *getenv();
+#include <string.h>
+#include <sys/wait.h>
+
+#if defined(__NUTC__)
+#include <winnutc.h>
 #endif
 
 #include <X11/Xlib.h>
@@ -48,72 +44,13 @@ extern char *getenv();
 #endif /* XmVersion >= 1002 */
 
 
-#include "P_.h"
-#include "patchlevel.h"
-#include "preferences.h"
+#include "xephem.h"
 
-extern String fallbacks[];
 extern char Version_resource[];
+extern char maincategory[];
 
 #define	NMINCOL	150	/* min colors we want for the main colormap */
 
-extern Colormap checkCM P_((Colormap cm, int nwant));
-extern FILE *fopenh P_((char *name, char *how));
-extern char *getShareDir P_((void));
-extern char *getXRes P_((char *name, char *def));
-extern char *syserrstr P_((void));
-extern char *userResFile P_((void));
-extern int confirm P_((void));
-extern int gif2pm P_((Display *dsp, Colormap cm, unsigned char gif[], int ngif,
-    int *wp, int *hp, Pixmap *pmp, char why[]));
-extern int sr_autosaveon P_((void));
-extern int sr_isUp P_((void));
-extern int sr_refresh P_((void));
-extern int sr_save P_((int talk));
-extern void av_manage P_((void));
-extern void c_manage P_((void));
-extern void db_manage P_((void));
-extern void dm_create_shell P_((void));
-extern void dm_manage P_((void));
-extern void e_manage P_((void));
-extern void fs_manage P_((void));
-extern void get_something P_((Widget w, char *resource, XtArgVal value));
-extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
-extern void jm_manage P_((void));
-extern void lst_manage P_((void));
-extern void m_manage P_((void));
-extern void mars_manage P_((void));
-extern void mm_create P_((Widget mainrc));
-extern void mm_external P_((void));
-extern void mm_go_cb P_((Widget w, XtPointer client, XtPointer call));
-extern void msg_manage P_((void));
-extern void net_create P_((void));
-extern void net_manage P_((void));
-extern void ng_manage P_((void));
-extern void obj_manage P_((void));
-extern void plot_manage P_((void));
-extern void pm_manage P_((void));
-extern void pref_create_pulldown P_((Widget mb_w));
-extern void query P_((Widget tw, char *msg, char *label1, char *label2,
-    char *label3, void (*func1)(void), void (*func2)(void),
-    void (*func3)(void)));
-extern void sah_manage P_((void));
-extern void set_something P_((Widget w, char *resource, XtArgVal value));
-extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void sm_manage P_((void));
-extern void sr_addFallbacks P_((void));
-extern void sr_xmanage P_((void));
-extern void srch_manage P_((void));
-extern void ss_manage P_((void));
-extern void sv_manage P_((void));
-extern void um_manage P_((void));
-extern void version P_((void));
-extern void watch_cursor P_((int want));
-extern void wt_manage P_((void));
-extern void wtip P_((Widget w, char *tip));
-extern void wtip_alldown P_((void));
-extern void xe_msg P_((char *msg, int app_modal));
-
 /* these are used to describe and semi-automate making the main pulldown menus
  */
 typedef struct {
@@ -137,32 +74,31 @@ typedef struct {
     int nbip;		/* number of entries in bip[] */
 } PullDownMenu;
 
-#define	EXAMPLES_N	"Examples"	/* special ButtonInfo->name */
 #define	SEPARATOR_N	"MainSep"	/* special ButtonInfo->name */
 
 
-static void addOurDBs P_((void));
-static void chk_args P_((int argc, char *argv[]));
-static void chk_version P_((void));
-static void set_title P_((void));
-static void make_main_window P_((void));
-static Widget make_pulldown P_((Widget mb_w, PullDownMenu *pdmp));
-static void setup_icon P_((void));
-static void m_activate_cb P_((Widget w, XtPointer client, XtPointer call));
-static void make_examples_pullright P_((Widget pulldown_w, Widget cascade_w));
-static void examples_cb P_((Widget w, XtPointer client, XtPointer call));
-static void x_quit P_((void));
+static void addOurDBs (void);
+static void chk_args (int argc, char *argv[]);
+static void chk_pos (void);
+static void chk_version (void);
+static void set_title (void);
+static void make_main_window (void);
+static Widget make_pulldown (Widget mb_w, PullDownMenu *pdmp);
+static void setup_sigs (void);
+static void m_activate_cb (Widget w, XtPointer client, XtPointer call);
+static void x_quit (void);
+static void initialUps(void);
 
 
 /* client arg to m_activate_cb().
  */
 typedef enum {
-    PROGRESS, QUIT, MSGTXT, NETSETUP, EXTIN,
-    DATA, MOON, EARTH, MARS, JUPMOON, SATMOON, UMOON, SKYVIEW, SOLARSYS,
-    PLOT, LIST, SEARCH, GLANCE, AAVSO, SETIAH,
-    DB, OBJS, CLOSEOBJS, FIELDSTARS, WEBTLE,
-    ABOUT, REFERENCES, INTRO, MAINMENU, FILEHLP, VIEWHLP, TOOLSHLP, OBJHLP,
-	PREFHLP, OPERATION, CONTEXT, DATETIME, NOTES
+    PROGRESS, QUIT, GALLERY, MSGTXT, NETSETUP, EXTIN,
+    SUNV, DATA, MOONV, EARTH, MARSV, JUPMOON, SATMOON, UMOON, SKYVIEW,
+    SOLARSYS, PLOT, LIST, SEARCH, GLANCE, AAVSO, CCONV, OBSLOG,
+    EDB, OBJS, CLOSEOBJS, FAVS, FIELDSTARS, WEBDB, MOVIELOOP,
+    ABOUT, REFERENCES, CONFIGHLP, INTRO, MAINMENU, FILEHLP, VIEWHLP, TOOLSHLP,
+        OBJHLP, PREFHLP, OPERATION, CONTEXTHLP, SRCHHLP, DATETIME, NOTES
 } MBOptions;
 
 Widget toplevel_w;
@@ -171,44 +107,29 @@ Colormap xe_cm;
 XtAppContext xe_app;
 char myclass[] = "XEphem";
 
-#define xephem_width 50
-#define xephem_height 50
+#define xephem_width 32
+#define xephem_height 32
 static unsigned char xephem_bits[] = {
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00,
-   0x00, 0x00, 0xf8, 0xff, 0xff, 0x7f, 0x00, 0x00, 0xc0, 0x07, 0x00, 0x00,
-   0xa0, 0x0f, 0x00, 0x3c, 0x00, 0x00, 0x00, 0x10, 0xf0, 0x00, 0x02, 0x00,
-   0x00, 0x00, 0x0c, 0x00, 0x01, 0x01, 0x00, 0x00, 0x08, 0x0c, 0x00, 0x02,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0xfc, 0xff, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x03,
-   0x00, 0x0f, 0x00, 0x00, 0x00, 0x3c, 0x00, 0x00, 0xf0, 0x0c, 0x00, 0x00,
-   0x03, 0x00, 0x00, 0x00, 0x1f, 0x00, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x1e,
-   0x00, 0x30, 0x00, 0xf8, 0x7f, 0x00, 0x3c, 0x00, 0x08, 0xb0, 0x07, 0x80,
-   0x07, 0x40, 0x00, 0x04, 0x78, 0x00, 0x00, 0x18, 0x80, 0x00, 0x04, 0x78,
-   0x00, 0x00, 0x20, 0x80, 0x00, 0x02, 0x30, 0x00, 0x02, 0x20, 0x00, 0x01,
-   0x02, 0x08, 0x80, 0x0f, 0x40, 0x00, 0x01, 0x82, 0x08, 0x00, 0x07, 0x40,
-   0x00, 0x01, 0x02, 0x08, 0x80, 0x0f, 0x40, 0x00, 0x01, 0x02, 0x10, 0x00,
-   0x02, 0x20, 0x04, 0x01, 0x04, 0x10, 0x00, 0x00, 0x20, 0x80, 0x00, 0x04,
-   0x60, 0x00, 0x00, 0x18, 0x80, 0x00, 0x08, 0x80, 0x07, 0x80, 0x07, 0x40,
-   0x00, 0x30, 0x00, 0xf8, 0x7f, 0x00, 0x30, 0x00, 0xc0, 0x00, 0x00, 0x00,
-   0x00, 0x0c, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x3c,
-   0x00, 0x00, 0xf0, 0x00, 0x00, 0x00, 0xc0, 0x03, 0x00, 0x0f, 0x00, 0x00,
-   0x00, 0x00, 0xfc, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x3c,
-   0x00, 0x00, 0x00, 0x60, 0xf0, 0x00, 0xc0, 0x07, 0x00, 0x00, 0xf0, 0x0f,
-   0x00, 0x00, 0xf8, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x20, 0x00,
-   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-   0x00, 0x00};
+   0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xfe, 0xff, 0xff, 0xbf,
+   0xfd, 0xff, 0xff, 0x7f, 0xfe, 0xff, 0xff, 0xbf, 0xfd, 0xff, 0xff, 0x7f,
+   0xfe, 0xff, 0xff, 0xbf, 0xfd, 0xff, 0xff, 0x7f, 0xfe, 0xff, 0xff, 0xbf,
+   0xfd, 0xff, 0xff, 0x7f, 0x1e, 0x8e, 0x01, 0xbc, 0x3d, 0xc4, 0x03, 0x7c,
+   0x7e, 0xe0, 0xe3, 0xbd, 0x7d, 0xe0, 0xe3, 0x7f, 0xfe, 0xf0, 0x63, 0xbf,
+   0xfd, 0xf0, 0x03, 0x7f, 0xfe, 0xf0, 0x03, 0xbf, 0xfd, 0xf0, 0x63, 0x7f,
+   0x7e, 0xe0, 0xe3, 0xbf, 0x7d, 0xe0, 0xe3, 0x7d, 0x3e, 0xc2, 0x03, 0xbc,
+   0x1d, 0x87, 0x01, 0x7c, 0xfe, 0xff, 0xff, 0xbf, 0xfd, 0xff, 0xff, 0x7f,
+   0xfe, 0xff, 0xff, 0xbf, 0xfd, 0xff, 0xff, 0x7f, 0xfe, 0xff, 0xff, 0xbf,
+   0xfd, 0xff, 0xff, 0x7f, 0xfe, 0xff, 0xff, 0xbf, 0xfd, 0xff, 0xff, 0x7f,
+   0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55};
 
 static XrmOptionDescRec options[] = {
+    {"--help",   ".help",    XrmoptionIsArg, NULL},
+    {"-help",    ".help",    XrmoptionIsArg, NULL},
     {"-install", ".install", XrmoptionSepArg, NULL},
-    {"-prfb", ".prfb", XrmoptionIsArg, NULL},
-    {"-help", ".help", XrmoptionIsArg, NULL},
+    {"-log",     ".log",     XrmoptionIsArg, NULL},
+    {"-nowin",   ".nowin",   XrmoptionIsArg, NULL},
+    {"-prfb",    ".prfb",    XrmoptionIsArg, NULL},
+    {"-resfile", ".resfile", XrmoptionSepArg, NULL},
 };
 
 int
@@ -219,25 +140,54 @@ char *argv[];
 	Arg args[10];
 	int n;
 
+	/* set up signal handling */
+	setup_sigs();
+
+	/* check for -nosplash or nosplash file .. can't wait for resources */
+	splashOpen (&argc, argv, options, XtNumber(options));
+
+#ifdef __APPLE__
+	/* hack around two-level namespace linker problem in libXt wrt libXm.
+	 * first appeared in XFree86 4.2.0 build; should remain harmless in
+	 * subsequent builds even if the problem is fixed.
+	 */
+	{
+	    extern void *topLevelShellClassRec;
+	    extern void *transientShellClassRec;
+	    extern void *vendorShellClassRec;
+	    topLevelShellClassRec  = (void *) &vendorShellClassRec;
+	    transientShellClassRec = (void *) &vendorShellClassRec;
+	}
+#endif
+
+	/* check for alternate env before starting toolkit.
+	 * (don't even ask why this is here)
+	 */
+	newEnv(&argc, argv);
+
 	/* set this before using fallbacks[] */
 	(void)sprintf (Version_resource,"%s.Version: %s", myclass, PATCHLEVEL);
 
 	/* open display and gather standard resources.
 	 * we grab fallbacks[] last
 	 */
+	splashMsg ("Building shell\n");
 	n = 0;
 	XtSetArg (args[n], XmNallowShellResize, True); n++;
 	toplevel_w = XtAppInitialize (&xe_app, myclass, options,
 			    XtNumber(options), &argc, argv, NULL, args, n);
 
 	/* add our resources from non-standard places */
+	splashMsg ("Reading X resources\n");
 	addOurDBs();
 
+	splashMsg ("Checking args\n");
 	chk_args (argc, argv);
 	chk_version();
 	set_title();
 
 	/* load xe_cm and toplevel_w with default or private colormap */
+	splashMsg ("Checking colormap\n");
 	xe_cm = checkCM (DefaultColormap(XtD,DefaultScreen(XtD)), NMINCOL);
 	set_something (toplevel_w, XmNcolormap, (XtArgVal)xe_cm);
 
@@ -253,30 +203,31 @@ char *argv[];
 	XmRepTypeInstallTearOffModelConverter();
 #endif
 
-	/* ignore FPE, though we do have a matherr() handler in misc.c. */
-	(void) signal (SIGFPE, SIG_IGN);
-
-#ifdef SIGPIPE
-	/* we deal with write errors directly -- don't want the signal */
-	(void) signal (SIGPIPE, SIG_IGN);
-#endif
-
 	/* connect up the icon pixmap */
-	setup_icon ();
+	splashMsg ("Building icon\n");
+	setup_icon (toplevel_w);
 
 	/* make the main menu bar and form (other stuff is in mainmenu.c) */
+	splashMsg ("Building main window system\n");
 	make_main_window ();
 
 	/* set up networking, but don't manage it here */
+	splashMsg ("Init networking\n");
 	net_create();
 
-	/* define the data table, but don't manage it.
-	 * we do this up front to avoid the delay on its first use later.
-	 */
-	dm_create_shell();
+	/* start in night mode? */
+	splashMsg ("Checking night mode\n");
+	sr_chknightv();
+
+	/* install then monitor position */
+	chk_pos();
 
 	/* here we go */
+	splashMsg ("Realizing top widget\n");
 	XtRealizeWidget(toplevel_w);
+	splashClose();
+	if (!getXRes ("nowin", NULL))
+	    initialUps();
 	XtAppMainLoop(xe_app);
 
 	printf ("XtAppMainLoop returned :-)\n");
@@ -299,83 +250,63 @@ Cursor c;
 	}
 }
 
-/* merge more resource files into final db; harmless if missing.
- * finally add any fallbacks[] not already in db.
+/* look for -env arg.
+ * if find replace as specified and remove from argv list.
  */
-static void
-addOurDBs()
+void
+newEnv (int *argcp, char *argv[])
 {
-	XrmDatabase dspdb = XtDatabase (XtD);
-	XrmDatabase fbdb = NULL;
-	char **pp, *p;
+	int argc = *argcp;
+	int i;
 
-	/* check in TELHOME */
-	if ((p = getenv("TELHOME")) != NULL) {
-	    char fullp[256];
-	    sprintf (fullp, "%s/archive/config/XEphem", p);
-	    XrmCombineFileDatabase(fullp, &dspdb, True);
+	/* look for any/all -env */
+	for (i = 1; i < argc; i++) {
+	    if (strcmp(argv[i], "-env") == 0) {
+		char *argeq, *newenv;
+		int n;
+
+		/* new value is in next arg */
+		if (i == argc-1) {
+		    printf ("-env requires another arg\n");
+		    abort();
+		}
+		newenv = argv[++i];
+
+		argeq = strchr (newenv, '=');
+		if (!argeq) {
+		    printf ("%s: -env requires X=Y\n", newenv);
+		    abort();
+		}
+
+#if defined(__NUTC__)
+		{
+		    /* convert filenames to internal format.
+		     * N.B. assumes exactly 1 file name
+		     */
+		    char *cp, *cpeq;
+
+		    cp = strcpy (malloc (PATH_MAX+1), newenv);
+		    cpeq = strchr (cp, '=');
+		    (void) _NutPathToNutc (argeq+1, cpeq+1, 1);
+		    newenv = cp;
+		}
+#endif
+		/* install new env */
+		putenv (newenv);
+
+		/* remove from argv */
+		(*argcp) -= 2;
+		for (n = i+1; n < argc; n++)
+		    argv[n-2] = argv[n];
+	    }
 	}
 
-	/* then per-user's so it has max priority */
-	XrmCombineFileDatabase(userResFile(), &dspdb, True);
-
-	/* finally add any fallbacks[] not already known */
-	for (pp = fallbacks; (p = *pp++) != NULL; )
-	    XrmPutLineResource (&fbdb, p);	/* creates fbdb if first */
-	XrmCombineDatabase (fbdb, &dspdb, False);
-
-	/* Combine evidently uses pointers.. we die if we destroy fbdb
-	XrmDestroyDatabase (fbdb);
-	*/
-}
-
-/* ARGSUSED */
-static void
-chk_args (argc, argv)
-int argc;
-char *argv[];
-{
-	if (getXRes ("prfb", NULL)) {
-	    String *fbp = fallbacks;
-	    for (fbp = fallbacks; *fbp != NULL; fbp++)
-		printf ("%s\n", *fbp);
-	    exit (0);
-	}
-
-	if (argc > 1 || getXRes ("help", NULL)) {
-	    fprintf (stderr, "xephem: [options]\n");
-	    fprintf (stderr, "  -install {yes no guess}: whether to install a private colormap\n");
-	    fprintf (stderr, "  -prfb: print the fallback resources then exit\n");
-	    exit (0);
-	}
-}
-
-
-/* insure that resource version matches. */
-static void
-chk_version()
-{
-	char *v = getXRes ("Version", "??");
-
-	if (strcmp (v, PATCHLEVEL)) {
-	    printf ("Version skew: Found %s but should be %s\n", v, PATCHLEVEL);
-	    exit (1);
-	}
-}
-
-static void
-set_title()
-{
-	char title[100];
-
-	(void) sprintf (title, "XEphem %s", PATCHLEVEL);
-	set_something (toplevel_w, XmNtitle, (XtArgVal) title);
+	argv[*argcp] = NULL;
 }
 
 /* connect up logo.gif */
-static void
-make_logo (rc)
-Widget rc;
+void
+make_logo (Widget rc)
 {
 	Display *dsp = XtDisplay(toplevel_w);
 	char fn[1024];
@@ -403,7 +334,7 @@ Widget rc;
 	    return; /* darn again */
 	}
 	if (ngif == sizeof(gif)) {
-	    (void) fprintf (stderr, "%s: Max size = %d\n", fn, sizeof(gif));
+	    (void) fprintf (stderr, "%s: Max size = %d\n", fn,(int)sizeof(gif));
 	    return; /* darn again */
 	}
 
@@ -433,6 +364,133 @@ Widget rc;
 	XtManageChild (l_w);
 }
 
+/* add the xephem icon to shell widget w */
+void
+setup_icon (Widget w)
+{
+	static Pixmap pm;
+
+	if (!pm) {
+	    Display *dsp = XtDisplay (w);
+	    Window win = RootWindow (dsp, DefaultScreen(dsp));
+	    pm = XCreateBitmapFromData (dsp, win, (char *)xephem_bits,
+						xephem_width, xephem_height);
+	}
+
+	set_something (w, XmNiconPixmap, (XtArgVal)pm);
+	set_something (w, XmNiconMask, (XtArgVal)pm);
+}
+
+/* merge more resource files into final db; harmless if missing.
+ * finally add any fallbacks[] not already in db.
+ */
+static void
+addOurDBs()
+{
+	XrmDatabase dspdb = XtDatabase (XtD);
+	XrmDatabase fbdb = NULL;
+	char **pp, *p;
+
+	/* check in TELHOME */
+	if ((p = getenv("TELHOME")) != NULL) {
+	    char fullp[256];
+	    sprintf (fullp, "%s/archive/config/XEphem", p);
+	    XrmCombineFileDatabase(fullp, &dspdb, True);
+	}
+
+	/* then non-standard system-wide */
+	XrmCombineFileDatabase("/etc/XEphem", &dspdb, True);
+
+	/* then per-user's so it has max priority */
+	XrmCombineFileDatabase(userResFile(), &dspdb, True);
+
+	/* finally add any fallbacks[] not already known */
+	for (pp = fallbacks; (p = *pp++) != NULL; )
+	    XrmPutLineResource (&fbdb, p);	/* creates fbdb if first */
+	XrmCombineDatabase (fbdb, &dspdb, False);
+
+	/* Combine evidently uses pointers.. we die if we destroy fbdb
+	XrmDestroyDatabase (fbdb);
+	*/
+}
+
+
+/* ARGSUSED */
+static void
+chk_args (argc, argv)
+int argc;
+char *argv[];
+{
+
+	if (getXRes ("log", NULL)) {
+	    /* trouble output, if any, goes to log flie */
+	    char buf[1024];
+	    sprintf (buf, "%s/exitlog.txt", getPrivateDir());
+	    freopen (buf, "at", stdout);
+	    freopen (buf, "at", stderr);
+	}
+
+	if (getXRes ("prfb", NULL)) {
+	    String *fbp = fallbacks;
+	    for (fbp = fallbacks; *fbp != NULL; fbp++)
+		printf ("%s\n", *fbp);
+	    exit (0);
+	}
+
+	if (argc > 1 || getXRes ("help", NULL)) {
+	    fprintf (stderr, "Usage: xephem [options]\n");
+	    fprintf (stderr, "Purpose: interactive astronomical ephemeris.\n");
+	    fprintf (stderr, "Version: %s Build: %s\n", PATCHLEVEL, PATCHDATE);
+	    fprintf (stderr, "Options:\n");
+	    fprintf (stderr, "  -env N=V  : set alternate env Name=Value\n");
+	    fprintf (stderr, "  -help     : print this message then exit\n");
+	    fprintf (stderr, "  -install {yes|no|guess}: whether to install a private colormap\n");
+	    fprintf (stderr, "  -log      : save exit trouble to private log\n");
+	    fprintf (stderr, "  -nosplash : disable splash screen\n");
+	    fprintf (stderr, "  -nowin    : do not restore windows\n");
+	    fprintf (stderr, "  -prfb     : print all fallback resources then exit\n");
+	    fprintf (stderr, "  -resfile f: load alternate resource file, f\n");
+	    fprintf (stderr, "  -splash   : enable splash screen\n");
+	    exit (0);
+	}
+}
+
+/* support position of main window in our preferences system */
+static void
+chk_pos()
+{
+	Position x = (Position)atoi(getXRes ("x", "100"));	/* XEphem.x */
+	Position y = (Position)atoi(getXRes ("y", "100"));	/* XEphem.y */
+
+	set_something (toplevel_w, XmNx, (XtArgVal)x);
+	set_something (toplevel_w, XmNy, (XtArgVal)y);
+
+	sr_reg (toplevel_w, "XEphem.x", maincategory, 0);
+	sr_reg (toplevel_w, "XEphem.y", maincategory, 0);
+}
+
+
+/* insure that resource version and (sometimes) X server matches. */
+static void
+chk_version()
+{
+	char *v = getXRes ("Version", "??");
+
+	if (strcmp (v, PATCHLEVEL)) {
+	    printf ("Version skew: Found %s but should be %s\n", v, PATCHLEVEL);
+	    abort();
+	}
+}
+
+static void
+set_title()
+{
+	char title[100];
+
+	(void) sprintf (title, "XEphem %s", PATCHLEVEL);
+	set_something (toplevel_w, XmNtitle, (XtArgVal) title);
+}
+
 /* put together the menu bar, the main form, and fill in the form with the
  * initial xephem buttons.
  */
@@ -441,23 +499,29 @@ make_main_window ()
 {
 	static ButtonInfo file_buttons[] = {
 	    {"Display a dialog containing supporting informational messages",
-	        "Messages", "Messages...", 0, 0, 'M', m_activate_cb,
+	        "SysLog", "System log...", 0, 0, 'l', m_activate_cb,
 							    (XtPointer)MSGTXT},
+	    {"Display pictures from the Gallery collection",
+	        "Gallery", "Gallery...", 0, 0, 'G', m_activate_cb,
+							    (XtPointer)GALLERY},
 	    {"Setup networking options", "Net", "Network setup...", 0, 0,
 				    'N', m_activate_cb, (XtPointer)NETSETUP},
 	    {"Set up to run xephem Time and Location from an external file",
-		"ExtIn", "External Time/Loc...", 0, 0, 'E', m_activate_cb,
+		"ExtIn", "External file...", 0, 0, 'E', m_activate_cb,
 							    (XtPointer)EXTIN},
 	    {"Display a simple Progress meter",
 		"Progress", "Progress Meter...", 0, 0, 'P', m_activate_cb,
 							   (XtPointer)PROGRESS},
 	    {NULL, SEPARATOR_N},
+	    {"Run 1 Step forward",
+		"Forward1", "Forward 1 Step", "Ctrl<Key>f", "Ctrl+f", 'F',
+						    mm_go_cb, (XtPointer)1},
+	    {"Run 1 Step backward",
+		"Backward1", "Backward 1 Step", "Ctrl<Key>b", "Ctrl+b", 'B',
+						    mm_go_cb, (XtPointer)-1},
 	    {"Run or stop the main execution loop",
 		"Update", "Update", "Ctrl<Key>u", "Ctrl+u", 'U',
 						    mm_go_cb, (XtPointer)0},
-	    {"Run in reverse or stop the main execution loop",
-		"Reverse", "Update in reverse", "Ctrl<Key>r", "Ctrl+r", 'r',
-						    mm_go_cb, (XtPointer)1},
 	    {NULL, SEPARATOR_N},
 	    {"Exit xephem",
 		"Quit", "Quit...", "Ctrl<Key>d", "Ctrl+d", 'Q', m_activate_cb,
@@ -468,12 +532,14 @@ make_main_window ()
 		"GenData", "Data Table...", 0, 0, 'D', m_activate_cb,
 							    (XtPointer)DATA},
 	    {NULL, SEPARATOR_N},
-	    {"Display an image of Moon and supporting information",
-		"Moon", "Moon...", 0, 0, 'M', m_activate_cb, (XtPointer)MOON},
+	    {"Display an image of the Sun and supporting information",
+		"Sun", "Sun...", 0, 0, 'S', m_activate_cb, (XtPointer)SUNV},
+	    {"Display an image of the Moon and supporting information",
+		"Moon", "Moon...", 0, 0, 'M', m_activate_cb, (XtPointer)MOONV},
 	    {"Display a map of Earth and supporting information",
 		"Earth", "Earth...", 0, 0, 'E', m_activate_cb,(XtPointer)EARTH},
 	    {"Display an image of Mars and supporting information",
-		"Mars", "Mars...", 0, 0, 'r', m_activate_cb, (XtPointer)MARS},
+		"Mars", "Mars...", 0, 0, 'r', m_activate_cb, (XtPointer)MARSV},
 	    {"Display a schematic of Jupiter, GRS, its moons, and other info",
 		"Jupiter", "Jupiter...", 0, 0, 'J', m_activate_cb,
 							(XtPointer)JUPMOON},
@@ -502,39 +568,48 @@ make_main_window ()
 		"Solve", "Solve equation...", 0, 0, 'S', m_activate_cb,
 							    (XtPointer)SEARCH},
 	    {"Find all pairs of close objects in memory",
-		"CloseObjs", "Find close pairs...", 0, 0, 'c',
+		"CloseObjs", "Find close pairs...", 0, 0, 'F',
 					m_activate_cb, (XtPointer)CLOSEOBJS},
 	    {"Show twilight and all basic objects on a 24 hour timeline map",
 		"Glance", "Night at a glance...", 0, 0, 'N',
 					    m_activate_cb, (XtPointer)GLANCE},
+#ifdef AAVSO
 	    {"Fetch light curves from AAVSO",
 		"AAVSO", "AAVSO light curves...", 0, 0, 'A', m_activate_cb,
 							    (XtPointer)AAVSO},
-	    {"Monitor local setiathome client",
-		"SetiAtHome", "Seti@Home...", 0, 0, 'H', m_activate_cb,
-							    (XtPointer)SETIAH},
+#endif
+	    {"Sky coordinates converter",
+		"Coordinates converter", "Coordinates converter...",0,0,'C',
+					    m_activate_cb, (XtPointer)CCONV},
+	    {"Manage Observers log", "ObsLog", "Observers log book...",0,0,'O',
+					    m_activate_cb, (XtPointer)OBSLOG},
+	    {"Movie loop control", "MLoop", "Movie loop...", 0, 0,'M',
+					m_activate_cb, (XtPointer)MOVIELOOP},
 	};
 	static ButtonInfo objs_buttons[] = {
-	    {"Load .edb files into memory, list current memory, or delete",
-		"DataBase", "Load/Delete...", 0, 0, 'D',
-						m_activate_cb, (XtPointer)DB},
-	    {"Search memory for an object, view definitions, assign ObjXYZ",
-		"ObjXYZ", "Search, ObjX,Y,Z...", 0, 0,'S',
-					    m_activate_cb, (XtPointer)OBJS},
+	    {"Load and delete .edb files to and from memory",
+		"Files", "Files...", 0, 0, 'L', m_activate_cb, (XtPointer)EDB},
+	    {"Search and inspect loaded objects", "Index", "Index...", 0,
+					0,'S', m_activate_cb, (XtPointer)OBJS},
+	    {"Manage list of favorites", "Favorites", "Favorites...", 0, 0,'F',
+					    m_activate_cb, (XtPointer)FAVS},
+	    {"Download .edb or .tle files from web",
+		"WebDB", "Download...", 0, 0, 'I',
+					m_activate_cb, (XtPointer)WEBDB},
 	    {"Setup field star options and catalog locations",
-		"FieldStars", "Field stars...", 0, 0, 'F',
+		"FieldStars", "Field stars...", 0, 0, 'C',
 					m_activate_cb, (XtPointer)FIELDSTARS},
-	    {"Get new Earth Satellite TLE orbital elements from the Web",
-		"WebTLE", "Update Earth Satellites...", 0, 0, 'U',
-					m_activate_cb, (XtPointer)WEBTLE},
 	};
 	static ButtonInfo help_buttons[] = {
+	    {"Configure Help system find a browser",
+		"Configure", "Configure help...", 0, 0, 'C', m_activate_cb,
+							(XtPointer)CONFIGHLP},
+	    {"Click here then roam over controls to see help tips",
+		"onContext", "on Context", 0, 0, 'x', m_activate_cb,
+							(XtPointer)CONTEXTHLP},
 	    {"Overall features of xephem",
 		"Introduction", "Introduction...", 0, 0, 'I', m_activate_cb,
 							    (XtPointer)INTRO},
-	    {"Click here then roam over controls to see help tips",
-		"onContext", "on Context", 0, 0, 'x', m_activate_cb,
-							(XtPointer)CONTEXT},
 	    {"How to control xephem's running behavior, including looping",
 		"onOperation", "on Operation...", 0, 0, 'e', m_activate_cb,
 							(XtPointer)OPERATION},
@@ -561,7 +636,6 @@ make_main_window ()
 		"onPrefs", "on Preferences...", 0, 0, 'P', m_activate_cb,
 							(XtPointer)PREFHLP},
 	    {NULL, SEPARATOR_N},
-	    {NULL, EXAMPLES_N, "Examples", 0, 0, 'E'},
 	    {"Credits, references and other kudos.",
 		"onReferences", "Credits...",  0, 0, 'C', m_activate_cb,
 							(XtPointer)REFERENCES},
@@ -601,9 +675,15 @@ make_main_window ()
 							(XtPointer)MAINMENU);
 	XtManageChild (mainrc);
 
+	/* connect actions */
+
+	splashMsg ("Connecting actions\n");
+	mm_connActions ();
+
 	/*	Create MenuBar in mainrc  */
 
 	n = 0;
+	splashMsg ("Building main menu bar\n");
 	mb_w = XmCreateMenuBar (mainrc, "MB", args, n); 
 	XtManageChild (mb_w);
 
@@ -621,6 +701,7 @@ make_main_window ()
 	    XtSetValues (mb_w, args, n);
 
 	/* make a spot for the logo */
+	splashMsg ("Building logo\n");
 	make_logo (mainrc);
 
 	/* add the remainder of the main window */
@@ -654,7 +735,6 @@ PullDownMenu *pdmp;
 
 	for (i = 0; i < pdmp->nbip; i++) {
 	    ButtonInfo *bip = &pdmp->bip[i];
-	    int examples = !strcmp (bip->name, EXAMPLES_N);
 	    int separator = !strcmp (bip->name, SEPARATOR_N);
 
 	    if (separator) {
@@ -677,10 +757,7 @@ PullDownMenu *pdmp;
 		XtSetArg (args[n], XmNlabelString, labstr); n++;
 	    }
 	    XtSetArg (args[n], XmNmnemonic, bip->mne); n++;
-	    if (examples)
-		button = XmCreateCascadeButton (pulldown_w, bip->name, args, n);
-	    else
-		button = XmCreatePushButton (pulldown_w, bip->name, args, n);
+	    button = XmCreatePushButton (pulldown_w, bip->name, args, n);
 	    XtManageChild (button);
 	    if (bip->cb)
 		XtAddCallback (button, XmNactivateCallback, bip->cb,
@@ -690,8 +767,6 @@ PullDownMenu *pdmp;
 	    if (labstr)
 		XmStringFree (labstr);
 
-	    if (examples)
-		make_examples_pullright (pulldown_w, button);
 	    if (bip->tip)
 		wtip (button, bip->tip);
 	}
@@ -717,15 +792,30 @@ PullDownMenu *pdmp;
 	return (cascade);
 }
 
+/* this method of avoiding zombies courtesy Dean Huxley */
 static void
-setup_icon ()
+reapchildren (int signo)
 {
-	Display *dsp = XtDisplay (toplevel_w);
-	Window win = RootWindow (dsp, DefaultScreen(dsp));
-	Pixmap pm = XCreateBitmapFromData (dsp, win, (char *)xephem_bits,
-						xephem_width, xephem_height);
+	while (wait3(NULL, WNOHANG, NULL) > 0)
+	    continue;
+}
 
-	set_something (toplevel_w, XmNiconPixmap, (XtArgVal)pm);
+static void
+setup_sigs()
+{
+	/* ignore FPE, though we do have a matherr() handler in misc.c. */
+	(void) signal (SIGFPE, SIG_IGN);
+
+	/* we deal with write errors directly -- don't want the signal */
+	(void) signal (SIGPIPE, SIG_IGN);
+
+	/* no zombies */
+	/* SIG_IGN doesn't do it on netbsd. better to reap. see
+	 * http://www.opengroup.org/onlinepubs/007908799/xsh/exit.html
+	 * http://www.opengroup.org/onlinepubs/007908799/xsh/sigaction.html
+	 * if you /do/ want to wait, the temporarily block SIGCHLD.
+	 */
+	signal (SIGCHLD, reapchildren);
 }
 
 static void
@@ -747,14 +837,14 @@ hlp_onContext()
 	/* grab pointer and allow tips to work until press any button */
 	if (XGrabPointer (dsp, root, True, ButtonPressMask, GrabModeAsync,
 		    GrabModeSync, None, qc, CurrentTime) != GrabSuccess) {
-	    xe_msg ("Could not grab pointer", 1);
+	    xe_msg (1, "Could not grab pointer");
 	    return;
 	}
 	oldtips = pref_get (PREF_TIPS);
 	pref_set (PREF_TIPS, PREF_TIPSON);
 	do {
 	    if (!XQueryPointer (dsp, root, &rw, &cw, &rx, &ry, &x, &y, &m)){
-		xe_msg ("XQueryPointer error", 1);
+		xe_msg (1, "XQueryPointer error");
 		break;
 	    }
 
@@ -779,6 +869,35 @@ hlp_onContext()
 	pref_set (PREF_TIPS, oldtips);
 }
 
+/* bring up the initial set of desired windows */
+static void
+initialUps()
+{
+	if (atoi(getXRes (dm_viewupres(), "0")))
+	    dm_manage();
+	if (atoi(getXRes (sun_viewupres(), "0")))
+	    sun_manage();
+	if (atoi(getXRes (m_viewupres(), "0")))
+	    m_manage();
+	if (atoi(getXRes (e_viewupres(), "0")))
+	    e_manage();
+	if (atoi(getXRes (mars_viewupres(), "0")))
+	    mars_manage();
+	if (atoi(getXRes (jm_viewupres(), "0")))
+	    jm_manage();
+	if (atoi(getXRes (sm_viewupres(), "0")))
+	    sm_manage();
+	if (atoi(getXRes (um_viewupres(), "0")))
+	    um_manage();
+	if (atoi(getXRes (sv_viewupres(), "0")))
+	    sv_manage();
+	if (atoi(getXRes (ss_viewupres(), "0")))
+	    ss_manage();
+	if (atoi(getXRes (ng_viewupres(), "0")))
+	    ng_manage();
+	if (atoi(getXRes (mm_autortres(), "0")))
+	    mm_startrt();
+}
 
 /* main menubar controls callback.
  * client is one of MBOptions.
@@ -790,7 +909,7 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	int code = (int)client;
+	int code = (long int)client;
 
 	watch_cursor(1);
 
@@ -801,12 +920,14 @@ XtPointer call;
 	case PROGRESS:	pm_manage(); break;
 	case QUIT:	x_quit(); break;
 	case DATA:	dm_manage(); break;
+	case SUNV:	sun_manage(); break;
 	case EARTH:	e_manage(); break;
-	case MOON:	m_manage(); break;
-	case MARS:	mars_manage(); break;
+	case MOONV:	m_manage(); break;
+	case MARSV:	mars_manage(); break;
 	case JUPMOON:	jm_manage(); break;
 	case SATMOON:	sm_manage(); break;
 	case UMOON:	um_manage(); break;
+	case GALLERY:	gal_manage(); break;
 	case SKYVIEW:	sv_manage(); break;
 	case SOLARSYS:	ss_manage(); break;
 	case PLOT:	plot_manage(); break;
@@ -814,103 +935,35 @@ XtPointer call;
 	case SEARCH:	srch_manage(); break;
 	case GLANCE:	ng_manage(); break;
 	case AAVSO:	av_manage(); break;
-	case SETIAH:	sah_manage(); break;
+	case CCONV:	cc_manage(); break;
+	case OBSLOG:	ol_manage(); break;
+	case FAVS:	fav_manage(); break;
 	case OBJS:	obj_manage(); break;
-	case DB:	db_manage(); break;
+	case EDB:	db_manage(); break;
 	case CLOSEOBJS:	c_manage(); break;
 	case FIELDSTARS:fs_manage(); break;
-	case WEBTLE:	wt_manage(); break;
+	case WEBDB:	wdb_manage(); break;
+	case MOVIELOOP:	ml_manage(); break;
 	case ABOUT:	version(); break;
+	case CONFIGHLP:	hlp_config (); break;
 	case REFERENCES:hlp_dialog ("Credits", NULL, 0); break;
 	case INTRO:	hlp_dialog ("Intro", NULL, 0); break;
 	case MAINMENU:	hlp_dialog ("MainMenu", NULL, 0); break;
-	case FILEHLP:	hlp_dialog ("MainMenu -- file", NULL, 0); break;
-	case VIEWHLP:	hlp_dialog ("MainMenu -- view", NULL, 0); break;
-	case TOOLSHLP:	hlp_dialog ("MainMenu -- tools", NULL, 0); break;
-	case OBJHLP:	hlp_dialog ("MainMenu -- objects", NULL, 0); break;
-	case PREFHLP:	hlp_dialog ("MainMenu -- preferences", NULL, 0); break;
+	case FILEHLP:	hlp_dialog ("MainMenu_file", NULL, 0); break;
+	case VIEWHLP:	hlp_dialog ("MainMenu_view", NULL, 0); break;
+	case TOOLSHLP:	hlp_dialog ("MainMenu_tools", NULL, 0); break;
+	case OBJHLP:	hlp_dialog ("MainMenu_objects", NULL, 0); break;
+	case PREFHLP:	hlp_dialog ("MainMenu_preferences", NULL, 0); break;
 	case OPERATION:	hlp_dialog ("Operation", NULL, 0); break;
-	case CONTEXT:	hlp_onContext (); break;
-	case DATETIME:	hlp_dialog ("Date/time", NULL, 0); break;
+	case DATETIME:	hlp_dialog ("Date_time", NULL, 0); break;
 	case NOTES:	hlp_dialog ("Notes", NULL, 0); break;
-	default: 	printf ("Main menu bug: code=%d\n", code); exit(1);
+	case CONTEXTHLP:hlp_onContext (); break;
+	default: 	printf ("Main menu bug: code=%d\n", code); abort();
 	}
 
 	watch_cursor(0);
 }
 
-
-/* build the examples pullright off the help pulldown driven by cascade_w */
-static void
-make_examples_pullright (pulldown_w, cascade_w)
-Widget pulldown_w;
-Widget cascade_w;
-{
-	typedef struct {
-	    char *tip;		/* tip text */
-	    char *label;	/* what goes on the help label */
-	    char *key;		/* string to call hlp_dialog() */
-	} HelpOn;
-	static HelpOn helpon[] = {
-	    {"Explore tonight's sky with the Sky View",
-		"The Sky Tonight",	 "Example - whats up"},
-	    {"Display a solar eclipse path on the Earth map",
-		"Solar eclipse path",	 "Example - eclipse path"},
-	    {"Making trails and using the Hubble GSC field stars",
-		"Sky trail and GSC","Example - sky trail"},
-	    {"Display a FITS file or retrieve a DSS image over the Internet",
-		"Displaying images","Example - images"},
-	    {"Make a plot of local sunrise times over the span of one year",
-		"Sunrise Plot",	 "Example - sun plot"},
-	    {"Solve for a Saturn ring-plane crossing event",
-		"Ring crossing",	 "Example - ring plane"},
-	    {"Display the moon overtaking a star",
-		"Lunar occultation","Example - lunar occultation"},
-	    {"Find next viewing opportunity for ISS and show sky and ground tracks",
-		"International Space Station", "Example - ISS"},
-	};
-	Widget pd_w;
-	Widget w;
-	Arg args[20];
-	int n;
-	int i;
-
-	n = 0;
-	pd_w = XmCreatePulldownMenu (pulldown_w, "ExPD", args, n);
-
-	for (i = 0; i < XtNumber(helpon); i++) {
-	    HelpOn *hop = &helpon[i];
-
-	    n = 0;
-	    w = XmCreatePushButton (pd_w, "ExPB", args, n);
-	    XtAddCallback (w, XmNactivateCallback, examples_cb,
-						    (XtPointer)(hop->key));
-	    set_xmstring (w, XmNlabelString, hop->label);
-	    wtip (w, hop->tip);
-	    XtManageChild (w);
-	}
-
-	n = 0;
-	XtSetArg (args[n], XmNsubMenuId, pd_w);  n++;
-	XtSetValues (cascade_w, args, n);
-	wtip (cascade_w, "Step-by-step examples using several xephem features");
-}
-
-/* called from any of the Help->Example entries.
- * client is a tag for hlp_dialog().
- */
-/* ARGSUSED */
-static void
-examples_cb (w, client, call)
-Widget w;
-XtPointer client;
-XtPointer call;
-{
-	char *tag = (char *)client;
-
-	hlp_dialog (tag, NULL, 0);
-}
-
 /* outta here */
 static void
 goodbye()
@@ -931,7 +984,7 @@ x_quit()
 					"  Yes  ",  "  No  ", 0,
 				        goodbye,    0,        0);
 	    } else  {
-		query (toplevel_w, "Review modified resources before exiting?",
+		query (toplevel_w, "Review unsaved Major\nresources before exiting?",
 					"  Yes  ",  "  No  ", 0,
 					sr_xmanage, goodbye,  0);
 	    }
@@ -943,4 +996,4 @@ x_quit()
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: xephem.c,v $ $Date: 2000/12/03 08:33:51 $ $Revision: 1.42 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: xephem.c,v $ $Date: 2009/01/05 20:55:54 $ $Revision: 1.111 $ $Name:  $"};
