@@ -22,14 +22,9 @@
 #include <stdio.h>
 #include <math.h>
 #include <string.h>
-
-#if defined(__STDC__)
 #include <stdlib.h>
-#endif
 
-#include "P_.h"
 #include "astro.h"
-#include "circum.h"
 #include "preferences.h"
 
 #include "vector.h"
@@ -41,38 +36,39 @@
 
 typedef double MAT3x3[3][3];
 
-static void esat_prop P_((Now *np, Obj *op, double *SatX, double *SatY, double
-    *SatZ, double *SatVX, double *SatVY, double *SatVZ));
-static void GetSatelliteParams P_((Obj *op));
-static void GetSiteParams P_((Now *np));
-static double Kepler P_((double MeanAnomaly, double Eccentricity));
-static void GetSubSatPoint P_((double SatX, double SatY, double SatZ,
-    double T, double *Latitude, double *Longitude, double *Height));
-static void GetSatPosition P_((double EpochTime, double EpochRAAN,
+static int crazyOp (Now *np, Obj *op);
+static void esat_prop (Now *np, Obj *op, double *SatX, double *SatY, double
+    *SatZ, double *SatVX, double *SatVY, double *SatVZ);
+static void GetSatelliteParams (Obj *op);
+static void GetSiteParams (Now *np);
+static double Kepler (double MeanAnomaly, double Eccentricity);
+static void GetSubSatPoint (double SatX, double SatY, double SatZ,
+    double T, double *Latitude, double *Longitude, double *Height);
+static void GetSatPosition (double EpochTime, double EpochRAAN,
     double EpochArgPerigee, double SemiMajorAxis, double Inclination,
     double Eccentricity, double RAANPrecession, double PerigeePrecession,
     double T, double TrueAnomaly, double *X, double *Y, double *Z,
-    double *Radius, double *VX, double *VY, double *VZ));
-static void GetSitPosition P_((double SiteLat, double SiteLong,
+    double *Radius, double *VX, double *VY, double *VZ);
+static void GetSitPosition (double SiteLat, double SiteLong,
     double SiteElevation, double CrntTime, double *SiteX, double *SiteY,
-    double *SiteZ, double *SiteVX, double *SiteVY, MAT3x3 SiteMatrix));
-static void GetRange P_((double SiteX, double SiteY, double SiteZ,
+    double *SiteZ, double *SiteVX, double *SiteVY, MAT3x3 SiteMatrix);
+static void GetRange (double SiteX, double SiteY, double SiteZ,
     double SiteVX, double SiteVY, double SatX, double SatY, double SatZ,
     double SatVX, double SatVY, double SatVZ, double *Range,
-    double *RangeRate));
-static void GetTopocentric P_((double SatX, double SatY, double SatZ,
+    double *RangeRate);
+static void GetTopocentric (double SatX, double SatY, double SatZ,
     double SiteX, double SiteY, double SiteZ, MAT3x3 SiteMatrix, double *X,
-    double *Y, double *Z));
-static void GetBearings P_((double SatX, double SatY, double SatZ,
+    double *Y, double *Z);
+static void GetBearings (double SatX, double SatY, double SatZ,
     double SiteX, double SiteY, double SiteZ, MAT3x3 SiteMatrix,
-    double *Azimuth, double *Elevation));
-static int Eclipsed P_((double SatX, double SatY, double SatZ,
-    double SatRadius, double CrntTime));
-static void InitOrbitRoutines P_((double EpochDay, int AtEod));
+    double *Azimuth, double *Elevation);
+static int Eclipsed (double SatX, double SatY, double SatZ,
+    double SatRadius, double CrntTime);
+static void InitOrbitRoutines (double EpochDay, int AtEod);
 
 #ifdef USE_ORBIT_PROPAGATOR 
-static void GetPrecession P_((double SemiMajorAxis, double Eccentricity,
-    double Inclination, double *RAANPrecession, double *PerigeePrecession));
+static void GetPrecession (double SemiMajorAxis, double Eccentricity,
+    double Inclination, double *RAANPrecession, double *PerigeePrecession);
 #endif /* USE_ORBIT_PROPAGATOR */
 
 /* stuff from orbit */
@@ -134,9 +130,7 @@ static double SinPenumbra,CosPenumbra;
  * return 0 if all ok, else -1.
  */
 int
-obj_earthsat (np, op)
-Now *np;
-Obj *op;
+obj_earthsat (Now *np, Obj *op)
 {
 	double Radius;              /* From geocenter                  */
 	double SatX,SatY,SatZ;	    /* In Right Ascension based system */
@@ -242,7 +236,7 @@ Obj *op;
 	/* find s_ra/dec, depending on current options. */
 	if (pref_get(PREF_EQUATORIAL) == PREF_TOPO) {
 	    double ha, lst;
-	    aa_hadec (lat, (double)op->s_alt, (double)op->s_az, &ha, &dec);
+	    aa_hadec (lat, Elevation, (double)op->s_az, &ha, &dec);
 	    now_lst (np, &lst);
 	    ra = hrrad(lst) - ha;
 	    range (&ra, 2*PI);
@@ -266,11 +260,8 @@ Obj *op;
  * set USE_ORBIT_PROPAGATOR depending on desired propagator to use.
  */
 static void
-esat_prop (np, op, SatX, SatY, SatZ, SatVX, SatVY, SatVZ)
-Now *np;
-Obj *op;
-double *SatX,*SatY,*SatZ;
-double *SatVX,*SatVY,*SatVZ;
+esat_prop (Now *np, Obj *op, double *SatX, double *SatY, double *SatZ,
+double *SatVX, double *SatVY, double *SatVZ)
 {
 #ifdef USE_ORBIT_PROPAGATOR
 	double ReferenceOrbit;      /* Floating point orbit # at epoch */
@@ -284,6 +275,11 @@ double *SatVX,*SatVY,*SatVZ;
 	double Radius;
 	double CrntTime;
 
+	if (crazyOp (np, op)) {
+	    *SatX = *SatY = *SatZ = *SatVX = *SatVY = *SatVZ = 0;
+	    return;
+	}
+
 	SemiMajorAxis = 331.25 * exp(2*log(MinutesPerDay/epochMeanMotion)/3);
 	GetPrecession(SemiMajorAxis,Eccentricity,Inclination,&RAANPrecession,
 			    &PerigeePrecession);
@@ -331,6 +327,11 @@ double *SatVX,*SatVY,*SatVZ;
 	double dt;
 	int yr;
 
+	if (crazyOp (np, op)) {
+	    *SatX = *SatY = *SatZ = *SatVX = *SatVY = *SatVZ = 0;
+	    return;
+	}
+
 	/* init */
 	memset ((void *)&se, 0, sizeof(se));
 	memset ((void *)&sd, 0, sizeof(sd));
@@ -391,12 +392,18 @@ double *SatVX,*SatVY,*SatVZ;
 #endif
 }
 
+/* return 1 if op is crazy @ np */
+static int
+crazyOp (Now *np, Obj *op)
+{
+	/* toss if more than a year old */
+	return (fabs(op->es_epoch - mjd) > 365);
+}
 
 /* grab the xephem stuff from op and copy into orbit's globals.
  */
 static void
-GetSatelliteParams(op)
-Obj *op;
+GetSatelliteParams(Obj *op)
 {
 	/* the following are for the orbit functions */
 	/* xephem uses noon 12/31/1899 as 0; orbit uses midnight 1/1/1900 as 1.
@@ -428,8 +435,7 @@ Obj *op;
 
  
 static void
-GetSiteParams(np)
-Now *np;
+GetSiteParams(Now *np)
 {
 	SiteLat = lat;
      
@@ -464,9 +470,7 @@ Now *np;
 /*                      current position.                       */
  
 static
-double Kepler(MeanAnomaly,Eccentricity)
-register double MeanAnomaly,Eccentricity;
- 
+double Kepler(double MeanAnomaly, double Eccentricity)
 {
 register double E;              /* Eccentric Anomaly                    */
 register double Error;
@@ -493,9 +497,8 @@ register double TrueAnomaly;
 }
  
 static void
-GetSubSatPoint(SatX,SatY,SatZ,T,Latitude,Longitude,Height)
-double SatX,SatY,SatZ,T;
-double *Latitude,*Longitude,*Height;
+GetSubSatPoint(double SatX, double SatY, double SatZ, double T,
+double *Latitude, double *Longitude, double *Height)
 {
     double r;
     /* ECD: long i; */
@@ -523,10 +526,8 @@ double *Latitude,*Longitude,*Height;
  
 #ifdef USE_ORBIT_PROPAGATOR
 static void
-GetPrecession(SemiMajorAxis,Eccentricity,Inclination,
-        RAANPrecession,PerigeePrecession)
-double SemiMajorAxis,Eccentricity,Inclination;
-double *RAANPrecession,*PerigeePrecession;
+GetPrecession(double SemiMajorAxis, double Eccentricity, double Inclination,
+double *RAANPrecession, double *PerigeePrecession)
 {
   *RAANPrecession = 9.95*pow(EarthRadius/SemiMajorAxis,3.5) * cos(Inclination)
                  / SQR(1-SQR(Eccentricity)) * RadiansPerDegree;
@@ -543,14 +544,11 @@ double *RAANPrecession,*PerigeePrecession;
  */
 
 static void
-GetSatPosition(EpochTime,EpochRAAN,EpochArgPerigee,SemiMajorAxis,
-	Inclination,Eccentricity,RAANPrecession,PerigeePrecession,
-	T,TrueAnomaly,X,Y,Z,Radius,VX,VY,VZ)
- 
-double EpochTime,EpochRAAN,EpochArgPerigee;
-double SemiMajorAxis,Inclination,Eccentricity;
-double RAANPrecession,PerigeePrecession,T, TrueAnomaly;
-double *X,*Y,*Z,*Radius,*VX,*VY,*VZ;
+GetSatPosition(double EpochTime, double EpochRAAN, double EpochArgPerigee,
+double SemiMajorAxis, double Inclination, double Eccentricity,
+double RAANPrecession, double PerigeePrecession, double T,
+double TrueAnomaly, double *X, double *Y, double *Z, double *Radius,
+double *VX, double *VY, double *VZ)
 
 {
     double RAAN,ArgPerigee;
@@ -606,13 +604,9 @@ double *X,*Y,*Z,*Radius,*VX,*VY,*VZ;
     coordinates. */
 
 static void
-GetSitPosition(SiteLat,SiteLong,SiteElevation,CrntTime,
-             SiteX,SiteY,SiteZ,SiteVX,SiteVY,SiteMatrix)
-
-double SiteLat,SiteLong,SiteElevation,CrntTime;
-double *SiteX,*SiteY,*SiteZ,*SiteVX,*SiteVY;
-MAT3x3 SiteMatrix;
-
+GetSitPosition(double SiteLat, double SiteLong, double SiteElevation,
+double CrntTime, double *SiteX, double *SiteY, double *SiteZ, double *SiteVX,
+double *SiteVY, MAT3x3 SiteMatrix)
 {
     static double G1,G2; /* Used to correct for flattening of the Earth */
     static double CosLat,SinLat;
@@ -662,12 +656,9 @@ MAT3x3 SiteMatrix;
 }
 
 static void
-GetRange(SiteX,SiteY,SiteZ,SiteVX,SiteVY,
-	SatX,SatY,SatZ,SatVX,SatVY,SatVZ,Range,RangeRate)
-
-double SiteX,SiteY,SiteZ,SiteVX,SiteVY;
-double SatX,SatY,SatZ,SatVX,SatVY,SatVZ;
-double *Range,*RangeRate;
+GetRange(double SiteX, double SiteY, double SiteZ, double SiteVX,
+double SiteVY, double SatX, double SatY, double SatZ, double SatVX,
+double SatVY, double SatVZ, double *Range, double *RangeRate)
 {
     double DX,DY,DZ;
 
@@ -683,10 +674,9 @@ double *Range,*RangeRate;
    (observer centered) coordinates */
 
 static void
-GetTopocentric(SatX,SatY,SatZ,SiteX,SiteY,SiteZ,SiteMatrix,X,Y,Z)
-double SatX,SatY,SatZ,SiteX,SiteY,SiteZ;
-double *X,*Y,*Z;
-MAT3x3 SiteMatrix;
+GetTopocentric(double SatX, double SatY, double SatZ, double SiteX,
+double SiteY, double SiteZ, MAT3x3 SiteMatrix, double *X, double *Y,
+double *Z)
 {
     SatX -= SiteX;
     SatY -= SiteY;
@@ -701,10 +691,9 @@ MAT3x3 SiteMatrix;
 }
 
 static void
-GetBearings(SatX,SatY,SatZ,SiteX,SiteY,SiteZ,SiteMatrix,Azimuth,Elevation)
-double SatX,SatY,SatZ,SiteX,SiteY,SiteZ;
-MAT3x3 SiteMatrix;
-double *Azimuth,*Elevation;
+GetBearings(double SatX, double SatY, double SatZ, double SiteX,
+double SiteY, double SiteZ, MAT3x3 SiteMatrix, double *Azimuth,
+double *Elevation)
 {
     double x,y,z;
 
@@ -719,8 +708,8 @@ double *Azimuth,*Elevation;
 }
 
 static int
-Eclipsed(SatX,SatY,SatZ,SatRadius,CrntTime)
-double SatX,SatY,SatZ,SatRadius,CrntTime;
+Eclipsed(double SatX, double SatY, double SatZ, double SatRadius,
+double CrntTime)
 {
     double MeanAnomaly,TrueAnomaly;
     double SunX,SunY,SunZ,SunRad;
@@ -750,9 +739,7 @@ double SatX,SatY,SatZ,SatRadius,CrntTime;
    Also init the sidereal reference				*/
 
 static void
-InitOrbitRoutines(EpochDay, AtEod)
-double EpochDay;
-int AtEod;
+InitOrbitRoutines(double EpochDay, int AtEod)
 {
     double T,T2,T3,Omega;
     int n;
@@ -796,4 +783,4 @@ int AtEod;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: earthsat.c,v $ $Date: 2000/10/07 05:12:17 $ $Revision: 1.5 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: earthsat.c,v $ $Date: 2009/07/03 23:14:02 $ $Revision: 1.12 $ $Name:  $"};
