@@ -2,22 +2,18 @@
 
 #include <stdio.h>
 #include <math.h>
-#if defined(__STDC__)
 #include <stdlib.h>
 #include <string.h>
-#endif
 
-#include "P_.h"
 #include "astro.h"
-#include "circum.h"
 
 #define	TMACC	(10./3600./24.0)	/* convergence accuracy, days */
 
-static void e_riset_cir P_((Now *np, Obj *op, double dis, RiseSet *rp));
-static int find_0alt P_((double dt, double dis, Now *np, Obj *op));
-static int find_transit P_((double dt, Now *np, Obj *op));
-static int find_max P_((Now *np, Obj *op, double tr, double ts, double *tp,
-    double *alp));
+static void e_riset_cir (Now *np, Obj *op, double dis, RiseSet *rp);
+static int find_0alt (double dt, double dis, Now *np, Obj *op);
+static int find_transit (double dt, Now *np, Obj *op);
+static int find_max (Now *np, Obj *op, double tr, double ts, double *tp,
+    double *alp);
 
 /* find where and when an object, op, will rise and set and
  *   it's transit circumstances. all times are utc mjd, angles rads e of n.
@@ -25,11 +21,7 @@ static int find_max P_((Now *np, Obj *op, double tr, double ts, double *tp,
  * N.B. dis should NOT include refraction, we do that here.
  */
 void
-riset_cir (np, op, dis, rp)
-Now *np;
-Obj *op;
-double dis;
-RiseSet *rp;
+riset_cir (Now *np, Obj *op, double dis, RiseSet *rp)
 {
 	double mjdn;	/* mjd of local noon */
 	double lstn;	/* lst at local noon */
@@ -63,8 +55,8 @@ RiseSet *rp;
 
 	/* first approximation is to find rise/set times of a fixed object
 	 * at the current epoch in its position at local noon.
-	 * N.B. add typical refraction for initial go/no-go test. if it
-	 *   passes, real code does refraction rigorously.
+	 * N.B. add typical refraction if dis is above horizon for initial
+	 *   go/no-go test. if it passes, real code does refraction rigorously.
 	 */
 	n.n_mjd = mjdn;
 	if (obj_cir (&n, &o) < 0) {
@@ -72,7 +64,8 @@ RiseSet *rp;
 	    return;
 	}
 	ran = o.s_gaera;
-	riset (o.s_gaera, o.s_gaedec, lat, dis+.01, &lr, &ls, &ar, &as, &rss);
+	riset (o.s_gaera, o.s_gaedec, lat, dis+(dis>.01 ? 0 : .01), &lr, &ls,
+								&ar, &as, &rss);
 	switch (rss) {
 	case  0:  break;
 	case  1: rp->rs_flags = RS_NEVERUP; return;
@@ -90,10 +83,14 @@ RiseSet *rp;
 	case -1: /* obj_cir error */
 	    rp->rs_flags |= RS_RISERR;
 	    break;
-	case -2: /* converged but not today */ /* FALLTHRU */
-	case -3: /* probably never up */
+	case -2: /* converged but not today, err but give times anyway */
+	    rp->rs_risetm = n.n_mjd;
+	    rp->rs_riseaz = o.s_az;
 	    rp->rs_flags |= RS_NORISE;
 	    break;
+	case -3: /* probably never up */
+	    rp->rs_flags |= RS_NEVERUP;
+	    break;
 	}
 
 	/* iterate to find better set time */
@@ -106,10 +103,14 @@ RiseSet *rp;
 	case -1: /* obj_cir error */
 	    rp->rs_flags |= RS_SETERR;
 	    break;
-	case -2: /* converged but not today */ /* FALLTHRU */
-	case -3: /* probably circumpolar */
+	case -2: /* converged but not today, err but give times anyway */
+	    rp->rs_settm = n.n_mjd;
+	    rp->rs_setaz = o.s_az;
 	    rp->rs_flags |= RS_NOSET;
 	    break;
+	case -3: /* probably circumpolar */
+	    rp->rs_flags |= RS_CIRCUMPOLAR;
+	    break;
 	}
 
 	/* can try transit even if rise or set failed */
@@ -132,17 +133,14 @@ RiseSet *rp;
 /* find local times when sun is dis rads below horizon.
  */
 void
-twilight_cir (np, dis, dawn, dusk, status)
-Now *np;
-double dis;
-double *dawn, *dusk;
-int *status;
+twilight_cir (Now *np, double dis, double *dawn, double *dusk, int *status)
 {
 	RiseSet rs;
 	Obj o;
 
+	memset (&o, 0, sizeof(o));
 	o.o_type = PLANET;
-	o.pl.pl_code = SUN;
+	o.pl_code = SUN;
 	(void) strcpy (o.o_name, "Sun");
 	riset_cir (np, &o, dis, &rs);
 	*dawn = rs.rs_risetm;
@@ -159,11 +157,7 @@ int *status;
  * N.B. we assume *np and *op are working copies we can mess up.
  */
 static void
-e_riset_cir (np, op, dis, rp)
-Now *np;
-Obj *op;
-double dis;
-RiseSet *rp;
+e_riset_cir (Now *np, Obj *op, double dis, RiseSet *rp)
 {
 #define	DEGSTEP	5		/* time step is about this many degrees */
 	int steps;		/* max number of time steps */
@@ -233,10 +227,9 @@ RiseSet *rp;
 	}
 
 	/* instead of transit, for satellites we find time of maximum
-	 * altitude, if we know both the rise and set times and the former
-	 * occurs before the latter.
+	 * altitude, if we know both the rise and set times.
 	 */
-	if (rise && set && rp->rs_risetm < rp->rs_settm) {
+	if (rise && set) {
 	    double tt, al;
 	    if (find_max (np, op, rp->rs_risetm, rp->rs_settm, &tt, &al) < 0) {
 		rp->rs_flags |= RS_TRANSERR;
@@ -271,24 +264,27 @@ RiseSet *rp;
  * return -3: if does not converge at all (probably circumpolar or never up);
  */
 static int
-find_0alt (dt, dis, np, op)
-double dt;	/* hours from noon to first guess at event */
-double dis;	/* horizon displacement, rads */
-Now *np;	/* working Now -- starts with mjd is noon, returns as answer */
-Obj *op;	/* working object -- returns as answer */
+find_0alt (
+double dt,	/* hours from noon to first guess at event */
+double dis,	/* horizon displacement, rads */
+Now *np,	/* working Now -- starts with mjd is noon, returns as answer */
+Obj *op)	/* working object -- returns as answer */
 {
 #define	MAXPASSES	20		/* max iterations to try */
 #define	FIRSTSTEP	(1.0/60.0/24.0)	/* first time step, days */
+#define	MAXSTEP		(12.0/24.0)/* max time step,days (to detect flat)*/
 
 	double a0 = 0;
 	double mjdn = mjd;
 	int npasses;
 
 	/* insure initial guess is today -- if not, move by 24 hours */
-	if (dt < -12.0)
-	    dt += 24.0;
-	if (dt > 12.0)
-	    dt -= 24.0;
+	if (dt < -12.0 && !find_0alt (dt+24, dis, np, op))
+	    return (0);
+	mjd = mjdn;
+	if (dt > 12.0 && !find_0alt (dt-24, dis, np, op))
+	    return (0);
+	mjd = mjdn;
 	
 	/* convert dt to days for remainder of algorithm */
 	dt /= 24.0;
@@ -306,15 +302,17 @@ Obj *op;	/* working object -- returns as answer */
 	    dt = (npasses == 0) ? FIRSTSTEP : (dis+a1)*dt/(a0-a1);
 	    a0 = a1;
 
-	} while (++npasses < MAXPASSES && fabs(dt) > TMACC);
+	    if (++npasses > MAXPASSES || fabs(dt) >= MAXSTEP)
+		return (-3);
+
+	} while (fabs(dt)>TMACC);
 
 	/* return codes */
-	if (npasses == MAXPASSES)
-	    return (-3);
 	return (fabs(mjdn-mjd) < .5 ? 0 : -2);
 
 #undef	MAXPASSES
 #undef	FIRSTSTEP
+#undef	MAXSTEP
 }
 
 /* find when the given object transits. start the search when LST matches the
@@ -324,10 +322,7 @@ Obj *op;	/* working object -- returns as answer */
  * N.B. we assume np is passed set to local noon.
  */
 static int
-find_transit (dt, np, op)
-double dt;
-Now *np;
-Obj *op;
+find_transit (double dt, Now *np, Obj *op)
 {
 #define	MAXLOOPS	10
 #define	MAXERR		(0.25/60.)		/* hours */
@@ -369,12 +364,15 @@ Obj *op;
  * return 0 if ok, else -1.
  */
 static int
-find_max (np, op, tr, ts, tp, alp)
-Now *np;
-Obj *op;
-double tr, ts;		/* times of rise and set */
-double *tp, *alp;	/* time of max altitude, and that altitude */
+find_max (
+Now *np,
+Obj *op,
+double tr, double ts,		/* times of rise and set */
+double *tp, double *alp)	/* time of max altitude, and that altitude */
 {
+	/* want rise before set */
+	while (ts < tr)
+	    tr -= 1.0/op->es_n;
 	mjd = (ts + tr)/2;
 	if (obj_cir (np, op) < 0)
 	    return (-1);
@@ -384,4 +382,4 @@ double *tp, *alp;	/* time of max altitude, and that altitude */
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: riset_cir.c,v $ $Date: 1998/12/22 06:58:30 $ $Revision: 1.3 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: riset_cir.c,v $ $Date: 2009/04/06 00:17:41 $ $Revision: 1.13 $ $Name:  $"};
